@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,6 +30,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Definition of program-specific options. */
 struct argp_option program_options[] =
   {
+    /* Input options. */
     {
       "globalhdu",
       UI_KEY_GLOBALHDU,
@@ -82,11 +83,25 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "arguments",
+      UI_KEY_ARGUMENTS,
+      "STR",
+      0,
+      "Plain-text file with command-line arguments.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->arguments,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
 
 
 
 
+    /* Output options. */
     {
       "onedasimage",
       UI_KEY_ONEDASIMAGE,
@@ -113,6 +128,58 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "metaname",
+      UI_KEY_METANAME,
+      "STR",
+      0,
+      "Internal name (FITS images: EXTNAME keyword).",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->metaname,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "metaunit",
+      UI_KEY_METAUNIT,
+      "STR",
+      0,
+      "Internal units (FITS images: BUNIT keyword).",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->metaunit,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "metacomment",
+      UI_KEY_METACOMMENT,
+      "STR",
+      0,
+      "Internal comments (FITS images: COMMENT keyword).",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->metacomment,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "writeall",
+      UI_KEY_WRITEALL,
+      0,
+      0,
+      "Write all remaining data in the output.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->writeall,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
     {0}
   };
