@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -69,8 +69,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
     if(a>0) return a;    }
 
 static size_t
-pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
-                       gal_data_t **params)
+pop_number_of_operands(struct arithmeticparams *p, int op,
+                       char *token_string, gal_data_t **params)
 {
   char *cstring="first";
   size_t c, numparams=0;
@@ -83,10 +83,26 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
     case GAL_ARITHMETIC_OP_QUANTILE:
       numparams=1;
       break;
+    case GAL_ARITHMETIC_OP_MADCLIP_STD:
     case GAL_ARITHMETIC_OP_SIGCLIP_STD:
+    case GAL_ARITHMETIC_OP_MADCLIP_MAD:
+    case GAL_ARITHMETIC_OP_SIGCLIP_MAD:
+    case GAL_ARITHMETIC_OP_MADCLIP_MEAN:
     case GAL_ARITHMETIC_OP_SIGCLIP_MEAN:
+    case GAL_ARITHMETIC_OP_MADCLIP_MEDIAN:
     case GAL_ARITHMETIC_OP_SIGCLIP_MEDIAN:
+    case GAL_ARITHMETIC_OP_MADCLIP_NUMBER:
     case GAL_ARITHMETIC_OP_SIGCLIP_NUMBER:
+    case GAL_ARITHMETIC_OP_MADCLIP_FILL_MAD:
+    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_MAD:
+    case GAL_ARITHMETIC_OP_MADCLIP_FILL_STD:
+    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_STD:
+    case GAL_ARITHMETIC_OP_MADCLIP_FILL_MEAN:
+    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_MEAN:
+    case GAL_ARITHMETIC_OP_MADCLIP_FILL_MEDIAN:
+    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_MEDIAN:
+    case GAL_ARITHMETIC_OP_MADCLIP_FILL_NUMBER:
+    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_NUMBER:
       numparams=2;
       break;
     }
@@ -108,7 +124,7 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
          or quantile estimation can be negative). */
       if( ((float *)(tmp->array))[0]<=0.0 )
         error(EXIT_FAILURE, 0, "the %s popped operand of the '%s' "
-              "operator cannot be negative", cstring, token_string);
+              "operator cannot be zero or negative", cstring, token_string);
 
       /* Increment the counter string. */
       cstring=c?"third":"second";
@@ -205,6 +221,7 @@ arithmetic_filter(void *in_prm)
   gal_data_t *input=afp->input;
 
   size_t sind=-1;
+  int clipflags=0;
   size_t ind, index, one=1;
   gal_data_t *sigclip, *result=NULL;
   size_t *hpfsize=afp->hpfsize, *hnfsize=afp->hnfsize;
@@ -282,15 +299,21 @@ arithmetic_filter(void *in_prm)
 
         case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:
         case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN:
-          /* Find the sigma-clipped results. */
-          sigclip=gal_statistics_sigma_clip(tile, afp->sclip_multip,
-                                            afp->sclip_param, 0, 1);
+          /* The median is always available with a sigma-clip, but the mean
+             needs to be explicitly requested. */
+          if(afp->operator == ARITHMETIC_OP_FILTER_SIGCLIP_MEAN)
+            clipflags = GAL_STATISTICS_CLIP_OUTCOL_OPTIONAL_MEAN;
+          sigclip=gal_statistics_clip_sigma(tile, afp->sclip_multip,
+                                            afp->sclip_param, clipflags,
+                                            0, 1);
 
           /* Set the required index. */
           switch(afp->operator)
             {
-            case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:   sind = 2; break;
-            case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN: sind = 1; break;
+            case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:
+              sind = GAL_STATISTICS_CLIP_OUTCOL_MEAN; break;
+            case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN:
+              sind = GAL_STATISTICS_CLIP_OUTCOL_MEDIAN; break;
             default:
               error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
                     "%s to fix the problem. The 'afp->operator' value "
@@ -917,8 +940,8 @@ arithmetic_interpolate(struct arithmeticparams *p, int operator, char *token)
       interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MEDIAN;
       break;
     default:
-      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
-            "the problem. The operator code %d isn't recognized",
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+            "fix the problem. The operator code %d isn't recognized",
             __func__, PACKAGE_BUGREPORT, operator);
     }
 
@@ -943,15 +966,16 @@ arithmetic_collapse_single_value(gal_data_t *input, char *counter,
                                  int checkint)
 {
   if( input->ndim!=1 || input->size!=1)
-    error(EXIT_FAILURE, 0, "%s popped operand of 'collapse-%s*' operators "
-          "(%s) must be a single number (single-element, "
-          "one-dimensional dataset). But it has %zu dimension(s) and %zu "
-          "element(s).", counter, opstr, description, input->ndim, input->size);
+    error(EXIT_FAILURE, 0, "%s popped operand of 'collapse-%s*' "
+          "operators (%s) must be a single number (single-element, "
+          "one-dimensional dataset). But it has %zu dimension(s) and "
+          "%zu element(s).", counter, opstr, description, input->ndim,
+          input->size);
   if(checkint)
     if(input->type==GAL_TYPE_FLOAT32 || input->type==GAL_TYPE_FLOAT64)
-      error(EXIT_FAILURE, 0, "%s popped operand of 'collapse-%s*' operators "
-            "(%s) must have an integer type, but it has a floating point "
-            "type ('%s')", counter, opstr, description,
+      error(EXIT_FAILURE, 0, "%s popped operand of 'collapse-%s*' "
+            "operators (%s) must have an integer type, but it has a "
+            "floating point type ('%s')", counter, opstr, description,
             gal_type_name(input->type,1));
 }
 
@@ -976,10 +1000,26 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
 
   /* If we are on any of the sigma-clipping operators, we need to pop two
      more operands. */
-  if(    operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD
+  if(    operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_MAD
+      || operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_STD
+      || operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_MEAN
+      || operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_MEDIAN
+      || operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_NUMBER
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_MAD
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD
       || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEAN
       || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEDIAN
-      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER )
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER
+      || operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MAD
+      || operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_STD
+      || operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEAN
+      || operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEDIAN
+      || operator==ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_NUMBER
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MAD
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_STD
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEAN
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEDIAN
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_NUMBER )
     {
       param2=operands_pop(p, token); /* Termination criteria. */
       param1=operands_pop(p, token); /* Multiple of sigma. */
@@ -996,9 +1036,9 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
   dimension=gal_data_copy_to_new_type_free(dimension, GAL_TYPE_LONG);
   dim=((long *)(dimension->array))[0];
   if(dim<=0)
-    error(EXIT_FAILURE, 0, "first popped operand of 'collapse-*' operators "
-          "(dimension to collapse) must be positive (larger than zero), it "
-          "is %ld", dim);
+    error(EXIT_FAILURE, 0, "first popped operand of 'collapse-*' "
+          "operators (dimension to collapse) must be positive (larger "
+          "than zero), it is %ld", dim);
   if(dim > input->ndim)
     error(EXIT_FAILURE, 0, "input dataset to '%s' has %zu dimension(s), "
           "but you have asked to collapse along dimension %ld", token,
@@ -1006,9 +1046,11 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
   if(param1)
     {
       arithmetic_collapse_single_value(param1, "third", "sigclip-",
-                                       "sigma-clip's multiple of sigma", 0);
+                                       "sigma-clip's multiple of sigma",
+                                       0);
       arithmetic_collapse_single_value(param2, "second", "sigclip-",
-                                       "sigma-clip's termination param", 0);
+                                       "sigma-clip's termination param",
+                                       0);
       param1=gal_data_copy_to_new_type_free(param1, GAL_TYPE_FLOAT32);
       param2=gal_data_copy_to_new_type_free(param2, GAL_TYPE_FLOAT32);
       p1=((float *)(param1->array))[0];
@@ -1057,30 +1099,110 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
                                               nt, mms, qmm);
       break;
 
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_MAD:
+      collapsed=gal_dimension_collapse_mclip_std(input, input->ndim-dim,
+                                                 p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MAD:
+      collapsed=gal_dimension_collapse_mclip_fill_std(input,
+                                input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_STD:
+      collapsed=gal_dimension_collapse_mclip_std(input, input->ndim-dim,
+                                                 p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_STD:
+      collapsed=gal_dimension_collapse_mclip_fill_std(input,
+                                input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_MEAN:
+      collapsed=gal_dimension_collapse_mclip_mean(input, input->ndim-dim,
+                                                  p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEAN:
+      collapsed=gal_dimension_collapse_mclip_fill_mean(input,
+                                 input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_MEDIAN:
+      collapsed=gal_dimension_collapse_mclip_median(input, input->ndim-dim,
+                                                    p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEDIAN:
+      collapsed=gal_dimension_collapse_mclip_fill_median(input,
+                                   input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_NUMBER:
+      collapsed=gal_dimension_collapse_mclip_number(input, input->ndim-dim,
+                                                    p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_NUMBER:
+      collapsed=gal_dimension_collapse_mclip_fill_number(input,
+                                   input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MAD:
+      collapsed=gal_dimension_collapse_sclip_std(input, input->ndim-dim,
+                                                 p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MAD:
+      collapsed=gal_dimension_collapse_sclip_fill_std(input,
+                                 input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
     case ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD:
       collapsed=gal_dimension_collapse_sclip_std(input, input->ndim-dim,
                                                  p1, p2, nt, mms, qmm);
       break;
 
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_STD:
+      collapsed=gal_dimension_collapse_sclip_fill_std(input,
+                                input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
     case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEAN:
       collapsed=gal_dimension_collapse_sclip_mean(input, input->ndim-dim,
                                                   p1, p2, nt, mms, qmm);
       break;
 
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEAN:
+      collapsed=gal_dimension_collapse_sclip_fill_mean(input,
+                                 input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
     case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEDIAN:
       collapsed=gal_dimension_collapse_sclip_median(input, input->ndim-dim,
                                                     p1, p2, nt, mms, qmm);
       break;
 
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEDIAN:
+      collapsed=gal_dimension_collapse_sclip_fill_median(input,
+                                   input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
     case ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER:
       collapsed=gal_dimension_collapse_sclip_number(input, input->ndim-dim,
                                                     p1, p2, nt, mms, qmm);
       break;
 
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_NUMBER:
+      collapsed=gal_dimension_collapse_sclip_fill_number(input,
+                                   input->ndim-dim, p1, p2, nt, mms, qmm);
+      break;
+
     default:
-      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
-            "problem. The operator code %d is not recognized", __func__,
-            PACKAGE_BUGREPORT, operator);
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+            "fix the problem. The operator code %d is not recognized",
+            __func__, PACKAGE_BUGREPORT, operator);
     }
 
 
@@ -1135,9 +1257,8 @@ arithmetic_tofile(struct arithmeticparams *p, char *token, int freeflag)
   popped->wcs=p->refdata.wcs;
   if(popped->ndim==1 && p->onedasimage==0)
     gal_table_write(popped, NULL, NULL, p->cp.tableformat, filename,
-                    "ARITHMETIC", 0);
-  else
-    gal_fits_img_write(popped, filename, NULL, PROGRAM_NAME);
+                    "ARITHMETIC", 0, 0);
+  else gal_fits_img_write(popped, filename, NULL, 0);
   if(!p->cp.quiet)
     printf(" - Write: %s\n", filename);
 
@@ -1384,6 +1505,18 @@ arithmetic_set_operator(char *string, size_t *num_operands, int *inlib)
         { op=ARITHMETIC_OP_COLLAPSE_MEDIAN;       *num_operands=0; }
       else if (!strcmp(string, "collapse-median"))
         { op=ARITHMETIC_OP_COLLAPSE_MEDIAN;       *num_operands=0; }
+      else if (!strcmp(string, "collapse-madclip-mad"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_MAD;  *num_operands=0; }
+      else if (!strcmp(string, "collapse-madclip-std"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_STD;  *num_operands=0; }
+      else if (!strcmp(string, "collapse-madclip-mean"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_MEAN; *num_operands=0; }
+      else if (!strcmp(string, "collapse-madclip-median"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_MEDIAN;*num_operands=0;}
+      else if (!strcmp(string, "collapse-madclip-number"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_NUMBER;*num_operands=0;}
+      else if (!strcmp(string, "collapse-sigclip-mad"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_MAD;  *num_operands=0; }
       else if (!strcmp(string, "collapse-sigclip-std"))
         { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD;  *num_operands=0; }
       else if (!strcmp(string, "collapse-sigclip-mean"))
@@ -1392,6 +1525,26 @@ arithmetic_set_operator(char *string, size_t *num_operands, int *inlib)
         { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEDIAN;*num_operands=0;}
       else if (!strcmp(string, "collapse-sigclip-number"))
         { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER;*num_operands=0;}
+      else if (!strcmp(string, "collapse-madclip-fill-mad"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MAD;  *num_operands=0; }
+      else if (!strcmp(string, "collapse-madclip-fill-std"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_STD;  *num_operands=0; }
+      else if (!strcmp(string, "collapse-madclip-fill-mean"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEAN; *num_operands=0; }
+      else if (!strcmp(string, "collapse-madclip-fill-median"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEDIAN;*num_operands=0;}
+      else if (!strcmp(string, "collapse-madclip-fill-number"))
+        { op=ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_NUMBER;*num_operands=0;}
+      else if (!strcmp(string, "collapse-sigclip-fill-mad"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MAD;  *num_operands=0; }
+      else if (!strcmp(string, "collapse-sigclip-fill-std"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_STD;  *num_operands=0; }
+      else if (!strcmp(string, "collapse-sigclip-fill-mean"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEAN; *num_operands=0; }
+      else if (!strcmp(string, "collapse-sigclip-fill-median"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEDIAN;*num_operands=0;}
+      else if (!strcmp(string, "collapse-sigclip-fill-number"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_NUMBER;*num_operands=0;}
       else if (!strcmp(string, "add-dimension-slow"))
         { op=ARITHMETIC_OP_ADD_DIMENSION_SLOW;    *num_operands=0; }
       else if (!strcmp(string, "add-dimension-fast"))
@@ -1447,7 +1600,7 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
   size_t i;
   unsigned int numop;
   int flags = GAL_ARITHMETIC_FLAGS_BASIC;
-  gal_data_t *d1=NULL, *d2=NULL, *d3=NULL, *d4=NULL;
+  gal_data_t *d1=NULL, *d2=NULL, *d3=NULL, *d4=NULL, *d5=NULL;
 
   /* Set the operating-mode flags if necessary. */
   if(p->cp.quiet) flags |= GAL_ARITHMETIC_FLAG_QUIET;
@@ -1488,6 +1641,14 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
           d1=operands_pop(p, operator_string);
           break;
 
+        case 5:
+          d5=operands_pop(p, operator_string);
+          d4=operands_pop(p, operator_string);
+          d3=operands_pop(p, operator_string);
+          d2=operands_pop(p, operator_string);
+          d1=operands_pop(p, operator_string);
+          break;
+
         case -1:
           /* This case is when the number of operands is itself an
              operand. So except for operators that have high-level
@@ -1507,17 +1668,52 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
                 PACKAGE_BUGREPORT, num_operands, operator_string);
         }
 
-      /* Meta operators (where only the information about the pixels is
-         relevant, not the pixel values themselves, like the 'index'
-         operator). For these (that only accept one operand), we should add
-         the first popped operand back on the stack, then  */
+      /* Operators with special attention. */
       switch(operator)
         {
+         /* Meta operators (where only the information about the pixels is
+            relevant, not the pixel values themselves, like the 'index'
+            operator). For these (that only accept one operand), we should
+            add the first popped operand back on the stack. */
         case GAL_ARITHMETIC_OP_INDEX:
         case GAL_ARITHMETIC_OP_COUNTER:
           operands_add(p, NULL, d1);
           d1=arithmetic_prepare_meta(d1, d2, d3);
           break;
+
+        /* Operators that also modify the reference WCS. */
+        case GAL_ARITHMETIC_OP_TRIM:
+          d1->wcs=gal_wcs_copy(p->refdata.wcs);
+          break;
+
+        /* Operators that need/modify the WCS. */
+        case GAL_ARITHMETIC_OP_POOLMAX:
+        case GAL_ARITHMETIC_OP_POOLMIN:
+        case GAL_ARITHMETIC_OP_POOLSUM:
+        case GAL_ARITHMETIC_OP_POOLMEAN:
+        case GAL_ARITHMETIC_OP_POOLMEDIAN:
+
+          /* Print warning in case of existing WCS. */
+          if(p->refdata.wcs)
+            {
+              if(p->cp.quiet==0)
+                error(EXIT_SUCCESS, 0, "WARNING: the WCS is not currently "
+                      "supported for the pooling operators (the output "
+                      "will not contain WCS). If it is necessary in your "
+                      "usage, please get in touch with us at '%s'. You "
+                      "can suppress this warning with the '--quiet' option",
+                      PACKAGE_BUGREPORT);
+              gal_wcs_free(p->refdata.wcs);
+              p->refdata.wcs=NULL;
+            }
+
+          /* In case you want to pass the WCS to the library function ...
+
+          d1->wcs=gal_wcs_copy(p->refdata.wcs);
+
+          ... uncomment the line above and comment the whole
+          'if(p->refdata.wcs)' check above (so the wcs is not freed).*/
+          break;
         }
 
       /* Run the arithmetic operation. Note that 'gal_arithmetic'
@@ -1526,7 +1722,16 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
          when the operator doesn't need three operands, the extra
          arguments will be ignored. */
       operands_add(p, NULL, gal_arithmetic(operator, p->cp.numthreads,
-                                           flags, d1, d2, d3, d4));
+                                           flags, d1, d2, d3, d4, d5));
+
+      /* Operators with special attention afterwards. */
+      switch(operator)
+        {
+        case GAL_ARITHMETIC_OP_TRIM:
+          gal_wcs_free(p->refdata.wcs);
+          p->refdata.wcs=gal_wcs_copy(p->operands->data->wcs);
+          break;
+        }
     }
 
   /* No need to call the arithmetic library, call the proper wrappers
@@ -1581,10 +1786,26 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
         case ARITHMETIC_OP_COLLAPSE_MEAN:
         case ARITHMETIC_OP_COLLAPSE_MEDIAN:
         case ARITHMETIC_OP_COLLAPSE_NUMBER:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_MAD:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MAD:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_STD:
         case ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_MEAN:
         case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEAN:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_MEDIAN:
         case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEDIAN:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_NUMBER:
         case ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MAD:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MAD:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_STD:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_STD:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEAN:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEAN:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEDIAN:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEDIAN:
+        case ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_NUMBER:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_NUMBER:
           arithmetic_collapse(p, operator_string, operator);
           break;
 
@@ -1647,7 +1868,7 @@ arithmetic_final_read_file(struct arithmeticparams *p,
       /* Read the data, note that the WCS has already been set. */
       out=gal_array_read_one_ch(filename, hdu, NULL,
                                 p->cp.minmapsize,
-                                p->cp.quietmmap);
+                                p->cp.quietmmap, "--hdu");
       out->ndim=gal_dimension_remove_extra(out->ndim, out->dsize,
                                             NULL);
       if(!p->cp.quiet) printf(" - %s (hdu %s) is read.\n", filename, hdu);
@@ -1700,7 +1921,7 @@ arithmetic_final_data(struct arithmeticparams *p)
 void
 reversepolish(struct arithmeticparams *p)
 {
-  char *printnum;
+  char *printnum=NULL;
   struct operand *otmp;
   size_t num_operands=0;
   gal_list_str_t *token;
@@ -1730,7 +1951,7 @@ reversepolish(struct arithmeticparams *p)
                    OPERATOR_PREFIX_LENGTH_TOFILE) )
         arithmetic_tofile(p, token->v, 0);
       else if( !strncmp(OPERATOR_PREFIX_TOFILEFREE, token->v,
-                   OPERATOR_PREFIX_LENGTH_TOFILE) )
+                        OPERATOR_PREFIX_LENGTH_TOFILEFREE) )
         arithmetic_tofile(p, token->v, 1);
       else if( !strncmp(token->v, GAL_ARITHMETIC_SET_PREFIX,
                         GAL_ARITHMETIC_SET_PREFIX_LENGTH) )
@@ -1757,8 +1978,10 @@ reversepolish(struct arithmeticparams *p)
          isn't an operator. */
       else
         {
-          operator=arithmetic_set_operator(token->v, &num_operands, &inlib);
-          arithmetic_operator_run(p, operator, token->v, num_operands, inlib);
+          operator=arithmetic_set_operator(token->v, &num_operands,
+                                           &inlib);
+          arithmetic_operator_run(p, operator, token->v, num_operands,
+                                  inlib);
         }
 
       /* Increment the token counter. */
@@ -1800,7 +2023,8 @@ reversepolish(struct arithmeticparams *p)
       printnum=gal_type_to_string(data->array, data->type, 0);
       printf("%s\n", printnum);
 
-      /* Clean up. */
+      /* Clean up (don't set it to 'NULL', this is used to specify if a
+         file was created in the end or not). */
       free(printnum);
     }
   else
@@ -1817,15 +2041,17 @@ reversepolish(struct arithmeticparams *p)
         { if(data->comment) free(data->comment);
           gal_checkset_allocate_copy(p->metacomment, &data->comment); }
 
-      /* Put a copy of the WCS structure from the reference image, it
-         will be freed while freeing 'data'. */
+      /* Put a copy of the WCS structure from the reference image, it will
+         be freed while freeing 'data'. But start the file with the 0-th
+         HDU keywords.*/
+      gal_fits_key_write(p->cp.ckeys, p->cp.output, "0", "NONE", 1, 1);
       if(data->ndim==1 && p->onedasimage==0)
         gal_table_write(data, NULL, NULL, p->cp.tableformat,
                         p->onedonstdout ? NULL : p->cp.output,
-                        "ARITHMETIC", 0);
+                        "ARITHMETIC", 0, 0);
       else
         for(tmp=data; tmp!=NULL; tmp=tmp->next)
-          gal_fits_img_write(tmp, p->cp.output, NULL, PROGRAM_NAME);
+          gal_fits_img_write(tmp, p->cp.output, NULL, 0);
 
       /* Let the user know that the job is done. */
       if(!p->cp.quiet)
@@ -1866,7 +2092,7 @@ reversepolish(struct arithmeticparams *p)
 
 
 /***************************************************************/
-/*************             Top function            *************/
+/*************         Top-level function          *************/
 /***************************************************************/
 void
 arithmetic(struct arithmeticparams *p)
