@@ -5,7 +5,7 @@ Convolve is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -147,7 +147,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
 {
   struct convolveparams *p = state->input;
 
-  /* Pass 'gal_options_common_params' into the child parser.  */
+  /* Pass 'gal_options_common_params' into the child parser. */
   state->child_inputs[0] = &p->cp;
 
   /* In case the user incorrectly uses the equal sign (for example
@@ -207,8 +207,10 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct convolveparams *p)
+ui_check_only_options(struct convolveparams *p)
 {
   struct gal_options_common_params *cp=&p->cp;
 
@@ -573,7 +575,7 @@ ui_preparations(struct convolveparams *p)
               "on 1D datasets");
       else
         {
-          /* Make sure the size of the kernel is the same as the input */
+          /* Make sure the size of the kernel is the same as the input. */
           if( p->input->dsize[0]!=p->kernel->dsize[0]
               || p->input->dsize[1]!=p->kernel->dsize[1] )
             error(EXIT_FAILURE, 0, "with the '--makekernel' ('-m') option, "
@@ -626,7 +628,7 @@ ui_preparations(struct convolveparams *p)
 
       /* If there are any NaN pixels, set them to zero and normalize it. A
          blank pixel in a kernel is going to make a completely blank
-         output.*/
+         output. */
       if( !p->nokernelnorm )
         {
           sumv=0;
@@ -730,7 +732,7 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct convolveparams *p)
 #include "args.h"
 
 
-  /* Initialize the options and necessary information.  */
+  /* Initialize the options and necessary information. */
   ui_initialize_options(p, program_options, gal_commonopts_options);
 
 
@@ -744,8 +746,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct convolveparams *p)
   gal_options_read_config_set(&p->cp);
 
 
-  /* Do a sanity check only on options. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
