@@ -5,7 +5,7 @@ CosmicCalculator is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -44,10 +44,6 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 
-
-/**************************************************************/
-/************            Main function            *************/
-/**************************************************************/
 static void
 cosmiccal_print_input(struct cosmiccalparams *p)
 {
@@ -76,38 +72,41 @@ cosmiccal_printall(struct cosmiccalparams *p)
   double curage, ccritd, distmod, outage, zcritd;
 
   /* The user wants everything, do all the calculations and print
-     everything with full descriptions. */
+     everything with full descriptions.
+
+     Note that 'quiet' is activated here because the same sanity check is
+     already done in CosmicCalculator's 'ui.c'. */
   curage=gal_cosmology_age(0.0f, p->H0, p->olambda, p->omatter,
-                           p->oradiation);
+                           p->oradiation, 1);
 
   ccritd=gal_cosmology_critical_density(0.0f, p->H0, p->olambda, p->omatter,
-                                        p->oradiation);
+                                        p->oradiation, 1);
 
   pd=gal_cosmology_proper_distance(p->redshift, p->H0, p->olambda, p->omatter,
-                                   p->oradiation);
+                                   p->oradiation, 1);
 
   ad=gal_cosmology_angular_distance(p->redshift, p->H0, p->olambda,
-                                    p->omatter, p->oradiation);
+                                    p->omatter, p->oradiation, 1);
 
   ld=gal_cosmology_luminosity_distance(p->redshift, p->H0, p->olambda,
-                                       p->omatter, p->oradiation);
+                                       p->omatter, p->oradiation, 1);
 
   distmod=gal_cosmology_distance_modulus(p->redshift, p->H0, p->olambda,
-                                         p->omatter, p->oradiation);
+                                         p->omatter, p->oradiation, 1);
 
   absmagconv=gal_cosmology_to_absolute_mag(p->redshift, p->H0, p->olambda,
-                                           p->omatter, p->oradiation);
+                                           p->omatter, p->oradiation, 1);
 
   outage=gal_cosmology_age(p->redshift, p->H0, p->olambda, p->omatter,
-                           p->oradiation);
+                           p->oradiation, 1);
 
   zcritd=gal_cosmology_critical_density(p->redshift, p->H0, p->olambda,
-                                        p->omatter, p->oradiation);
+                                        p->omatter, p->oradiation, 1);
 
   vel=gal_cosmology_velocity_from_z(p->redshift);
 
   vz=gal_cosmology_comoving_volume(p->redshift, p->H0, p->olambda, p->omatter,
-                                   p->oradiation);
+                                   p->oradiation, 1);
 
   /* Print out results: */
   cosmiccal_print_input(p);
@@ -146,6 +145,9 @@ cosmiccal_printall(struct cosmiccalparams *p)
 
 
 
+/***************************************************************/
+/*************         Top-level function          *************/
+/***************************************************************/
 void
 cosmiccal(struct cosmiccalparams *p)
 {
@@ -158,13 +160,16 @@ cosmiccal(struct cosmiccalparams *p)
   if(isnan(p->redshift))
     {
       cosmiccal_print_input(p);
-      printf("\n\nPlease specify a redshift with the '--redshift' (or '-z') "
-             "option.\n");
+      printf("\n\nPlease specify a redshift with the '--redshift' (or "
+             "'-z') option.\n");
       return;
     }
 
   /* In case the user just wants one number, only print that and
-     return. */
+     return.
+
+     Note that 'quiet' is activated here because the same sanity check is
+     already done in CosmicCalculator's 'ui.c'. */
   if(p->specific)
     {
       for(tmp=p->specific;tmp!=NULL;tmp=tmp->next)
@@ -178,85 +183,99 @@ cosmiccal(struct cosmiccalparams *p)
 
             case UI_KEY_AGENOW:
               printf("%f", gal_cosmology_age(0.0f, p->H0, p->olambda,
-                                              p->omatter, p->oradiation));
+                                             p->omatter, p->oradiation, 1));
               break;
 
             case UI_KEY_CRITICALDENSITYNOW:
               printf("%e", gal_cosmology_critical_density(0.0f, p->H0,
-                                                           p->olambda,
-                                                           p->omatter,
-                                                           p->oradiation));
+                                                          p->olambda,
+                                                          p->omatter,
+                                                          p->oradiation,
+                                                          1));
               break;
 
             case UI_KEY_PROPERDISTANCE:
-              printf("%f", gal_cosmology_proper_distance(p->redshift, p->H0,
-                                                          p->olambda,
-                                                          p->omatter,
-                                                          p->oradiation));
+              printf("%f", gal_cosmology_proper_distance(p->redshift,
+                                                         p->H0,
+                                                         p->olambda,
+                                                         p->omatter,
+                                                         p->oradiation,
+                                                         1));
               break;
 
-            case UI_KEY_ANGULARDIMDIST:
-              printf("%f", gal_cosmology_angular_distance(p->redshift, p->H0,
-                                                           p->olambda,
-                                                           p->omatter,
-                                                           p->oradiation));
+            case UI_KEY_ANGULARDIAMDIST:
+              printf("%f", gal_cosmology_angular_distance(p->redshift,
+                                                          p->H0,
+                                                          p->olambda,
+                                                          p->omatter,
+                                                          p->oradiation,
+                                                          1));
               break;
 
             case UI_KEY_ARCSECTANDIST:
-              printf("%f", ( gal_cosmology_angular_distance(p->redshift, p->H0,
-                                                             p->olambda,
-                                                             p->omatter,
-                                                             p->oradiation)
-                              * 1000 * M_PI / 3600 / 180 ) );
+              printf("%f", ( gal_cosmology_angular_distance(p->redshift,
+                                                            p->H0,
+                                                            p->olambda,
+                                                            p->omatter,
+                                                            p->oradiation,
+                                                            1)
+                             * 1000 * M_PI / 3600 / 180 ) );
               break;
 
             case UI_KEY_LUMINOSITYDIST:
               printf("%f", gal_cosmology_luminosity_distance(p->redshift,
-                                                              p->H0,
-                                                              p->olambda,
-                                                              p->omatter,
-                                                              p->oradiation));
+                                                             p->H0,
+                                                             p->olambda,
+                                                             p->omatter,
+                                                             p->oradiation,
+                                                             1));
               break;
 
             case UI_KEY_DISTANCEMODULUS:
-              printf("%f", gal_cosmology_distance_modulus(p->redshift, p->H0,
-                                                           p->olambda,
-                                                           p->omatter,
-                                                           p->oradiation));
+              printf("%f", gal_cosmology_distance_modulus(p->redshift,
+                                                          p->H0,
+                                                          p->olambda,
+                                                          p->omatter,
+                                                          p->oradiation,
+                                                          1));
               break;
 
             case UI_KEY_ABSMAGCONV:
               printf("%f", gal_cosmology_to_absolute_mag(p->redshift, p->H0,
-                                                          p->olambda,
-                                                          p->omatter,
-                                                          p->oradiation));
+                                                         p->olambda,
+                                                         p->omatter,
+                                                         p->oradiation,
+                                                         1));
               break;
 
             case UI_KEY_AGE:
               printf("%f", gal_cosmology_age(p->redshift, p->H0, p->olambda,
-                                              p->omatter, p->oradiation));
+                                             p->omatter, p->oradiation, 1));
               break;
 
             case UI_KEY_LOOKBACKTIME:
               curage=gal_cosmology_age(0.0f, p->H0, p->olambda, p->omatter,
-                                       p->oradiation);
-              zage=gal_cosmology_age(p->redshift, p->H0, p->olambda, p->omatter,
-                                     p->oradiation);
+                                       p->oradiation, 1);
+              zage=gal_cosmology_age(p->redshift, p->H0, p->olambda,
+                                     p->omatter, p->oradiation, 1);
               printf("%f", curage-zage);
               break;
 
             case UI_KEY_CRITICALDENSITY:
-              printf("%e", gal_cosmology_critical_density(p->redshift, p->H0,
-                                                           p->olambda,
-                                                           p->omatter,
-                                                           p->oradiation));
+              printf("%e", gal_cosmology_critical_density(p->redshift,
+                                                          p->H0,
+                                                          p->olambda,
+                                                          p->omatter,
+                                                          p->oradiation,
+                                                          1));
               break;
 
             case UI_KEY_VOLUME:
               printf("%f", gal_cosmology_comoving_volume(p->redshift, p->H0,
-                                                          p->olambda,
-                                                          p->omatter,
-                                                          p->oradiation));
+                                                         p->olambda,
+                                                         p->omatter,
+                                                         p->oradiation,
+                                                         1));
               break;
 
             case UI_KEY_USEDVELOCITY:
@@ -264,17 +283,19 @@ cosmiccal(struct cosmiccalparams *p)
               break;
 
             case UI_KEY_LINEATZ:
-              printf("%g", gal_list_f64_pop(&p->specific_arg)*(1+p->redshift));
+              printf("%g", ( gal_list_f64_pop(&p->specific_arg)
+                             * (1+p->redshift) * p->lineunitmultip) );
               break;
 
             default:
-              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
-                    "fix the problem. The code %d is not recognized as a "
-                    "single value calculation code", __func__,
+              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
+                    "to fix the problem. The code %d is not recognized as "
+                    "a single value calculation code", __func__,
                     PACKAGE_BUGREPORT, tmp->v);
             }
 
-          /* Only add a space-character if there are more results to print. */
+          /* Only add a space-character if there are more results to
+             print. */
           if(tmp->next) printf(" ");
         }
 
@@ -283,4 +304,18 @@ cosmiccal(struct cosmiccalparams *p)
     }
   else
     cosmiccal_printall(p);
+
+  /* Print a warning if the redshift is too close for the hubble flow to be
+     significant. This is done at the end because it is important and may
+     be missed at the start of the program (before the outputs are
+     printed). */
+  if(p->redshift<MAIN_REDSHIFT_SIG_HUBBLE_FLOW && p->cp.quiet==0)
+    error(EXIT_SUCCESS, 0, "WARNING: at very low redshifts "
+          "(approximately below %g), the peculiar velocity of the "
+          "particular galaxy may be more significant than hubble's "
+          "law (which is the basis of the measurements here). This "
+          "gets worse as the redshift decreases. Therefore the "
+          "results above may not be accurate on a per-object basis. "
+          "You can suppress this warning with the '--quiet' option",
+          MAIN_REDSHIFT_SIG_HUBBLE_FLOW);
 }
