@@ -6,7 +6,7 @@ Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
      Pedram Ashofteh-Ardakani <pedramardakani@pm.me>
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -59,11 +59,13 @@ struct fitsparams
   int    hdu_in_commandline;   /* HDU wasn't given in config. file.     */
   gal_list_str_t     *input;   /* Name of input file.                   */
   char              *outhdu;   /* HDU of output (only when necessary).  */
+  char           *arguments;   /* List of input files.                  */
   gal_list_str_t    *remove;   /* Remove extensions from a file.        */
   gal_list_str_t      *copy;   /* Copy extensions to output.            */
   gal_list_str_t       *cut;   /* Copy ext. to output and remove.       */
   uint8_t           numhdus;   /* Print number of HDUs in FITS file.    */
   uint8_t           datasum;   /* Calculate and print HDU's datasum.    */
+  uint8_t    datasumencoded;   /* Calculate and print HDU's datasum.    */
   uint8_t        pixelscale;   /* Calculate and print HDU's pixelscale. */
   uint8_t  pixelareaarcsec2;   /* Return pixel area in arcsec^2.        */
   uint8_t       skycoverage;   /* Calculate and image coverage in WCS.  */
