@@ -6,7 +6,7 @@ Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
      Sachin Kumar Singh <sachinkumarsingh092@gmail.com>
-Copyright (C) 2017-2022 Free Software Foundation, Inc.
+Copyright (C) 2017-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -45,8 +45,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Number of columns in a file. */
 static gal_list_str_t *
-match_add_all_cols(char *filename, char *extname, gal_list_str_t *stdinlines,
-                   gal_list_str_t *incols, size_t *num)
+match_add_all_cols(char *filename, char *extname,
+                   gal_list_str_t *stdinlines,
+                   gal_list_str_t *incols, size_t *num,
+                   char *hdu_option_name)
 {
   char *tstr;
   int tableformat;
@@ -64,8 +66,9 @@ match_add_all_cols(char *filename, char *extname, gal_list_str_t *stdinlines,
           if( numcols == GAL_BLANK_SIZE_T )
             {
               colinfo=gal_table_info(filename, extname,
-                                     filename ? NULL : stdinlines, &numcols,
-                                     &numrows, &tableformat);
+                                     filename ? NULL : stdinlines,
+                                     &numcols, &numrows, &tableformat,
+                                     hdu_option_name);
               gal_data_array_free(colinfo, numcols, 1);
             }
 
@@ -224,23 +227,25 @@ static void
 match_arrange_in_new_col(struct matchparams *p, gal_data_t *in,
                          size_t *permutation, size_t nummatched)
 {
-  size_t c=0, i;
+  size_t c=0, i, n;
   size_t istart=p->notmatched ? nummatched : 0;
-  size_t iend=p->notmatched ? in->size : nummatched;
-  size_t outsize=p->notmatched ? in->size - nummatched : nummatched;
+  size_t iend=p->notmatched ? in->dsize[0] : nummatched;
+  size_t outrows=p->notmatched ? in->dsize[0] - nummatched : nummatched;
+
+  /* Set the number of values in this column (for vectors). */
+  n = in->ndim==1 ? 1 : in->dsize[1];
 
   /* Allocate the array. */
-  void *out=gal_pointer_allocate_ram_or_mmap(in->type, outsize, 0,
+  void *out=gal_pointer_allocate_ram_or_mmap(in->type, outrows*n, 0,
                                              p->cp.minmapsize,
                                              &in->mmapname, p->cp.quietmmap,
                                              __func__, "out");
 
   /* Copy the matched rows into the output array. */
   for(i=istart;i<iend;++i)
-    memcpy(gal_pointer_increment(out, c++, in->type),
-           gal_pointer_increment(in->array, permutation[i],
-                                 in->type),
-           gal_type_sizeof(in->type));
+    memcpy(gal_pointer_increment(out,       n*c++,            in->type),
+           gal_pointer_increment(in->array, n*permutation[i], in->type),
+           gal_type_sizeof(in->type) * n);
 
   /**********************************/
   /* Add a check so if the column is a string, we free the strings that
@@ -248,8 +253,9 @@ match_arrange_in_new_col(struct matchparams *p, gal_data_t *in,
   /**********************************/
 
   /* Free the existing array, and correct the sizes. */
-  in->size = in->dsize[0] = outsize;
   free(in->array);
+  in->dsize[0] = outrows;
+  in->size = in->dsize[0] * (in->ndim==1 ? 1 : in->dsize[1]);
   in->array=out;
 }
 
@@ -313,6 +319,7 @@ match_catalog_read_write_all(struct matchparams *p, size_t *permutation,
   gal_data_t *tmp, *cat;
   gal_list_str_t *cols, *tcol;
 
+  char *hopt             = (f1s2==1) ? "--hdu"       : "--hdu2";
   char *hdu              = (f1s2==1) ? p->cp.hdu     : p->hdu2;
   gal_list_str_t *incols = (f1s2==1) ? p->acols      : p->bcols;
   size_t *numcols        = (f1s2==1) ? &p->anum      : &p->bnum;
@@ -335,7 +342,7 @@ match_catalog_read_write_all(struct matchparams *p, size_t *permutation,
       if(hasall)
         {
           cols=match_add_all_cols(filename, hdu, p->stdinlines, incols,
-                                  numcols);
+                                  numcols, hopt);
           if(f1s2==1) { gal_list_str_free(p->acols, 0); p->acols=cols; }
           else        { gal_list_str_free(p->bcols, 0); p->bcols=cols; }
         }
@@ -357,7 +364,7 @@ match_catalog_read_write_all(struct matchparams *p, size_t *permutation,
     cat=gal_table_read(filename, hdu, filename ? NULL : p->stdinlines,
                        cols, p->cp.searchin, p->cp.ignorecase,
                        p->cp.numthreads, p->cp.minmapsize,
-                       p->cp.quietmmap, *numcolmatch);
+                       p->cp.quietmmap, *numcolmatch, hopt);
   else
     cat=match_cat_from_coord(p, cols, *numcolmatch);
 
@@ -402,7 +409,7 @@ match_catalog_read_write_all(struct matchparams *p, size_t *permutation,
     {
       /* Write the catalog to a file. */
       gal_table_write(cat, NULL, NULL, p->cp.tableformat, outname,
-                      extname, 0);
+                      extname, 0, 0);
 
       /* Clean up. */
       gal_list_data_free(cat);
@@ -478,7 +485,7 @@ match_catalog_write_one_row(struct matchparams *p, gal_data_t *a,
       /* Reverse the table and write it out. */
       gal_list_data_reverse(&cat);
       gal_table_write(cat, NULL, NULL, p->cp.tableformat,
-                      p->out1name, "MATCHED", 0);
+                      p->out1name, "MATCHED", 0, 0);
       gal_list_data_free(cat);
     }
 
@@ -486,7 +493,7 @@ match_catalog_write_one_row(struct matchparams *p, gal_data_t *a,
      it ('a' will be freed in the higher-level function). */
   else
     gal_table_write(a, NULL, NULL, p->cp.tableformat, p->out1name,
-                    "MATCHED", 0);
+                    "MATCHED", 0, 0);
 }
 
 
@@ -544,7 +551,7 @@ match_catalog_write_one_col(struct matchparams *p, gal_data_t *a,
   /* Reverse the table and write it out. */
   gal_list_data_reverse(&cat);
   gal_table_write(cat, NULL, NULL, p->cp.tableformat, p->out1name,
-                  "MATCHED", 0);
+                  "MATCHED", 0, 0);
   gal_list_data_free(cat);
 }
 
@@ -584,10 +591,10 @@ match_catalog_kdtree_build(struct matchparams *p)
   gal_fits_key_write_filename("KDTIN", p->input1name, &keylist, 0,
                               p->cp.quiet);
   gal_fits_key_list_add_end(&keylist, GAL_TYPE_SIZE_T,
-                            MATCH_KDTREE_ROOT_KEY, 0,
-                            &root, 0, comment, 0, unit, 0);
-  gal_table_write(kdtree, &keylist, NULL, GAL_TABLE_FORMAT_BFITS,
-                  p->out1name, "kdtree", 0);
+                            MATCH_KDTREE_ROOT_KEY, 0, &root, 0,
+                            comment, 0, unit, 0);
+  gal_table_write(kdtree, keylist, NULL, GAL_TABLE_FORMAT_BFITS,
+                  p->out1name, "kdtree", 0, 1);
 
   /* Let the user know that the k-d tree has been built. */
   if(!p->cp.quiet)
@@ -780,8 +787,8 @@ match_catalog(struct matchparams *p)
          tables. So if the log file is a FITS table, covert the two
          index columns to uint32. */
       tmp=gal_data_copy_to_new_type(mcols, GAL_TYPE_UINT32);
+      tmp->size=tmp->dsize[0]=nummatched;
       tmp->next=mcols->next;
-      tmp->size=nummatched;
       gal_data_free(mcols);
       mcols=tmp;
 
@@ -795,9 +802,9 @@ match_catalog(struct matchparams *p)
       /* Same for the second set of indexs. */
       tmp=gal_data_copy_to_new_type(mcols->next, GAL_TYPE_UINT32);
       uf = (u=tmp->array) + tmp->size; do (*u)++; while(++u<uf);
+      tmp->size=tmp->dsize[0]=nummatched;
       tmp->next=mcols->next->next;
       gal_data_free(mcols->next);
-      tmp->size=nummatched;
       mcols->next=tmp;
 
       /* Correct the comments. */
@@ -809,11 +816,15 @@ match_catalog(struct matchparams *p)
 
       /* Write them into the table. */
       gal_table_write(mcols, NULL, NULL, p->cp.tableformat, p->logname,
-                      "LOG_INFO", 0);
+                      "LOG_INFO", 0, 0);
 
       /* Set the comment pointer to NULL: they weren't allocated. */
       mcols->comment=NULL;
       mcols->next->comment=NULL;
+
+      /* Inform the user that a log-file has been created. */
+      if(!p->cp.quiet)
+        fprintf(stdout, "  - Output (log): %s\n", p->logname);
     }
 
   /* Clean up. */
@@ -850,7 +861,7 @@ match_catalog(struct matchparams *p)
 
 
 /*******************************************************************/
-/*************            Top level function           *************/
+/*************            Top-level function           *************/
 /*******************************************************************/
 void
 match(struct matchparams *p)
@@ -874,11 +885,10 @@ match(struct matchparams *p)
       gal_fits_key_write_filename("input1", ( p->input1name
                                               ? p->input1name
                                               : "Standard input" ),
-                                  &p->cp.okeys, 1, p->cp.quiet);
+                                  &p->cp.ckeys, 1, p->cp.quiet);
       gal_fits_key_write_filename("input2",
                                   p->input2name?p->input2name:"--coord",
-                                  &p->cp.okeys, 1, p->cp.quiet);
-      gal_fits_key_write_config(&p->cp.okeys, "Match configuration",
-                                "MATCH-CONFIG", p->out1name, "0");
+                                  &p->cp.ckeys, 1, p->cp.quiet);
+      gal_fits_key_write(p->cp.ckeys, p->out1name, "0", "NONE", 1, 0);
     }
 }
