@@ -5,7 +5,7 @@ Match is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2022 Free Software Foundation, Inc.
+Copyright (C) 2017-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -213,10 +213,10 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
-/* Read and check ONLY the options. When arguments are involved, do the
-   check in 'ui_check_options_and_arguments'. */
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct matchparams *p)
+ui_check_only_options(struct matchparams *p)
 {
   /* k-d tree specific sanity checks. */
   if(p->kdtree)
@@ -360,7 +360,7 @@ ui_set_mode(struct matchparams *p)
      on plain text. */
   if( p->input1name && gal_fits_file_recognized(p->input1name) )
     {
-      tin1=gal_fits_hdu_format(p->input1name, p->cp.hdu);
+      tin1=gal_fits_hdu_format(p->input1name, p->cp.hdu, "--hdu");
       p->mode = (tin1 == IMAGE_HDU) ? MATCH_MODE_WCS : MATCH_MODE_CATALOG;
     }
   else
@@ -384,7 +384,7 @@ ui_set_mode(struct matchparams *p)
     {
       if(gal_fits_file_recognized(p->input2name))
         {
-          tin2=gal_fits_hdu_format(p->input2name, p->hdu2);
+          tin2=gal_fits_hdu_format(p->input2name, p->hdu2, "--hdu2");
           if(tin1==IMAGE_HDU && tin2!=IMAGE_HDU)
             {
               t1 = (tin1==IMAGE_HDU) ? "image" : "catalog";
@@ -628,14 +628,15 @@ ui_set_columns_sanity_check_read_aperture(struct matchparams *p)
   if(p->coord || p->kdtreemode==MATCH_KDTREE_BUILD)
     {
       if(p->ccol1==NULL)
-        error(EXIT_FAILURE, 0, "no value given to '--ccol1' (necessary with "
-              "'--coord')");
+        error(EXIT_FAILURE, 0, "no value given to '--ccol1' (necessary "
+              "with '--coord')");
     }
   else
     {
       if(p->ccol1==NULL || p->ccol2==NULL)
-        error(EXIT_FAILURE, 0, "both '--ccol1' and '--ccol2' must be given. "
-              "They specify the columns containing the coordinates to match");
+        error(EXIT_FAILURE, 0, "both '--ccol1' and '--ccol2' must be "
+              "given. They specify the columns containing the "
+              "coordinates to match");
     }
 
   /* Make sure the same number of columns is given to both. Note that a
@@ -661,8 +662,8 @@ ui_set_columns_sanity_check_read_aperture(struct matchparams *p)
       {
       case 1:
         if(p->aperture->size>1)
-          error(EXIT_FAILURE, 0, "%zu values given to '--aperture'. In a 1D "
-                "match, this option can only take one value",
+          error(EXIT_FAILURE, 0, "%zu values given to '--aperture'. In "
+                "a 1D match, this option can only take one value",
                 p->aperture->size);
         break;
 
@@ -719,8 +720,9 @@ ui_set_columns_from_coord(struct matchparams *p)
 /* We want to keep the columns as double type. So what-ever their original
    type is, convert it. */
 static gal_data_t *
-ui_read_columns_to_double(struct matchparams *p, char *filename, char *hdu,
-                          gal_list_str_t *cols, size_t numcols)
+ui_read_columns_to_double(struct matchparams *p, char *filename,
+                          char *hdu, gal_list_str_t *cols,
+                          size_t numcols, char *hdu_option_name)
 {
   gal_data_t *tmp, *ttmp, *tout, *out=NULL;
   struct gal_options_common_params *cp=&p->cp;
@@ -739,7 +741,8 @@ ui_read_columns_to_double(struct matchparams *p, char *filename, char *hdu,
                                           "input");
   tout=gal_table_read(filename, hdu, filename ? NULL : p->stdinlines,
                       cols, cp->searchin, cp->ignorecase, cp->numthreads,
-                      cp->minmapsize, p->cp.quietmmap, NULL);
+                      cp->minmapsize, p->cp.quietmmap, NULL,
+                      hdu_option_name);
 
   /* A small sanity check. */
   if(gal_list_data_number(tout)!=numcols)
@@ -752,6 +755,12 @@ ui_read_columns_to_double(struct matchparams *p, char *filename, char *hdu,
   tmp=tout;
   while(tmp!=NULL)
     {
+      /* If the coordinate column has more than one dimension (a vector
+         column), the abort with an error: */
+      if(tmp->ndim!=1)
+        error(EXIT_FAILURE, 0, "only single-valued columns can be given "
+              "to '--ccol1' or '--ccol2'");
+
       /* We need ot set the 'next' pointer  */
       ttmp=tmp->next;
       tmp->next=NULL;
@@ -795,7 +804,7 @@ ui_read_kdtree(struct matchparams *p)
   p->kdtreedata=gal_table_read(p->kdtree, p->kdtreehdu, NULL,
                                NULL, GAL_TABLE_SEARCH_NAME, 1,
                                p->cp.numthreads, p->cp.minmapsize,
-                               p->cp.quietmmap, NULL);
+                               p->cp.quietmmap, NULL, "--kdtreehdu");
 
   /* It has to have only two columns, with an unsigned 32-bit integer
      type in each. */
@@ -822,7 +831,7 @@ ui_read_kdtree(struct matchparams *p)
   /* Read the k-d tree root. */
   keysll[0].type=GAL_TYPE_SIZE_T;
   keysll[0].name=MATCH_KDTREE_ROOT_KEY;
-  gal_fits_key_read(p->kdtree, p->kdtreehdu, keysll, 0, 0);
+  gal_fits_key_read(p->kdtree, p->kdtreehdu, keysll, 0, 0, "--kdtreehdu");
   if(keysll[0].status)
     error(EXIT_FAILURE, 0, "%s (hdu: %s, that was given to "
           "'--kdtree') doesn't have the '%s' keyword, or it "
@@ -869,12 +878,12 @@ ui_read_columns(struct matchparams *p)
 
   /* Read-in the columns. */
   p->cols1=ui_read_columns_to_double(p, p->input1name, p->cp.hdu,
-                                     cols1, ndim);
+                                     cols1, ndim, "--hdu");
   if( p->kdtreemode!=MATCH_KDTREE_BUILD )
     p->cols2=( p->coord
                ? ui_set_columns_from_coord(p)
                : ui_read_columns_to_double(p, p->input2name, p->hdu2,
-                                           cols2, ndim) );
+                                           cols2, ndim, "--hdu2") );
 
   /* If an external k-d tree is given, read it and make sure it has the
      same number of rows as the first input and the proper datatype. */
@@ -993,6 +1002,7 @@ ui_preparations_out_cols(struct matchparams *p)
 
 
 
+#define UI_OUT_SUFFNAME "-matched"
 static void
 ui_preparations_out_name(struct matchparams *p)
 {
@@ -1010,14 +1020,14 @@ ui_preparations_out_name(struct matchparams *p)
         {
           if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
             p->logname=gal_checkset_automatic_output(&p->cp, refname,
-                                                     "_matched.txt");
+                                                     UI_OUT_SUFFNAME".txt");
           else
             p->logname=gal_checkset_automatic_output(&p->cp, refname,
-                                                     "_matched.fits");
+                                                     UI_OUT_SUFFNAME".fits");
         }
 
       /* Make sure a file with this name doesn't exist. */
-      gal_checkset_writable_remove(p->logname, 0, p->cp.dontdelete);
+      gal_checkset_writable_remove(p->logname, NULL, 0, p->cp.dontdelete);
 
       /* The main output name needs to be available in p->out1name (for the
          final step when we want to write the input configurations as FITS
@@ -1033,13 +1043,15 @@ ui_preparations_out_name(struct matchparams *p)
           else
             {
               suffix = ( p->kdtreemode==MATCH_KDTREE_BUILD
-                         ? "_kdtree.fits"
+                         ? "-kdtree.fits"
                          : ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
-                             ? "_matched.txt" : "_matched.fits") );
+                             ? UI_OUT_SUFFNAME".txt"
+                             : UI_OUT_SUFFNAME".fits") );
               p->out1name = gal_checkset_automatic_output(&p->cp,
                                                           refname, suffix);
             }
-          gal_checkset_writable_remove(p->out1name, 0, p->cp.dontdelete);
+          gal_checkset_writable_remove(p->out1name, refname, 0,
+                                       p->cp.dontdelete);
         }
       else
         {
@@ -1063,10 +1075,12 @@ ui_preparations_out_name(struct matchparams *p)
                   p->cp.keepinputdir=1;
                   p->out1name=gal_checkset_automatic_output(&p->cp,
                                                             p->cp.output,
-                                                            "_matched_1.txt");
+                                                            UI_OUT_SUFFNAME
+                                                            "-1.txt");
                   p->out2name=gal_checkset_automatic_output(&p->cp,
                                                             p->cp.output,
-                                                            "_matched_2.txt");
+                                                            UI_OUT_SUFFNAME
+                                                            "-2.txt");
                   p->cp.keepinputdir=keepinputdir_orig;
                 }
             }
@@ -1075,32 +1089,34 @@ ui_preparations_out_name(struct matchparams *p)
               if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
                 {
                   p->out1name=gal_checkset_automatic_output(&p->cp, refname,
-                                                            "_matched_1.txt");
+                                                            UI_OUT_SUFFNAME
+                                                            "-1.txt");
                   p->out2name=gal_checkset_automatic_output(&p->cp,
                                                             p->input2name,
-                                                            "_matched_2.txt");
+                                                            UI_OUT_SUFFNAME
+                                                            "-2.txt");
                 }
               else
                 {
                   p->out1name=gal_checkset_automatic_output(&p->cp, refname,
-                                                            "_matched.fits");
+                                                            UI_OUT_SUFFNAME
+                                                            ".fits");
                   gal_checkset_allocate_copy(p->out1name, &p->out2name);
                 }
             }
 
           /* Make sure no file with these names exists. */
-          gal_checkset_writable_remove(p->out1name, 0, p->cp.dontdelete);
-          gal_checkset_writable_remove(p->out2name, 0, p->cp.dontdelete);
+          gal_checkset_writable_remove(p->out1name, refname, 0,
+                                       p->cp.dontdelete);
+          gal_checkset_writable_remove(p->out2name, refname, 0,
+                                       p->cp.dontdelete);
         }
 
       /* If a log file is necessary, set its name here. */
       if(p->cp.log)
-        {
-          p->logname = ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
-                         ? PROGRAM_EXEC".txt"
-                         : PROGRAM_EXEC".fits" );
-          gal_checkset_writable_remove(p->logname, 0, p->cp.dontdelete);
-        }
+        p->logname=gal_checkset_automatic_output(&p->cp, p->out1name,
+                                                 UI_OUT_SUFFNAME
+                                                 "-log.fits");
     }
 }
 
@@ -1182,9 +1198,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct matchparams *p)
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
