@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2023 Free Software Foundation, Inc.
+Copyright (C) 2016-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -359,10 +359,10 @@ ui_check_upperlimit(struct argp_option *option, char *arg,
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
-/* Read and check ONLY the options. When arguments are involved, do the
-   check in 'ui_check_options_and_arguments'. */
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct mkcatalogparams *p)
+ui_check_only_options(struct mkcatalogparams *p)
 {
   float tmp;
   double *darr;
@@ -544,7 +544,7 @@ ui_wcs_info(struct mkcatalogparams *p)
   /* Read the WCS meta-data. */
   p->objects->wcs=gal_wcs_read(p->objectsfile, p->cp.hdu,
                                p->cp.wcslinearmatrix, 0, 0,
-                               &p->objects->nwcs);
+                               &p->objects->nwcs, "--hdu");
 
   /* Read the basic WCS information. */
   if(p->objects->wcs)
@@ -575,26 +575,35 @@ ui_wcs_info(struct mkcatalogparams *p)
 
 
 
+/* When the number of clumps is not specified a priori (through the
+   'NUMLABS' keyword in the clumps array, we need to map the original
+   labels to new labels that start counting from one and are
+   congtiguous. */
 static size_t
-ui_num_clumps(struct mkcatalogparams *p)
+ui_num_clumps_orig_labs(struct mkcatalogparams *p)
 {
-  int32_t olab;
-  char *basename;
-  int keepinputdir;
-  size_t i, counter, numclumps=0;
-  gal_list_i32_t *tmp, **labsinobj;
-  int32_t *o=p->objects->array, *of=o+p->objects->size, *c=p->clumps->array;
+  size_t ncinobj;
+  gal_list_i32_t *tmp, **origclumplist;
+  int32_t olab, **origclpid, *c=p->clumps->array;
+  size_t i, j, numclumps=0, nobjpone=p->numobjects+1;
+  int32_t *o=p->objects->array, *of=o+p->objects->size;
 
-  /* Allocate array of lists to keep the unique labels within each
-     object. */
+  /* Allocate array of lists and arrays to keep the unique labels within
+     each object. The list is used in the first run and the array is later
+     filled based on the list.*/
   errno=0;
-  labsinobj=calloc(p->numobjects+1, sizeof *labsinobj);
-  if(labsinobj==NULL)
+  origclumplist=calloc(nobjpone, sizeof *origclumplist);
+  if(origclumplist==NULL)
     error(EXIT_FAILURE, 0, "%s: couldn't allocate %zu bytes for "
-          "'labsinobj'", __func__, p->numobjects * sizeof *labsinobj);
+          "'origclumplist'", __func__, nobjpone * sizeof *origclumplist);
+  errno=0;
+  origclpid=calloc(nobjpone, sizeof *origclpid);
+  if(origclpid==NULL)
+    error(EXIT_FAILURE, 0, "%s: couldn't allocate %zu bytes for "
+          "'origclpid'", __func__, nobjpone * sizeof *origclpid);
+  p->origclpid=origclpid; /* To avoid 'p->' in the lines below. */
 
-  /* Go over each pixel and find the unique clump labels within each
-     object. */
+  /* Go over each pixel and find the list of original clump labels. */
   do
     {
       /* Do the steps if we are on a clump. */
@@ -602,14 +611,14 @@ ui_num_clumps(struct mkcatalogparams *p)
         {
           /* See if the label has already been found. */
           olab = p->outlabsinv ? p->outlabsinv[*o] : *o;
-          for(tmp=labsinobj[olab];tmp!=NULL;tmp=tmp->next)
+          for(tmp=origclumplist[olab];tmp!=NULL;tmp=tmp->next)
             if(tmp->v==*c) break;
 
           /* When it wasn't found, 'tmp==NULL'. */
           if(tmp==NULL)
             {
               ++numclumps;
-              gal_list_i32_add(&labsinobj[olab], *c);
+              gal_list_i32_add(&origclumplist[olab], *c);
             }
         }
 
@@ -618,8 +627,67 @@ ui_num_clumps(struct mkcatalogparams *p)
     }
   while(++o<of);
 
-  /* Re-write the clump values so their numbering is contiguous, since this
-     is assumed during the later steps. */
+  /* For a check of the list.
+  for(i=1;i<p->numobjects+1;++i)
+    {
+      printf("list%zu: ", i);
+      for(tmp=origclumplist[i];tmp!=NULL;tmp=tmp->next)
+        printf("%d%s", tmp->v, tmp->next?" ":"");
+      printf("\n");
+    }
+  */
+
+  /* Build a separate array of original labels for each objects. */
+  for(i=1;i<p->numobjects+1;++i)
+    {
+      /* Find the number of clumps in this object and allocate. */
+      ncinobj=gal_list_i32_number(origclumplist[i]);
+      origclpid[i]=gal_pointer_allocate(GAL_TYPE_INT32, ncinobj+1, 0,
+                                          __func__, "origclpid");
+
+      /* Put the list into the array in reverse because it was filled as
+         last-in-last-out. We are also setting the last element to blank so
+         we can parse through the labels when necessary. */
+      j=ncinobj;
+      origclpid[i][j--]=GAL_BLANK_INT32;
+      for(tmp=origclumplist[i];tmp!=NULL;tmp=tmp->next)
+        origclpid[i][j--]=tmp->v;
+    }
+
+  /* For a check of the array.
+  for(i=1;i<p->numobjects+1;++i)
+    if(i==9)
+    {
+      printf("arr%zu: ", i);
+      for(j=0; origclpid[i][j]!=GAL_BLANK_INT32; ++j)
+        printf("%d%s", origclpid[i][j],
+               origclpid[i][j+1]==GAL_BLANK_INT32?"":" ");
+      printf("\n");
+    }
+  */
+
+  /* Clean up and return the total number of clumps. */
+  for(i=1;i<p->numobjects;++i) gal_list_i32_free(origclumplist[i]);
+  free(origclumplist);
+  return numclumps;
+}
+
+
+
+
+static size_t
+ui_num_clumps(struct mkcatalogparams *p)
+{
+  size_t i, numclumps=0;
+  int32_t olab, *c=p->clumps->array;
+  int32_t *o=p->objects->array, *of=o+p->objects->size;
+
+  /* Count the total number of clumps and extract the original labels
+     within the clumps into an array ('p->origclpid'). */
+  numclumps=ui_num_clumps_orig_labs(p);
+
+  /* Re-write the clump values so their numbering is contiguous (this is
+     assumed during the later steps). */
   o=p->objects->array;
   c=p->clumps->array;
   do
@@ -627,10 +695,9 @@ ui_num_clumps(struct mkcatalogparams *p)
       /* Do the steps if we are on a clump. */
       if(*o>0 && *c>0)
         {
-          counter=0;
           olab = p->outlabsinv ? p->outlabsinv[*o] : *o;
-          for(tmp=labsinobj[olab];tmp!=NULL;tmp=tmp->next)
-            { counter++; if(tmp->v==*c) {*c=counter; break;} }
+          for(i=0;p->origclpid[olab][i]!=GAL_BLANK_INT32;++i)
+            if(p->origclpid[olab][i]==*c) { *c=i+1; break; }
         }
 
       /* Increment the clumps pointer.*/
@@ -638,19 +705,18 @@ ui_num_clumps(struct mkcatalogparams *p)
     }
   while(++o<of);
 
-  /* Write the created file into a file for the user to inspect. */
+  /* For a check
+  char *basename;
+  char *checkname;
+  int keepinputdir;
   keepinputdir=p->cp.keepinputdir;
   p->cp.keepinputdir = p->cp.output ? 1 : 0;
   basename = p->cp.output ? p->cp.output : p->objectsfile;
-  p->relabclumps=gal_checkset_automatic_output(&p->cp, basename,
-                                               "-clumps-relab.fits");
-  gal_fits_img_write(p->clumps, p->relabclumps, NULL, PROGRAM_STRING);
+  checkname=gal_checkset_automatic_output(&p->cp, basename,
+                                          "-clumps-relab.fits");
+  gal_fits_img_write(p->clumps, checkname, NULL, PROGRAM_STRING);
   p->cp.keepinputdir=keepinputdir;
-
-  /* Clean up. */
-  for(i=0;i<p->numobjects;++i)
-    if(labsinobj[i]) gal_list_i32_free(labsinobj[i]);
-  free(labsinobj);
+  */
 
   /* Return the number of clumps. */
   return numclumps;
@@ -843,11 +909,11 @@ ui_read_labels(struct mkcatalogparams *p)
 
   /* Read it into memory. */
   p->objects = gal_array_read_one_ch(p->objectsfile, p->cp.hdu, NULL,
-                                     p->cp.minmapsize, p->cp.quietmmap);
+                                     p->cp.minmapsize, p->cp.quietmmap,
+                                     "--hdu");
   p->objects->ndim=gal_dimension_remove_extra(p->objects->ndim,
                                               p->objects->dsize, NULL);
 
-
   /* Make sure it has an integer type. */
   ui_check_type_int(p->objectsfile, p->cp.hdu, p->objects->type);
 
@@ -887,21 +953,25 @@ ui_read_labels(struct mkcatalogparams *p)
   keys[0].name="NUMLABS";
   keys[0].type=GAL_TYPE_SIZE_T;
   keys[0].array=&p->numobjects;
-  gal_fits_key_read(p->objectsfile, p->cp.hdu, keys, 0, 0);
-  if(keys[0].status) /* status!=0: the key couldn't be read by CFITSIO. */
+  gal_fits_key_read(p->objectsfile, p->cp.hdu, keys, 0, 0, "--hdu");
+  if(keys[0].status) /* status!=0: the key didn't exist. */
     {
+      /* Get the maximum of the labels.*/
       tmp=gal_statistics_maximum(p->objects);
       p->numobjects=*((int32_t *)(tmp->array)); /*numobjects is int32_t.*/
+
+      /* In case the input is all blank, the maximum will be blank, so in
+         effect, there we no objects. */
+      if(p->numobjects==GAL_BLANK_INT32) p->numobjects=0;
       gal_data_free(tmp);
     }
 
-
   /* If there were no objects in the input, then inform the user with an
      error (it is pointless to build a catalog). */
   if(p->numobjects==0)
-    error(EXIT_FAILURE, 0, "no object labels (non-zero pixels) in "
-          "%s (hdu %s). To make a catalog, labeled regions must be "
-          "defined", p->objectsfile, p->cp.hdu);
+    error(EXIT_FAILURE, 0, "no object labels (non-zero and non-blank "
+          "pixels) in %s (hdu %s). To make a catalog, labeled regions "
+          "must be defined", p->objectsfile, p->cp.hdu);
 
 
   /* See if the labels image has blank pixels and set the flags
@@ -935,7 +1005,7 @@ ui_read_labels(struct mkcatalogparams *p)
       /* Read the clumps image. */
       p->clumps = gal_array_read_one_ch(p->usedclumpsfile, p->clumpshdu,
                                         NULL, p->cp.minmapsize,
-                                        p->cp.quietmmap);
+                                        p->cp.quietmmap, "--clumpshdu");
       p->clumps->ndim=gal_dimension_remove_extra(p->clumps->ndim,
                                                  p->clumps->dsize, NULL);
 
@@ -955,7 +1025,8 @@ ui_read_labels(struct mkcatalogparams *p)
       keys[0].name="CLUMPSN";               keys[1].name="NUMLABS";
       keys[0].type=GAL_TYPE_FLOAT32;        keys[1].type=GAL_TYPE_SIZE_T;
       keys[0].array=&p->clumpsn;            keys[1].array=&p->numclumps;
-      gal_fits_key_read(p->usedclumpsfile, p->clumpshdu, keys, 0, 0);
+      gal_fits_key_read(p->usedclumpsfile, p->clumpshdu, keys, 0, 0,
+                        "--clumpshdu");
       if(keys[0].status) p->clumpsn=NAN;
       if(keys[1].status) p->numclumps=ui_num_clumps(p);
 
@@ -1111,6 +1182,7 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
           case CCOL_NUM:              *values        = 1;          break;
           case CCOL_NUMXY:            *values        = 1;          break;
           case CCOL_SUM:              *values        = 1;          break;
+          case CCOL_SUMP2:            *values        = 1;          break;
           case CCOL_SUM_VAR:          *values = *std = 1;          break;
           case CCOL_SUM_VAR_NUM:      *values = *std = 1;          break;
           case CCOL_MEDIAN:           *values        = 1;          break;
@@ -1121,6 +1193,8 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
           case CCOL_SIGCLIPSTD:       *values        = 1;          break;
           case CCOL_RIV_NUM:          /* Only clump labels. */     break;
           case CCOL_RIV_SUM:          *values        = 1;          break;
+          case CCOL_RIV_MIN:          *values        = 1;          break;
+          case CCOL_RIV_MAX:          *values        = 1;          break;
           case CCOL_RIV_SUM_VAR:      *values = *std = 1;          break;
           case CCOL_VX:               *values        = 1;          break;
           case CCOL_VY:               *values        = 1;          break;
@@ -1298,7 +1372,8 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
                                               p->valueshdu,
                                               NULL, GAL_TYPE_FLOAT32,
                                               p->cp.minmapsize,
-                                              p->cp.quietmmap);
+                                              p->cp.quietmmap,
+                                              "--valueshdu");
       p->values->ndim=gal_dimension_remove_extra(p->values->ndim,
                                                  p->values->dsize, NULL);
 
@@ -1354,7 +1429,8 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
           p->sky=gal_array_read_one_ch_to_type(p->usedskyfile, p->skyhdu,
                                                NULL, GAL_TYPE_FLOAT32,
                                                p->cp.minmapsize,
-                                               p->cp.quietmmap);
+                                               p->cp.quietmmap,
+                                               "--skyhdu");
           p->sky->ndim=gal_dimension_remove_extra(p->sky->ndim,
                                                   p->sky->dsize, NULL);
 
@@ -1386,7 +1462,7 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
       p->std=gal_array_read_one_ch_to_type(p->usedstdfile, p->stdhdu,
                                            NULL, GAL_TYPE_FLOAT32,
                                            p->cp.minmapsize,
-                                           p->cp.quietmmap);
+                                           p->cp.quietmmap, "--stdhdu");
       p->std->ndim=gal_dimension_remove_extra(p->std->ndim,
                                               p->std->dsize, NULL);
 
@@ -1422,7 +1498,8 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
           /* Read the mask image. */
           p->upmask = gal_array_read_one_ch(p->upmaskfile, p->upmaskhdu,
                                             NULL, p->cp.minmapsize,
-                                            p->cp.quietmmap);
+                                            p->cp.quietmmap,
+                                            "--upmaskhdu");
           p->upmask->ndim=gal_dimension_remove_extra(p->upmask->ndim,
                                                      p->upmask->dsize,
                                                      NULL);
@@ -1480,7 +1557,8 @@ ui_preparations_read_keywords(struct mkcatalogparams *p)
           keys[0].name="MINSTD";          keys[1].name="MEDSTD";
           keys[0].type=GAL_TYPE_FLOAT32;  keys[1].type=GAL_TYPE_FLOAT32;
           keys[0].array=&minstd;          keys[1].array=&p->medstd;
-          gal_fits_key_read(p->usedstdfile, p->stdhdu, keys, 0, 0);
+          gal_fits_key_read(p->usedstdfile, p->stdhdu, keys, 0, 0,
+                            "--stdhdu");
 
           /* If the two keywords couldn't be read. We don't want to slow
              down the user for the median (which needs sorting). So we'll
@@ -1851,9 +1929,8 @@ ui_read_check_inputs_setup(int argc, char *argv[],
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
@@ -1890,9 +1967,6 @@ ui_read_check_inputs_setup(int argc, char *argv[],
       if(p->clumps)
         printf("  - Clumps:  %s (hdu: %s)\n", p->usedclumpsfile,
                p->clumpshdu);
-      if(p->relabclumps)
-        printf("  - RELABELED CLUMPS (no NUMLABS in original): %s\n",
-               p->relabclumps);
       if(p->values)
         printf("  - Values:  %s (hdu: %s)\n", p->usedvaluesfile,
                p->valueshdu);
@@ -1922,7 +1996,7 @@ ui_read_check_inputs_setup(int argc, char *argv[],
 
       if(p->upmaskfile)
         printf("  - Upper limit magnitude mask: %s (hdu: %s)\n",
-               p->upmaskfile, p->cp.hdu);
+               p->upmaskfile, p->upmaskhdu);
       if(p->upperlimit)
         {
           printf("  - Random number generator name: %s\n", p->rng_name);
@@ -1956,7 +2030,7 @@ ui_read_check_inputs_setup(int argc, char *argv[],
 void
 ui_free_report(struct mkcatalogparams *p, struct timeval *t1)
 {
-  size_t d;
+  size_t i, d;
 
   /* The temporary arrays for WCS coordinates. */
   if(p->wcs_vo ) gal_list_data_free(p->wcs_vo);
@@ -2009,6 +2083,16 @@ ui_free_report(struct mkcatalogparams *p, struct timeval *t1)
   if(p->upcheckout) free(p->upcheckout);
   gal_data_array_free(p->tiles, p->numobjects, 0);
 
+  /* The original clump labels. */
+  if(p->origclpid)
+    {
+      for(i=1;i<p->numobjects+1;++i)
+        if(p->origclpid[i])
+          free(p->origclpid[i]);
+      free(p->origclpid);
+    }
+  /**/
+
   /* If the Sky or its STD image were given in tiles, then we defined a
      tile structure to deal with them. The initialization of the tile
      structure is checked with its 'ndim' element. */
