@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -365,7 +365,8 @@ ui_parse_kernel(struct argp_option *option, char *arg,
               "astmkprof' command) for the meaning of the numbers");
 
       /* Read the parameters. */
-      kernel=gal_options_parse_list_of_numbers(arg, filename, lineno);
+      kernel=gal_options_parse_list_of_numbers(arg, filename, lineno,
+                                               GAL_TYPE_FLOAT64);
 
       /* Put the kernel dataset into the main program structure. */
       *(gal_data_t **)(option->value) = kernel;
@@ -519,10 +520,10 @@ ui_parse_coordinate_mode(struct argp_option *option, char *arg,
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
-/* Read and check ONLY the options. When arguments are involved, do the
+/* Check ONLY the options. When arguments are involved, do the
    check in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct mkprofparams *p)
+ui_check_only_options(struct mkprofparams *p)
 {
   size_t i;
 
@@ -546,13 +547,13 @@ ui_read_check_only_options(struct mkprofparams *p)
               "coordinate columns");
     }
 
-  /* The zeropoint magnitude is only necessary when 'mcolisbrightness' is
+  /* The zeropoint magnitude is only necessary when 'mcolissum' is
      not called.  */
-  if( p->mcolisbrightness==0 && isnan(p->zeropoint) )
+  if( p->mcolissum==0 && isnan(p->zeropoint) )
     error(EXIT_FAILURE, 0, "no zeropoint magnitude given. A zeropoint "
-          "magnitude is necessary when '--mcolisbrightness' is not "
-          "called (i.e., when the contents of '--mcol' must be "
-          "interpretted as a magnitude, not brightness).");
+          "magnitude is necessary when '--mcolissum' is not called (i.e., "
+          "when the contents of '--mcol' must be interpretted as a "
+          "magnitude, not brightness).");
 
   /* The kernel should always be normalized to 1.0. So '--magatpeak' should
      never be called with '--kernel'. */
@@ -597,7 +598,7 @@ ui_read_check_only_options(struct mkprofparams *p)
 
 
 /* Sanity check on options AND arguments. If only option values are to be
-   checked, use 'ui_read_check_only_options'. */
+   checked, use 'ui_check_only_options'. */
 static void
 ui_check_options_and_arguments(struct mkprofparams *p)
 {
@@ -636,7 +637,7 @@ ui_check_options_and_arguments(struct mkprofparams *p)
 
 
   /* Set the necessary output names. */
-  d0f1=gal_checkset_dir_0_file_1(p->cp.output, p->cp.dontdelete);
+  d0f1=gal_checkset_dir_0_file_1(&p->cp, p->cp.output, p->catname);
   if(d0f1)                        /* --output is a file name. */
     {
       p->mergedimgname=p->cp.output;
@@ -662,7 +663,7 @@ ui_check_options_and_arguments(struct mkprofparams *p)
   /* If a merged image is requested (or '--kernel' is called), then delete
      the final filename if it exists. */
   if(p->nomerged==0 && p->kernel)
-    gal_checkset_writable_remove(p->mergedimgname, p->cp.keep,
+    gal_checkset_writable_remove(p->mergedimgname, p->catname, p->cp.keep,
                                  p->cp.dontdelete);
 }
 
@@ -702,7 +703,7 @@ ui_read_cols_general(struct mkprofparams *p, gal_list_str_t *colstrs)
   lines=gal_options_check_stdin(p->catname, p->cp.stdintimeout, "input");
   cols=gal_table_read(p->catname, p->cp.hdu, lines, colstrs,
                       p->cp.searchin, p->cp.ignorecase, p->cp.numthreads,
-                      p->cp.minmapsize, p->cp.quietmmap, NULL);
+                      p->cp.minmapsize, p->cp.quietmmap, NULL, "--hdu");
   gal_list_str_free(lines, 1);
 
   /* The name of the input catalog is only for informative steps from now
@@ -919,7 +920,7 @@ ui_read_cols_2d(struct mkprofparams *p)
 
   /* Make sure flat profiles aren't given a value of zero. */
   counter=0;
-  if( !p->cp.quiet && (p->mforflatpix || p->mcolisbrightness) )
+  if( !p->cp.quiet && (p->mforflatpix || p->mcolissum) )
     for(i=0;i<p->num;++i)
       if( p->m[i]==0.0 && ( p->f[i]==PROFILE_POINT
                             || p->f[i]==PROFILE_FLAT
@@ -1279,7 +1280,7 @@ ui_prepare_columns(struct mkprofparams *p)
       p->n[0]  = n;
       p->p1[0] = 0.0f;
       p->q1[0] = 1.0f;
-      p->m[0]  = p->mcolisbrightness ? 1.0f : p->zeropoint;
+      p->m[0]  = p->mcolissum ? 1.0f : p->zeropoint;
       p->t[0]  = t;
       if(p->ndim==3)
         {
@@ -1335,7 +1336,7 @@ ui_prepare_columns(struct mkprofparams *p)
     {
       if(p->f[i]==PROFILE_CUSTOM_PROF)
         {
-          if(p->customprofname==NULL)
+          if(p->customtablename==NULL)
             error(EXIT_FAILURE, 0, "at least one custom profile "
                   "requested (first occurrence in row %zu), but no "
                   "file/table was given to the '--customtable' "
@@ -1520,9 +1521,10 @@ ui_prepare_canvas(struct mkprofparams *p)
          no merged image is desired, we just need the WCS information of
          the background image and the number of its dimensions. So
          'ndim==0' and what 'dsize' points to is irrelevant. */
-      tdsize=gal_fits_img_info_dim(p->backname, p->backhdu, &tndim);
+      tdsize=gal_fits_img_info_dim(p->backname, p->backhdu, &tndim,
+                                   "--backhdu");
       p->wcs=gal_wcs_read(p->backname, p->backhdu, p->cp.wcslinearmatrix,
-                          0, 0, &p->nwcs);
+                          0, 0, &p->nwcs, "--backhdu");
       tndim=gal_dimension_remove_extra(tndim, tdsize, p->wcs);
       free(tdsize);
       if(p->nomerged==0)
@@ -1678,7 +1680,7 @@ ui_prepare_canvas(struct mkprofparams *p)
       if(p->out->name) free(p->out->name);
       gal_checkset_allocate_copy("Mock profiles", &p->out->name);
       if(p->out->unit==NULL)
-        gal_checkset_allocate_copy("Brightness", &p->out->unit);
+        gal_checkset_allocate_copy("counts", &p->out->unit);
     }
 }
 
@@ -1733,7 +1735,7 @@ ui_finalize_coordinates(struct mkprofparams *p)
          conversion) and print a warning for those rows. IMPORTANT: we
          don't want to update 'p->num' just yet since 'flag' has the size
          of the pre-blank-removal rows. */
-      flag=gal_blank_remove_rows(coords, NULL);
+      flag=gal_blank_remove_rows(coords, NULL, 0);
       if(p->cp.quiet==0)
         {
           fl=flag->array;
@@ -1842,20 +1844,20 @@ ui_read_custom_table(struct mkprofparams *p)
   double *min, *max;
 
   /* Read the input radial table. */
-  cols=gal_table_read(p->customprofname, p->customprofhdu,
+  cols=gal_table_read(p->customtablename, p->customtablehdu,
                       NULL, NULL, p->cp.searchin, p->cp.ignorecase,
                       p->cp.numthreads, p->cp.minmapsize,
-                      p->cp.quietmmap, NULL);
+                      p->cp.quietmmap, NULL, "--customtablehdu");
 
   /* Make sure the table only has three columns. */
   if(gal_list_data_number(cols) != 3 )
     error(EXIT_FAILURE, 0, "%s: has %zu columns, but it should only "
-          "have three columns. Column1: the radial interval's lower "
+          "have three columns. Column 1: the radial interval's lower "
           "value. Column 2: the radial interval's higher value. "
           "Column 3: the value to use for pixels within that radius "
           "interval",
-          gal_fits_name_save_as_string(p->customprofname,
-                                       p->customprofhdu),
+          gal_fits_name_save_as_string(p->customtablename,
+                                       p->customtablehdu),
           gal_list_data_number(cols));
 
   /* Make sure none of the three columns are string type. */
@@ -1863,8 +1865,8 @@ ui_read_custom_table(struct mkprofparams *p)
       || cols->next->type==GAL_TYPE_STRING
       || cols->next->next->type==GAL_TYPE_STRING )
     error(EXIT_FAILURE, 0, "%s: the columns should only have numeric "
-          "data types", gal_fits_name_save_as_string(p->customprofname,
-                                                     p->customprofhdu));
+          "data types", gal_fits_name_save_as_string(p->customtablename,
+                                                     p->customtablehdu));
 
   /* Fill the final table as a double type. */
   p->custom=gal_data_copy_to_new_type(cols, GAL_TYPE_FLOAT64);
@@ -1884,8 +1886,8 @@ ui_read_custom_table(struct mkprofparams *p)
             "value %g). However, the first column is the lower-limit "
             "of the radial interval and the second column is the "
             "upper-limit. So the first column must have a lower value",
-            gal_fits_name_save_as_string(p->customprofname,
-                                         p->customprofhdu), i+1,
+            gal_fits_name_save_as_string(p->customtablename,
+                                         p->customtablehdu), i+1,
             min[i], max[i]);
 
   /* Check if the input table is regular and sorted (which can greatly
@@ -1943,7 +1945,7 @@ ui_read_ndim(struct mkprofparams *p)
             {
               /* Get the number of the background image's dimensions. */
               dsize=gal_fits_img_info_dim(p->backname, p->backhdu,
-                                          &p->ndim);
+                                          &p->ndim, "--backhdu");
               p->ndim=gal_dimension_remove_extra(p->ndim, dsize, NULL);
               free(dsize);
             }
@@ -1953,7 +1955,8 @@ ui_read_ndim(struct mkprofparams *p)
               p->out=gal_array_read_one_ch_to_type(p->backname, p->backhdu,
                                                    NULL, GAL_TYPE_FLOAT32,
                                                    p->cp.minmapsize,
-                                                   p->cp.quietmmap);
+                                                   p->cp.quietmmap,
+                                                   "--backhdu");
               p->out->ndim=gal_dimension_remove_extra(p->out->ndim,
                                                       p->out->dsize, NULL);
               p->ndim=p->out->ndim;
@@ -1997,7 +2000,7 @@ ui_preparations(struct mkprofparams *p)
   ui_prepare_columns(p);
 
   /* Read the radial table. */
-  if(p->customprofname) ui_read_custom_table(p);
+  if(p->customtablename) ui_read_custom_table(p);
 
   /* If the kernel option was given, some parameters need to be
      over-written: */
@@ -2160,9 +2163,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkprofparams *p)
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
