@@ -5,7 +5,7 @@
 # Original author:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
-# Copyright (C) 2023-2023 Free Software Foundation, Inc.
+# Copyright (C) 2023-2024 Free Software Foundation, Inc.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -109,7 +109,7 @@ EOF
 print_version() {
      cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2023-2023 Free Software Foundation, Inc.
+Copyright (C) 2023-2024 Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -141,12 +141,15 @@ Paper describing this script
             eid = {211},
           pages = {211},
             doi = {10.3847/2515-5172/acfc44},
+archivePrefix = {arXiv},
+       eprint = {2310.15006},
+ primaryClass = {astro-ph.IM},
          adsurl = {https://ui.adsabs.harvard.edu/abs/2023RNAAS...7..211A},
         adsnote = {Provided by the SAO/NASA Astrophysics Data System}
   }
 
-First paper introducing Gnuastro (currently main citation)
-----------------------------------------------------------
+Paper introducing Gnuastro (currently main citation)
+----------------------------------------------------
   @ARTICLE{gnuastro,
      author = {{Akhlaghi}, M. and {Ichikawa}, T.},
       title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
