@@ -8,7 +8,7 @@
 # Contributing author(s):
 #   Mohammad Akhlaghi <mohammad@akhlaghi.org>
 #   Carlos Morales-Socorro <cmorsoc@gmail.com>
-# Copyright (C) 2021-2023 Free Software Foundation, Inc.
+# Copyright (C) 2021-2024 Free Software Foundation, Inc.
 #
 # Gnuastro is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -122,7 +122,7 @@ EOF
 print_version() {
     cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2020-2023 Free Software Foundation, Inc.
+Copyright (C) 2020-2024 Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -472,7 +472,7 @@ xdiff=$(astarithmetic $xcenter float32 $xpsfcenter float32 - --quiet)
 ydiff=$(astarithmetic $ycenter float32 $ypsfcenter float32 - --quiet)
 
 innertranslated=$tmpdir/"$bname_inner"_translated.fits
-astwarp $inner --translate=$xdiff,$ydiff \
+astwarp $inner --hdu=$innerhdu --translate=$xdiff,$ydiff \
         --output=$innertranslated $quiet
 
 
