@@ -12,11 +12,11 @@
 # Run with '--help' for more information.
 #
 # Current maintainer:
-#     2022-2023 Sepideh Eskandarlou <sepideh.eskandarlou@gmail.com>
+#     Sepideh Eskandarlou <sepideh.eskandarlou@gmail.com>
 # Contributing authors:
-#     2022-2023 Mohammad Akhlaghi <mohammad@akhlaghi.org>
-#     2022-2023 Raul Infante-Sainz <infantesainz@gmail.com>
-# Copyright (C) 2022-2023 Free Software Foundation, Inc.
+#     Mohammad Akhlaghi <mohammad@akhlaghi.org>
+#     Raul Infante-Sainz <infantesainz@gmail.com>
+# Copyright (C) 2022-2024 Free Software Foundation, Inc.
 #
 # Gnuastro is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -149,7 +149,7 @@ EOF
 print_version() {
      cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2022-2023 Free Software Foundation, Inc.
+Copyright (C) 2022-2024 Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -158,9 +158,6 @@ Written/developed by Sepideh Eskandarlou et al.
 EOF
 }
 
-
-
-
 # Output of `--cite':
 print_citation() {
     empty="" # needed for the ascii art!
@@ -172,8 +169,29 @@ Citations and acknowledgement are vital for the continued work on Gnuastro.
 
 Please cite the following record(s) and add the acknowledgement statement below in your work to support us. Please note that different Gnuastro programs may have different corresponding papers. Hence, please check all the programs you used. Don't forget to also include the version as shown above for reproducibility.
 
-First paper introducing Gnuastro
---------------------------------
+Paper introducing this script
+-----------------------------
+  @ARTICLE{astscrip-zeropoint,
+         author = {{Eskandarlou}, Sepideh and {Akhlaghi}, Mohammad and {Infante-Sainz}, Ra{\'u}l and {Saremi}, Elham and {Raji}, Samane and {Sharbaf}, Zahra and {Golini}, Giulia and {Ghaffari}, Zohreh and {Knapen}, Johan H.},
+          title = "{Gnuastro: Estimating the Zero-point Magnitude in Astronomical Imaging}",
+        journal = {Research Notes of the American Astronomical Society},
+       keywords = {Flux calibration, Astronomy software, Open source software, Astronomical techniques, 544, 1855, 1866, 1684, Astrophysics - Instrumentation and Methods for Astrophysics},
+           year = 2023,
+          month = dec,
+         volume = {7},
+         number = {12},
+            eid = {269},
+          pages = {269},
+            doi = {10.3847/2515-5172/ad14f4},
+  archivePrefix = {arXiv},
+         eprint = {2312.04263},
+   primaryClass = {astro-ph.IM},
+         adsurl = {https://ui.adsabs.harvard.edu/abs/2023RNAAS...7..269E},
+        adsnote = {Provided by the SAO/NASA Astrophysics Data System}
+  }
+
+Paper introducing Gnuastro (currently main citation)
+----------------------------------------------------
   @ARTICLE{gnuastro,
      author = {{Akhlaghi}, M. and {Ichikawa}, T.},
       title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
