@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2023 Free Software Foundation, Inc.
+Copyright (C) 2018-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -225,10 +225,10 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
-/* Read and check ONLY the options. When arguments are involved, do the
-   check in 'ui_check_options_and_arguments'. */
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct segmentparams *p)
+ui_check_only_options(struct segmentparams *p)
 {
   /* If the full area is to be used as a single detection, we can't find
      the S/N value from the un-detected regions, so the user must have
@@ -414,17 +414,21 @@ ui_prepare_inputs(struct segmentparams *p)
   p->input = gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu,
                                            NULL, GAL_TYPE_FLOAT32,
                                            p->cp.minmapsize,
-                                           p->cp.quietmmap);
+                                           p->cp.quietmmap, "--hdu");
   p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu,
                                p->cp.wcslinearmatrix, 0, 0,
-                               &p->input->nwcs);
+                               &p->input->nwcs, "--hdu");
   p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
                                             p->input->dsize,
                                             p->input->wcs);
 
-  /* Set the name. */
+
+  /* Set the name (the sky will be subtracted here in 'ui.c' before any
+     processing. We want the name to be similar to be descriptive and
+     remind the user that the sky (which may be given to Segment) has been
+     subtracted (this is also the same name in NoiseChisel's output). */
   if(p->input->name) free(p->input->name);
-  gal_checkset_allocate_copy("INPUT", &p->input->name);
+  gal_checkset_allocate_copy("INPUT-NO-SKY", &p->input->name);
 
 
   /* Check for blank values to help later processing.  */
@@ -445,7 +449,7 @@ ui_prepare_inputs(struct segmentparams *p)
       p->conv = gal_array_read_one_ch_to_type(p->convolvedname, p->chdu,
                                               NULL, GAL_TYPE_FLOAT32,
                                               p->cp.minmapsize,
-                                              p->cp.quietmmap);
+                                              p->cp.quietmmap, "--chdu");
       p->conv->ndim=gal_dimension_remove_extra(p->conv->ndim,
                                                p->conv->dsize,
                                                p->conv->wcs);
@@ -467,7 +471,7 @@ ui_prepare_inputs(struct segmentparams *p)
       /* Read the dataset into memory. */
       p->olabel = gal_array_read_one_ch(p->useddetectionname, p->dhdu,
                                         NULL, p->cp.minmapsize,
-                                        p->cp.quietmmap);
+                                        p->cp.quietmmap, "--dhdu");
       p->olabel->ndim=gal_dimension_remove_extra(p->olabel->ndim,
                                                  p->olabel->dsize, NULL);
       if( gal_dimension_is_different(p->input, p->olabel) )
@@ -564,7 +568,7 @@ ui_prepare_kernel(struct segmentparams *p)
           /* Read the kernel into memory. */
           p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
                                              p->cp.minmapsize,
-                                             p->cp.quietmmap);
+                                             p->cp.quietmmap, "--khdu");
           p->kernel->ndim=gal_dimension_remove_extra(p->kernel->ndim,
                                                      p->kernel->dsize,
                                                      NULL);
@@ -639,12 +643,12 @@ ui_prepare_tiles(struct segmentparams *p)
     {
       /* Large tiles. */
       check=gal_tile_block_check_tiles(ltl->tiles);
-      gal_fits_img_write(check, tl->tilecheckname, NULL, PROGRAM_NAME);
+      gal_fits_img_write(check, tl->tilecheckname, NULL, 0);
       gal_data_free(check);
 
       /* Small tiles. */
       check=gal_tile_block_check_tiles(tl->tiles);
-      gal_fits_img_write(check, tl->tilecheckname, NULL, PROGRAM_NAME);
+      gal_fits_img_write(check, tl->tilecheckname, NULL, 0);
       gal_data_free(check);
 
       /* If 'continueaftercheck' hasn't been called, abort NoiseChisel. */
@@ -775,7 +779,8 @@ ui_read_std_and_sky(struct segmentparams *p)
       /* Read the STD image. */
       p->std=gal_array_read_one_ch_to_type(p->usedstdname, p->stdhdu,
                                            NULL, GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize, p->cp.quietmmap);
+                                           p->cp.minmapsize,
+                                           p->cp.quietmmap, "--stdhdu");
       p->std->ndim=gal_dimension_remove_extra(p->std->ndim,
                                               p->std->dsize, NULL);
 
@@ -802,7 +807,8 @@ ui_read_std_and_sky(struct segmentparams *p)
       keys[1].array=&p->minstd;     keys[1].name="MINSTD";
       keys[2].array=&p->maxstd;     keys[2].name="MAXSTD";
       keys[0].type=keys[1].type=keys[2].type=GAL_TYPE_FLOAT32;
-      gal_fits_key_read(p->usedstdname, p->stdhdu, keys, 0, 0);
+      gal_fits_key_read(p->usedstdname, p->stdhdu, keys, 0, 0,
+                        "--stdhdu");
       if(keys[0].status) p->medstd=NAN;
       if(keys[1].status) p->minstd=NAN;
       if(keys[2].status) p->maxstd=NAN;
@@ -835,13 +841,14 @@ ui_read_std_and_sky(struct segmentparams *p)
           /* Read the Sky dataset. */
           sky=gal_array_read_one_ch_to_type(p->skyname, p->skyhdu,
                                             NULL, GAL_TYPE_FLOAT32,
-                                            p->cp.minmapsize, p->cp.quietmmap);
+                                            p->cp.minmapsize,
+                                            p->cp.quietmmap, "--skyhdu");
           sky->ndim=gal_dimension_remove_extra(sky->ndim, sky->dsize,
                                                NULL);
 
           /* Check its size. */
-          ui_check_size(p->input, sky, tl->tottiles, p->inputname, p->cp.hdu,
-                        p->skyname, p->skyhdu);
+          ui_check_size(p->input, sky, tl->tottiles, p->inputname,
+                        p->cp.hdu, p->skyname, p->skyhdu);
         }
 
       /* Subtract the sky from the input. */
@@ -941,9 +948,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct segmentparams *p)
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
