@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -35,6 +35,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/txt.h>
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/list.h>
 #include <gnuastro/table.h>
 #include <gnuastro/qsort.h>
 #include <gnuastro/pointer.h>
@@ -52,12 +53,30 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 
+
+
 /**************************************************************/
 /********     Selecting and ordering of columns      **********/
 /**************************************************************/
+static void
+table_error_no_column(char *optionname, char *id)
+{
+  error(EXIT_FAILURE, 0, "no column could be found with the '%s' "
+        "identifier (given to '%s'). The value to this option can "
+        "either be a column name or counter (counting from 1). For "
+        "more on how to select columns in Gnuastro, please run the "
+        "command below (press 'q' to come back to the command-line):\n\n"
+        "    info gnuastro \"selecting table columns\"\n",
+        id, optionname);
+}
+
+
+
+
+
 static void
 table_apply_permutation(gal_data_t *table, size_t *permutation,
-                        size_t newsize, int inverse)
+                        size_t permsize, int inverse)
 {
   gal_data_t *tmp;
 
@@ -65,12 +84,24 @@ table_apply_permutation(gal_data_t *table, size_t *permutation,
     {
       /* Apply the permutation. */
       if(inverse)
-        gal_permutation_apply_inverse(tmp, permutation);
+        {
+          if(tmp->ndim==1)
+            gal_permutation_apply_inverse(tmp, permutation);
+          else
+            error(EXIT_FAILURE, 0, "%s: inverse permutation on "
+                  "vector columns is not yet supported. Please "
+                  "get in touch with us at '%s' to add this "
+                  "feature", __func__, PACKAGE_BUGREPORT);
+        }
       else
-        gal_permutation_apply(tmp, permutation);
+        {
+          if(tmp->ndim==1) gal_permutation_apply(tmp, permutation);
+          else    gal_permutation_apply_onlydim0(tmp, permutation);
+        }
 
       /* Correct the size. */
-      tmp->size=tmp->dsize[0]=newsize;
+      tmp->dsize[0]=permsize;
+      tmp->size = tmp->dsize[0] * (tmp->ndim==1 ? 1 : tmp->dsize[1]);
     }
 }
 
@@ -83,16 +114,18 @@ table_bring_to_top(gal_data_t *table, gal_data_t *rowids)
 {
   char **strarr;
   gal_data_t *col;
-  size_t i, *ids=rowids->array;
+  size_t i, n, *ids=rowids->array;
 
-  /* Make sure the rowids are sorted by increasing index. */
+  /* Make sure the rowids are sorted by increasing index.
   gal_statistics_sort_increasing(rowids);
+  */
 
   /* Go over each column and move the desired rows to the top. */
   for(col=table;col!=NULL;col=col->next)
     {
-      /* For easy operation if the column is a string. */
+      /* For easy operation if the column is a string or vector. */
       strarr = col->type==GAL_TYPE_STRING ? col->array : NULL;
+      n = col->ndim==1 ? 1 : col->dsize[1];
 
       /* Move the desired rows up to the top. */
       for(i=0;i<rowids->size;++i)
@@ -107,9 +140,11 @@ table_bring_to_top(gal_data_t *table, gal_data_t *rowids)
                 strarr[ ids[i] ]=NULL;
               }
             else
-              memcpy(gal_pointer_increment(col->array, i,      col->type),
-                     gal_pointer_increment(col->array, ids[i], col->type),
-                     gal_type_sizeof(col->type));
+              {
+                memcpy(gal_pointer_increment(col->array, i*n,     col->type),
+                       gal_pointer_increment(col->array, ids[i]*n,col->type),
+                       n * gal_type_sizeof(col->type));
+              }
           }
 
       /* For string arrays, free the pointers of the remaining rows. */
@@ -119,9 +154,12 @@ table_bring_to_top(gal_data_t *table, gal_data_t *rowids)
 
       /* Correct the size (this should be after freeing of the string
          pointers. */
-      col->size = col->dsize[0] = rowids->size;
-    }
+      col->dsize[0] = rowids->size;
+      col->size = col->dsize[0] * n;
 
+      /* If there is no elements, free 'array' and set it to NULL. */
+      if(col->size==0 && col->array) { free(col->array); col->array=NULL; }
+    }
 }
 
 
@@ -161,7 +199,7 @@ table_selection_range(struct tableparams *p, gal_data_t *col)
   gal_data_free(tmp);
 
   /* Find all the elements outside this range (smaller than the minimum,
-     larger than the maximum or blank) as separate binary flags.. */
+     larger than the maximum or blank) as separate binary flags. */
   ltmin=gal_arithmetic(GAL_ARITHMETIC_OP_LT, 1, numok, col, min);
   gemax=gal_arithmetic(GAL_ARITHMETIC_OP_GE, 1, numok, col, max);
 
@@ -238,7 +276,7 @@ table_selection_polygon(struct tableparams *p, gal_data_t *col1,
                      NULL, NULL, NULL);
   oarr=out->array;
 
-  /* Loop through all the rows in the given columns and check the points.*/
+  /* Loop through all the rows in the given columns and check the points. */
   for(i=0; i<col1->size; i++)
     {
       /* Read the column values as a double. */
@@ -331,14 +369,14 @@ table_selection_equal_or_notequal(struct tableparams *p, gal_data_t *col,
      elements. */
   for(i=0;i<arg->size;++i)
     {
-      /* Write the value  */
+      /* Write the value. */
       if(col->type==GAL_TYPE_STRING)
         eq=table_selection_string_eq_ne(col, strarr[i], e0n1);
       else
         {
           /* Allocate the value dataset. */
-          value=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &one, NULL, 0, -1, 1,
-                               NULL, NULL, NULL);
+          value=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &one, NULL, 0,
+                               -1, 1, NULL, NULL, NULL);
           varr=value->array;
 
           /* Read the stored string as a float64. */
@@ -374,7 +412,7 @@ table_selection_equal_or_notequal(struct tableparams *p, gal_data_t *col,
 
   /* Move the main pointer to the next possible call of the given
      option. Note that 'arg' already points to 'p->equal' or 'p->notequal',
-     so it will automatically be freed with the next step.*/
+     so it will automatically be freed with the next step. */
   if(e0n1) p->notequal=p->notequal->next;
   else     p->equal=p->equal->next;
 
@@ -392,23 +430,47 @@ static void
 table_select_by_value(struct tableparams *p)
 {
   gal_data_t *rowids;
+  size_t i, *s, ngood=0;
   struct list_select *tmp;
   uint8_t *u, *uf, *ustart;
-  size_t i, *s, ngood=0;
   int inplace=GAL_ARITHMETIC_FLAG_INPLACE;
-  gal_data_t *mask, *blmask, *addmask=NULL;
+  gal_data_t *mask, *col, *blmask, *addmask=NULL;
 
   /* It may happen that the input table is empty! In such cases, just
      return and don't bother with this step. */
-  if(p->table->dsize==NULL) return;
+  if(p->table->size==0 || p->table->array==NULL || p->table->dsize==NULL)
+    return;
 
   /* Allocate datasets for the necessary numbers and write them in. */
-  mask=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, p->table->dsize, NULL, 1,
-                      p->cp.minmapsize, p->cp.quietmmap, NULL, NULL, NULL);
+  mask=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &p->table->dsize[0],
+                      NULL, 1, p->cp.minmapsize, p->cp.quietmmap,
+                      NULL, NULL, NULL);
 
   /* Go over each selection criteria and remove the necessary elements. */
   for(tmp=p->selectcol;tmp!=NULL;tmp=tmp->next)
     {
+      /* Make sure the selection column isn't a vector column. */
+      if(tmp->col->ndim!=1)
+        error(EXIT_FAILURE, 0, "row selection by value (for example with "
+              "'--range', '--inpolygon', '--equal' or '--noblank') is "
+              "currently not available for vector columns. If you need "
+              "this feature, please get in touch with us at '%s' to add "
+              "it", PACKAGE_BUGREPORT);
+
+      /* Make sure all the to-be-used table columns at this point have the
+         same number of rows as the selection column (can be different when
+         transposing). */
+      for(col=p->table;col!=NULL;col=col->next)
+        if(tmp->col->dsize[0]!=col->dsize[0])
+          error(EXIT_FAILURE, 0, "the number of rows in the column given "
+                "for selection by value (for example '--range' or '--equal') "
+                "is not the same as the number of rows in the table when "
+                "they are applied. This may happen due to operators like "
+                "'--transpose'. In such cases, you probably want row-based "
+                "operators to have precedence over column-based operators. "
+                "If this is your case, please call '--rowfirst'");
+
+      /* Do the specific type of selection. */
       switch(tmp->type)
         {
         case SELECT_TYPE_RANGE:
@@ -432,7 +494,8 @@ table_select_by_value(struct tableparams *p)
           break;
 
         case SELECT_TYPE_NOBLANK:
-          addmask = gal_arithmetic(GAL_ARITHMETIC_OP_ISBLANK, 1, 0, tmp->col);
+          addmask = gal_arithmetic(GAL_ARITHMETIC_OP_ISBLANK, 1, 0,
+                                   tmp->col);
           break;
 
         default:
@@ -445,7 +508,8 @@ table_select_by_value(struct tableparams *p)
       /* Remove any blank elements (incase we are on a noblank column. */
       if(tmp->type!=SELECT_TYPE_NOBLANK && gal_blank_present(tmp->col, 1))
         {
-          blmask = gal_arithmetic(GAL_ARITHMETIC_OP_ISBLANK, 1, 0, tmp->col);
+          blmask = gal_arithmetic(GAL_ARITHMETIC_OP_ISBLANK, 1, 0,
+                                  tmp->col);
           addmask=gal_arithmetic(GAL_ARITHMETIC_OP_OR, 1, inplace,
                                  addmask, blmask);
           gal_data_free(blmask);
@@ -460,7 +524,7 @@ table_select_by_value(struct tableparams *p)
            float *f=ref->array;
            uint8_t *m=mask->array;
            uint8_t *u=addmask->array, *uf=u+addmask->size;
-           printf("\n\nInput column: %s\n", ref->name ? ref->name : "No Name");
+           printf("\n\nInput column: %s\n", ref->name?ref->name:"No Name");
            printf("Range: %g, %g\n", rarr[0], rarr[1]);
            printf("%-20s%-20s%-20s\n", "Value", "This mask",
            "Including previous");
@@ -514,14 +578,16 @@ static void
 table_sort(struct tableparams *p)
 {
   gal_data_t *perm;
-  size_t c=0, *s, *sf;
+  size_t c=0, *s, *sf, dsize0=p->table->dsize[0];
   int (*qsortfn)(const void *, const void *)=NULL;
 
   /* In case there are no columns to sort, skip this function. */
-  if(p->table->size==0) return;
+  if(p->table->size==0 || p->table->array==NULL || p->table->dsize==NULL)
+    return;
 
-  /* Allocate the permutation array and fill it. */
-  perm=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, p->table->dsize, NULL, 0,
+  /* Allocate the permutation array and fill it. Note that we need 'dsize0'
+     because the first column may be a vector column (which is 2D). */
+  perm=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &dsize0, NULL, 0,
                       p->cp.minmapsize, p->cp.quietmmap, NULL, NULL, NULL);
   sf=(s=perm->array)+perm->size; do *s=c++; while(++s<sf);
 
@@ -616,8 +682,7 @@ table_random_rows(gal_data_t *table, gsl_rng *rng, size_t numrandom,
   size_t i, j, *ids, ind;
 
   /* Sanity check. */
-  if(numrandom>table->size)
-    return EXIT_FAILURE;
+  if(numrandom>table->size) return EXIT_FAILURE;
 
   /* Allocate space for the list of rows to use. */
   rowids=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &numrandom, NULL, 0,
@@ -627,7 +692,7 @@ table_random_rows(gal_data_t *table, gsl_rng *rng, size_t numrandom,
   ids=rowids->array;
   for(i=0;i<numrandom;++i)
     {
-      /* Select a random index and make sure its new. */
+      /* Select a random index and make sure it is new. */
       bad=1;
       while(bad)
         {
@@ -655,18 +720,26 @@ table_select_by_position(struct tableparams *p)
 {
   char **strarr;
   gal_data_t *col;
-  size_t i, start, end;
+  size_t i, start, end, nelem;
   double *darr = p->rowrange ? p->rowrange->array : NULL;
 
+  /* If the table is already empty, then don't bother continuing. */
+  if(p->table->size==0 || p->table->array==NULL || p->table->dsize==NULL)
+    return;
+
   /* If the head or tail values are given and are larger than the number of
      rows, just set them to the number of rows (print the all the final
      rows). This is how the 'head' and 'tail' programs of GNU Coreutils
-     operate. */
-  p->head = ( ((p->head!=GAL_BLANK_SIZE_T) && (p->head > p->table->size))
-              ? p->table->size
+     operate. Note that some columns may be vector (multi-value per
+     column), in this case, they will be 2D. So we should use 'dsize[0]'
+     for the generic way to find the number of rows. */
+  p->head = ( ( (p->head!=GAL_BLANK_SIZE_T)
+                && (p->head > p->table->dsize[0]) )
+              ? p->table->dsize[0]
               : p->head );
-  p->tail = ( ((p->tail!=GAL_BLANK_SIZE_T) && (p->tail > p->table->size))
-              ? p->table->size
+  p->tail = ( ( (p->tail!=GAL_BLANK_SIZE_T)
+                && (p->tail > p->table->dsize[0]) )
+              ? p->table->dsize[0]
               : p->tail );
 
   /* Random row selection (by position, not value). This step is
@@ -689,24 +762,29 @@ table_select_by_position(struct tableparams *p)
      rows until this point. */
   if(p->rowrange)
     {
-      if(darr[0]>=p->table->size)
+      if(darr[0]>=p->table->dsize[0])
         error(EXIT_FAILURE, 0, "the first value to '--rowrange' (%g) "
               "is larger than the number of rows (%zu)",
-              darr[0]+1, p->table->size);
-      else if( darr[1]>=p->table->size )
+              darr[0]+1, p->table->dsize[0]);
+      else if( darr[1]>=p->table->dsize[0] )
         error(EXIT_FAILURE, 0, "the second value to '--rowrange' (%g) "
               "is larger than the number of rows (%zu)",
-              darr[1]+1, p->table->size);
+              darr[1]+1, p->table->dsize[0]);
     }
 
   /* Go over all the columns and make the necessary corrections. */
-  for(col=p->table;col!=NULL;col=col->next)
+  for(col=p->table; col!=NULL; col=col->next)
     {
+      /* Set the increment (number of elements in this column). For vector
+         columns, this is the number of elements in each row, and for
+         normal columns, this is 1. */
+      nelem=col->size/col->dsize[0];
+
       /* FOR STRING: we'll need to free the individual strings that will
          not be used (outside the allocated array directly
          'gal_data_t'). We don't have to worry about the space for the
          actual pointers (they will be free'd by 'free' in any case, since
-         they are in the initially allocated array).*/
+         they are in the initially allocated array). */
       if(col->type==GAL_TYPE_STRING)
         {
           /* Parse the rows and free extra pointers. */
@@ -718,7 +796,7 @@ table_select_by_position(struct tableparams *p)
                  it starts from 0). */
               start = darr[0];
               end   = darr[1];
-              for(i=0;i<p->table->size;++i)
+              for(i=0;i<p->table->dsize[0];++i)
                 if(i<start || i>end) { free(strarr[i]); strarr[i]=NULL; }
             }
           else
@@ -727,8 +805,8 @@ table_select_by_position(struct tableparams *p)
                  space of each string. */
               start = p->head!=GAL_BLANK_SIZE_T ? p->head : 0;
               end   = ( p->head!=GAL_BLANK_SIZE_T
-                        ? p->table->size
-                        : p->table->size - p->tail );
+                        ? p->table->dsize[0]
+                        : p->table->dsize[0] - p->tail );
               for(i=start; i<end; ++i) { free(strarr[i]); strarr[i]=NULL; }
             }
         }
@@ -737,10 +815,11 @@ table_select_by_position(struct tableparams *p)
       if(p->rowrange)
         {
           /* Move the values up to the top and correct the size. */
-          col->size=darr[1]-darr[0]+1;
+          col->dsize[0]=darr[1]-darr[0]+1;
           memmove(col->array,
-                  gal_pointer_increment(col->array, darr[0], col->type),
-                  (darr[1]-darr[0]+1)*gal_type_sizeof(col->type));
+                  gal_pointer_increment(col->array, darr[0]*nelem,
+                                        col->type),
+                  (darr[1]-darr[0]+1)*nelem*gal_type_sizeof(col->type));
         }
       else
         {
@@ -749,16 +828,19 @@ table_select_by_position(struct tableparams *p)
              safe with overlap. */
           if(p->tail!=GAL_BLANK_SIZE_T)
             memmove(col->array,
-                    gal_pointer_increment(col->array, col->size - p->tail,
+                    gal_pointer_increment(col->array,
+                                          (col->dsize[0]-p->tail)*nelem,
                                           col->type),
-                    p->tail*gal_type_sizeof(col->type));
+                    p->tail*nelem*gal_type_sizeof(col->type));
 
-          /* In any case (head or tail), the new number of column elements
-             is the given value. */
-          col->size = col->dsize[0] = ( p->head!=GAL_BLANK_SIZE_T
-                                        ? p->head
-                                        : p->tail );
+          /* In any case (head or tail), the new number of rows, then
+             update the total number of elements (may be vector). */
+          col->dsize[0] = p->head!=GAL_BLANK_SIZE_T ? p->head : p->tail;
         }
+
+      /* The 'dsize[0]' component was set above, we should not update the
+         total size. */
+      col->size = col->dsize[0] * (col->ndim==1 ? 1 : col->dsize[1]);
     }
 }
 
@@ -796,7 +878,7 @@ table_catcolumn(struct tableparams *p)
       tocat=gal_table_read(filell->v, hdu, NULL, p->catcolumns,
                            cp->searchin, cp->ignorecase,
                            cp->numthreads, cp->minmapsize,
-                           p->cp.quietmmap, NULL);
+                           p->cp.quietmmap, NULL, "--catcolumnhdu");
 
       /* Check the number of rows. */
       if(tocat->dsize[0]!=p->table->dsize[0])
@@ -832,7 +914,7 @@ table_catcolumn(struct tableparams *p)
                   }
             }
 
-      /* Find the final column of the main table and add this table.*/
+      /* Find the final column of the main table and add this table. */
       final=gal_list_data_last(p->table);
       final->next=tocat;
       ++counter;
@@ -843,6 +925,171 @@ table_catcolumn(struct tableparams *p)
 
 
 
+static void
+table_transpose(struct tableparams *p)
+{
+  gal_data_t *tmp;
+  size_t refwidth=p->table->dsize[1];
+
+  /* Temporary error. */
+  if(p->colpack)
+    error(EXIT_FAILURE, 0, "'--transpose' is currently not supported "
+          "with Column Arithmetic. Please pipe the transposed table "
+          "into a new 'asttable' command for doing column arithmetic "
+          "on the new columns");
+
+  /* Basic sanity checks and counting of final output columns. */
+  for(tmp=p->table; tmp!=NULL; tmp=tmp->next)
+    {
+      /* Inputs should be vectors. */
+      if(tmp->ndim!=2)
+        error(EXIT_FAILURE, 0, "only vector columns should be present "
+              "in the table when using '--transpose'");
+      if(tmp->dsize[1] != refwidth)
+        error(EXIT_FAILURE, 0, "all vector columns given to '--transpose' "
+              "should have the same length (number of tokens/elements)");
+
+      /* Apply the transposition. */
+      gal_permutation_transpose_2d(tmp);
+
+      /* Remove extra dimensions if necessary. */
+      if(tmp->dsize[1]==1) tmp->ndim=1;
+    }
+}
+
+
+
+
+
+static void
+table_fromvector(struct tableparams *p)
+{
+  size_t i, *iarr;
+  gal_list_sizet_t *indexs=NULL;
+  gal_data_t *tmp, *vector=NULL, *ext;
+
+  /* Parse the values given to this option. */
+  for(tmp=p->fromvector;tmp!=NULL;tmp=tmp->next)
+    {
+      /* Extract the name and element counters. */
+      vector=gal_list_data_select_by_id(p->table, tmp->name, NULL);
+      if(vector==NULL) table_error_no_column("--fromvector", tmp->name);
+
+      /* If we don't need this vector column any more, remove it from the
+         table. */
+      if(p->keepvectfin==0) gal_list_data_remove(&p->table, vector);
+
+      /* Make sure the selected column is actually a vector. */
+      if(vector->ndim!=2)
+        error(EXIT_FAILURE, 0, "column '%s' (given to '--fromvector') "
+              "is not a vector", tmp->name);
+
+      /* Loop over the values and make sure they are within the range. */
+      indexs=NULL;
+      iarr=tmp->array;
+      for(i=0;i<tmp->size;++i)
+        {
+          /* Check if it is reasonable. */
+          if(iarr[i]>vector->dsize[1])
+            error(EXIT_FAILURE, 0, "column '%s' (given to "
+                  "'--fromvector') only has a length of %zu, but you "
+                  "have asked for element %zu", tmp->name,
+                  vector->dsize[1], iarr[i]);
+
+          /* Make sure the user didn't give a value of 0. */
+          if(iarr[i]==0)
+            error(EXIT_FAILURE, 0, "integers given to '--fromvector' "
+                  "must be larger than 1, but you have given '0'");
+
+          /* Add it to the list of indexs. Note that the user has given a
+             "counter" (starting from 1), while we want an "index"
+             (counting from 0). */
+          gal_list_sizet_add(&indexs, iarr[i]-1);
+        }
+
+      /* Reverse the list of indexes to be the same as the requested. */
+      gal_list_sizet_reverse(&indexs);
+
+      /* Extract the columns and append them to the end of the table. */
+      ext=gal_table_col_vector_extract(vector, indexs);
+      gal_list_data_last(p->table)->next=ext;
+
+      /* Clean up (remove the vector column, if requested). */
+      gal_list_sizet_free(indexs);
+      if(p->keepvectfin==0) gal_data_free(vector);
+    }
+}
+
+
+
+
+
+static void
+table_tovector(struct tableparams *p)
+{
+  size_t i;
+  char **strarr;
+  gal_list_str_t *tstr;
+  gal_data_t *ids, *col, *tcol, *list, *vector, **torm=NULL;
+
+  /* Loop over all the calls to this option. */
+  for(tstr=p->tovector;tstr!=NULL;tstr=tstr->next)
+    {
+      /* Extract the separate csv. */
+      ids=gal_options_parse_csv_strings_to_data(tstr->v, NULL, 0);
+
+      /* Allocate an array of dataset pointers to keep the columns that
+         should be removed. */
+      if(p->keepvectfin==0)
+        {
+          errno=0;
+          torm=malloc(ids->size*sizeof *torm);
+          if(torm==NULL)
+            error(EXIT_FAILURE, errno, "%s: allocating %zu bytes",
+                  __func__, ids->size*sizeof *torm);
+        }
+
+      /* Parse the given values, and extract them from the table. */
+      list=NULL;
+      strarr=ids->array;
+      for(i=0;i<ids->size;++i)
+        {
+          /* Extract this column. */
+          tcol=gal_list_data_select_by_id(p->table, strarr[i], NULL);
+          if(tcol==NULL) table_error_no_column("--tovector", strarr[i]);
+
+          /* Keep a copy of the column and put it in the list of columns to
+             add. */
+          col=gal_data_copy(tcol); col->next=NULL;
+          gal_list_data_add(&list, col);
+
+          /* Keep the pointer to this column for removal (if necessary). */
+          if(p->keepvectfin==0) torm[i]=tcol;
+        }
+
+      /* Reverse the list to be in the same order as the input, and convert
+         it to a vector. */
+      gal_list_data_reverse(&list);
+      vector=gal_table_cols_to_vector(list);
+      gal_list_data_free(list);
+
+      /* Add this vector column to the output. */
+      gal_list_data_last(p->table)->next=vector;
+
+      /* Free the input columns if the user wanted to. */
+      if(p->keepvectfin==0)
+        for(i=0;i<ids->size;++i)
+          gal_list_data_remove(&p->table, torm[i]);
+
+      /* Clean up. */
+      gal_data_free(ids);
+      if(torm) free(torm);
+    }
+}
+
+
+
+
 /* Find the HDU of the table to read. */
 static char *
 table_catrows_findhdu(char *filename, gal_list_str_t **hdull)
@@ -855,7 +1102,7 @@ table_catrows_findhdu(char *filename, gal_list_str_t **hdull)
       if(*hdull) { hdu=(*hdull)->v; *hdull=(*hdull)->next; }
       else
         error(EXIT_FAILURE, 0, "not enough '--catrowhdu's (or "
-              "'-H'). For every FITS table given to '--catrowfile'. "
+              "'-X'). For every FITS table given to '--catrowfile'. "
               "A call to '--catrowhdu' is necessary to identify "
               "its HDU/extension");
     }
@@ -878,8 +1125,8 @@ table_catrows_prepare(struct tableparams *p)
   char *hdu=NULL;
   int tableformat;
   gal_data_t *ocol, *tmp;
-  size_t i, nrows=p->table->size;
   gal_list_str_t *filell, *hdull;
+  size_t i, dsize[2], nrows=p->table->size;
   size_t numcols, numrows, filledrows=p->table->size;
 
   /* Go over all the given tables and find the final number of rows. */
@@ -888,7 +1135,7 @@ table_catrows_prepare(struct tableparams *p)
     {
       hdu=table_catrows_findhdu(filell->v, &hdull);
       gal_table_info(filell->v, hdu, NULL, &numcols, &numrows,
-                     &tableformat);
+                     &tableformat, "--catrowhdu");
       nrows+=numrows;
     }
 
@@ -900,14 +1147,18 @@ table_catrows_prepare(struct tableparams *p)
      larger array, and free the temporary 'gal_data_t'. */
   for(tmp=p->table; tmp!=NULL; tmp=tmp->next)
     {
+      /* Set the final amount of allocated space for this column. */
+      dsize[0]=nrows;
+      if(tmp->ndim==2) dsize[1]=tmp->dsize[1];
+
       /* Allocate a temporary dataset (we just want its allocated space,
          not the actual 'gal_data_t' pointer)! */
-      ocol=gal_data_alloc(NULL, tmp->type, 1, &nrows, NULL,
+      ocol=gal_data_alloc(NULL, tmp->type, tmp->ndim, dsize, NULL,
                           0, p->cp.minmapsize, p->cp.quietmmap,
                           tmp->name, tmp->unit, tmp->comment);
 
       /* Put the full contents of the existing column into the new
-         column: this will be the first set of rows,  */
+         column: this will be the first set of rows. */
       memcpy(ocol->array, tmp->array, tmp->size*gal_type_sizeof(tmp->type));
 
       /* If the column type is a string, we should set the input pointers
@@ -944,6 +1195,7 @@ table_catrows_prepare(struct tableparams *p)
 static void
 table_catrows(struct tableparams *p)
 {
+  size_t increment;
   char *hdu=NULL, **strarr;
   gal_data_t *new, *ttmp, *tmp;
   gal_list_str_t *filell, *hdull;
@@ -964,7 +1216,7 @@ table_catrows(struct tableparams *p)
       new=gal_table_read(filell->v, hdu, NULL, p->columns,
                          p->cp.searchin, p->cp.ignorecase,
                          p->cp.numthreads, p->cp.minmapsize,
-                         p->cp.quietmmap, NULL);
+                         p->cp.quietmmap, NULL, "--catrowhdu");
 
       /* Make sure that the same number of columns were extracted from this
          table as they were from the original table. */
@@ -999,8 +1251,31 @@ table_catrows(struct tableparams *p)
                   gal_fits_name_save_as_string(filell->v, hdu), colcount,
                   gal_type_name(tmp->type, 1), gal_type_name(ttmp->type, 1));
 
+          /* Make sure the two columns have the same dimensions (vector or
+             single element). */
+          if(tmp->ndim!=ttmp->ndim)
+            error(EXIT_FAILURE, 0, "%s: column %zu is a %s column. "
+                  "However, in the final table (before adding rows) this "
+                  "column is a %s column",
+                  gal_fits_name_save_as_string(filell->v, hdu), colcount,
+                  tmp->ndim==1?"single-valued":"vector",
+                  ttmp->ndim==1?"single-valued":"vector");
+
+          /* If the column is vector, make sure it has the same number of
+             elements. */
+          if(tmp->ndim==2 && tmp->dsize[1]!=ttmp->dsize[1])
+            error(EXIT_FAILURE, 0, "%s: vector column %zu has %zu elements "
+                  "However, in the final table (before adding rows) this "
+                  "vector column has %zu elements",
+                  gal_fits_name_save_as_string(filell->v, hdu), colcount,
+                  tmp->dsize[1], ttmp->dsize[1]);
+
+          /* Set the increment on the existing table (column may be
+             vector). */
+          increment = filledrows * ( tmp->ndim==1 ? 1 : tmp->dsize[1] );
+
           /* Add the new rows and incremenet the counter. */
-          memcpy(gal_pointer_increment(ttmp->array, filledrows, ttmp->type),
+          memcpy(gal_pointer_increment(ttmp->array, increment, ttmp->type),
                  tmp->array, tmp->size*gal_type_sizeof(tmp->type));
 
           /* If the column type is a string, we should set the input
@@ -1011,13 +1286,13 @@ table_catrows(struct tableparams *p)
               for(i=0;i<tmp->size;++i) strarr[i]=NULL;
             }
 
-          /* Take 'ttmp' to the next column and increment the counter */
+          /* Take 'ttmp' to the next column and increment the counter. */
           ttmp=ttmp->next;
           ++colcount;
         }
 
       /* Clean up the columns of the table and increment 'filledrows'. */
-      filledrows += new->size;
+      filledrows += new->dsize[0];
       gal_list_data_free(new);
     }
 }
@@ -1045,7 +1320,7 @@ table_colmetadata(struct tableparams *p)
           /* We have been given a string, so find the first column that has
              the same name. */
           for(col=p->table; col!=NULL; col=col->next)
-            if(!strcmp(col->name, meta->name)) break;
+            if(!strcasecmp(col->name, meta->name)) break;
         }
       /* The column specifier is a number. */
       else
@@ -1097,6 +1372,33 @@ table_colmetadata(struct tableparams *p)
 
 
 
+void
+table_noblankend_check_add(struct tableparams *p,
+                           gal_list_sizet_t **column_indexs, size_t colind)
+{
+  size_t i=0;
+  gal_data_t *tmp;
+  static int warningprinted=0;
+
+  /* Before adding, be sure that the column is not a vector column. */
+  for(tmp=p->table;tmp!=NULL;tmp=tmp->next)
+    if(i++==colind)
+      {
+        if( tmp->ndim==1 ) gal_list_sizet_add(column_indexs, colind);
+        else if(p->cp.quiet==0 && warningprinted==0)
+          {
+            warningprinted=1;
+            error(EXIT_SUCCESS, 0, "WARNING: vector columns will be "
+                  "ignored for the '--noblankend' option. To remove "
+                  "this warning, run with '--quiet' (or '-q')");
+          }
+      }
+}
+
+
+
+
+
 void
 table_noblankend(struct tableparams *p)
 {
@@ -1114,7 +1416,7 @@ table_noblankend(struct tableparams *p)
       && !strcmp(p->noblankend->v,"_all") )
     {
       for(i=0;i<gal_list_data_number(p->table);++i)
-        gal_list_sizet_add(&column_indexs, i);
+        table_noblankend_check_add(p, &column_indexs, i);
     }
 
   /* Only certain columns should be checked, so find/add their index. */
@@ -1124,15 +1426,15 @@ table_noblankend(struct tableparams *p)
         /* First go through the column names and if they match, add
            them. Note that we don't want to stop once a name is found, in
            this scenario, if multiple columns have the same name, we should
-           use all.*/
+           use all. */
         j=0;
         found=0;
         for(tcol=p->table; tcol!=NULL; tcol=tcol->next)
           {
-            if( tcol->name && !strcmp(tcol->name, tmp->v) )
+            if( tcol->name && !strcasecmp(tcol->name, tmp->v) )
               {
                 found=1;
-                gal_list_sizet_add(&column_indexs, j);
+                table_noblankend_check_add(p, &column_indexs, j);
               }
             ++j;
           }
@@ -1152,11 +1454,11 @@ table_noblankend(struct tableparams *p)
             /* Make sure its not zero (the user counts from 1). */
             if(*index==0)
               error(EXIT_FAILURE, 0, "the column number (given to the "
-                    "'--noblankend' option) should start from 1, but you have "
-                    "given 0.");
+                    "'--noblankend' option) should start from 1, but you "
+                    "have given 0");
 
             /* Make sure that the index falls within the number (note that
-               it still counts from 1).  */
+               it still counts from 1). */
             if(*index > gal_list_data_number(p->table))
               error(EXIT_FAILURE, 0, "the final output table only has "
                     "%zu columns, but you have given column %zu to "
@@ -1169,7 +1471,7 @@ table_noblankend(struct tableparams *p)
 
             /* Everything is fine, add the index to the list of columns to
                check. */
-            gal_list_sizet_add(&column_indexs, *index-1);
+            table_noblankend_check_add(p, &column_indexs, *index-1);
 
             /* Clean up. */
             free(index);
@@ -1181,9 +1483,16 @@ table_noblankend(struct tableparams *p)
       }
 
   /* Remove all blank rows from the output table, note that we don't need
-     the flags of the removed columns here. So we can just free it up. */
-  flag=gal_blank_remove_rows(p->table, column_indexs);
-  gal_data_free(flag);
+     the flags of the removed columns here. So we can just free it up.
+
+     Vector columns are currently ignored in '--noblankend', so if the user
+     only asks for no blanks in a vector column, in effect, no rows should
+     be removed. */
+  if(column_indexs)
+    {
+      flag=gal_blank_remove_rows(p->table, column_indexs, 1);
+      gal_data_free(flag);
+    }
 }
 
 
@@ -1201,11 +1510,13 @@ table_txt_formats(struct tableparams *p)
         {
         case GAL_TYPE_FLOAT32:
           if(p->txtf32format)    tmp->disp_fmt=p->txtf32format;
-          if(p->txtf32precision) tmp->disp_precision=p->txtf32precision;
+          if(p->txtf32precision!=GAL_BLANK_INT)
+            tmp->disp_precision=p->txtf32precision;
           break;
         case GAL_TYPE_FLOAT64:
           if(p->txtf64format)    tmp->disp_fmt=p->txtf64format;
-          if(p->txtf64precision) tmp->disp_precision=p->txtf64precision;
+          if(p->txtf64precision!=GAL_BLANK_INT)
+            tmp->disp_precision=p->txtf64precision;
           break;
         }
     }
@@ -1227,15 +1538,29 @@ table_txt_formats(struct tableparams *p)
 
 
 
-/**************************************************************/
-/***************       Top function         *******************/
-/**************************************************************/
 void
-table(struct tableparams *p)
+table_column(struct tableparams *p)
 {
   /* Concatenate the columns of tables (if required). */
   if(p->catcolumnfile) table_catcolumn(p);
 
+  /* Extract columns from vector. */
+  if(p->fromvector) table_fromvector(p);
+
+  /* Merge columns into a vector column. */
+  if(p->tovector) table_tovector(p);
+
+  /* If any arithmetic operations are needed, do them. */
+  if(p->colpack) arithmetic_operate(p);
+}
+
+
+
+
+
+void
+table_row(struct tableparams *p)
+{
   /* Concatenate the rows of multiple tables (if required). */
   if(p->catrowfile) table_catrows(p);
 
@@ -1246,24 +1571,41 @@ table(struct tableparams *p)
   if(p->sort) table_sort(p);
 
   /* If the output number of rows is limited, apply them. */
-  if( p->rowrange
+  if(    p->rowrange
       || p->rowrandom
       || p->head!=GAL_BLANK_SIZE_T
       || p->tail!=GAL_BLANK_SIZE_T )
     table_select_by_position(p);
 
-  /* If any arithmetic operations are needed, do them. */
-  if(p->outcols)
-    arithmetic_operate(p);
+  /* Transpose the input table. */
+  if(p->transpose) table_transpose(p);
+}
 
-  /* When column metadata should be updated. */
+
+
+
+
+/***************************************************************/
+/*************         Top-level function          *************/
+/***************************************************************/
+void
+table(struct tableparams *p)
+{
+  /* Do the requested operations. */
+  if(p->rowfirst) { table_row(p);    table_column(p); }
+  else            { table_column(p); table_row(p);    }
+
+  /* Last steps (independent of '--rowfirst'). */
   if(p->colmetadata) table_colmetadata(p);
-
-  /* When any columns with blanks should be removed. */
   if(p->noblankend) table_noblankend(p);
 
-  /* Write the output. */
-  table_txt_formats(p);
-  gal_table_write(p->table, NULL, NULL, p->cp.tableformat, p->cp.output,
-                  "TABLE", p->colinfoinstdout);
+  /* Write the output or a warning/error (it can become NULL!) */
+  if(p->table)
+    {
+      table_txt_formats(p);
+      gal_table_write(p->table, NULL, NULL, p->cp.tableformat,
+                      p->cp.output, "TABLE", p->colinfoinstdout, 0);
+    }
+  else
+    error(EXIT_FAILURE, 0, "no output columns");
 }
