@@ -6,7 +6,7 @@ Corresponding author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
      Pedram Ashofteh-Ardakani <pedramardakani@pm.me>
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -50,7 +50,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /***************************************************************/
 /* Multiply a 2 element vector with a transformation matrix and put the
    result in the 2 element output array. It is assumed that the input is
-   from a flat coordinate systemy. */
+   from a flat coordinate system. */
 #define WARP_MAPPOINT(V, T, O)                                  \
   {                                                             \
     (O)[0]=( ( (T)[0]*(V)[0] + (T)[1]*(V)[1] + (T)[2] )         \
@@ -416,7 +416,8 @@ warp_write_to_file(struct warpparams *p, int hasmatrix)
         sprintf(&keyword[i*FLEN_KEYWORD], "WMTX%zu_%zu", i/3+1, i%3+1);
         gal_fits_key_list_add_end(&headers, GAL_TYPE_FLOAT64,
                                   &keyword[i*FLEN_KEYWORD], 0, &m[i], 0,
-                                  "Warp matrix element value", 0, NULL, 0);
+                                  "Warp matrix element value",
+                                  0, NULL, 0);
       }
 
   /* Convert the output image if needed. */
@@ -425,16 +426,15 @@ warp_write_to_file(struct warpparams *p, int hasmatrix)
 
   /* Save the output and 'MAX-FRAC' if available. */
   for(tmp=p->output;tmp!=NULL;tmp=tmp->next)
-    gal_fits_img_write(tmp, p->cp.output, NULL, PROGRAM_NAME);
+    gal_fits_img_write(tmp, p->cp.output, NULL, 0);
 
   /* Write the configuration keywords on HDU/extension '0'. */
-  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys,
+  gal_fits_key_write_filename("input", p->inputname, &p->cp.ckeys,
                               1, p->cp.quiet);
-  gal_fits_key_write_config(&p->cp.okeys, "Warp configuration",
-                            "WARP-CONFIG", p->cp.output, "0", "NONE");
+  gal_fits_key_write(p->cp.ckeys, p->cp.output, "0", "NONE", 1, 0);
 
   /* Write headers on HDU/extension '1'. */
-  gal_fits_key_write(&headers, NULL, p->cp.output, "1", "NONE");
+  gal_fits_key_write(headers, p->cp.output, "1", "NONE", 1, 0);
 }
 
 
@@ -526,7 +526,7 @@ warp_write_wcs_linear(struct warpparams *p)
 
 
 /***************************************************************/
-/**************       Outside function        ******************/
+/**************      Top-level function       ******************/
 /***************************************************************/
 void
 warp(struct warpparams *p)
@@ -537,7 +537,7 @@ warp(struct warpparams *p)
   /* Do the preparations and set the pointers to the functions to use. */
   if( p->wcsalign )
     {
-      /* Calculate and allocate the output image size and WCS */
+      /* Calculate and allocate the output image size and WCS. */
       if(!p->cp.quiet)
         {
           gal_timing_report(NULL, "Initializing the output image...", 1);
@@ -545,7 +545,7 @@ warp(struct warpparams *p)
         }
       gal_warp_wcsalign_init(wa);
 
-      /* Fill the output image */
+      /* Fill the output image. */
       if(!p->cp.quiet)
         {
           gal_timing_report(&t0, "Done", 2);
@@ -572,7 +572,7 @@ warp(struct warpparams *p)
                            p->cp.numthreads, p->cp.minmapsize,
                            p->cp.quietmmap);
 
-      /* Fix the linear matrix before saving the output image to disk */
+      /* Fix the linear matrix before saving the output image to disk. */
       warp_write_wcs_linear(p);
     }
 
