@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2022 Free Software Foundation, Inc.
+Copyright (C) 2017-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -29,6 +29,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <inttypes.h>
 
+#include <gnuastro/wcs.h>
 #include <gnuastro/data.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/blank.h>
@@ -47,7 +48,7 @@ gal_blank_write(void *ptr, uint8_t type)
 {
   switch(type)
     {
-    /* Numeric types */
+    /* Numeric types. */
     case GAL_TYPE_UINT8:   *(uint8_t  *)ptr = GAL_BLANK_UINT8;    break;
     case GAL_TYPE_INT8:    *(int8_t   *)ptr = GAL_BLANK_INT8;     break;
     case GAL_TYPE_UINT16:  *(uint16_t *)ptr = GAL_BLANK_UINT16;   break;
@@ -64,7 +65,7 @@ gal_blank_write(void *ptr, uint8_t type)
       gal_checkset_allocate_copy(GAL_BLANK_STRING, ptr);
       break;
 
-    /* Complex types */
+    /* Complex types. */
     case GAL_TYPE_COMPLEX32:
     case GAL_TYPE_COMPLEX64:
       error(EXIT_FAILURE, 0, "%s: complex types are not yet supported",
@@ -129,7 +130,7 @@ gal_blank_initialize(gal_data_t *input)
 
 
 
-/* Initialize an array to the given type's blank values.*/
+/* Initialize an array to the given type's blank values. */
 void
 gal_blank_initialize_array(void *array, size_t size, uint8_t type)
 {
@@ -339,7 +340,7 @@ gal_blank_is(void *pointer, uint8_t type)
   /* Do the checks based on the type. */
   switch(type)
     {
-    /* Numeric types */
+    /* Numeric types. */
     case GAL_TYPE_UINT8:     return *(uint8_t  *)pointer==GAL_BLANK_UINT8;
     case GAL_TYPE_INT8:      return *(int8_t   *)pointer==GAL_BLANK_INT8;
     case GAL_TYPE_UINT16:    return *(uint16_t *)pointer==GAL_BLANK_UINT16;
@@ -355,7 +356,7 @@ gal_blank_is(void *pointer, uint8_t type)
     case GAL_TYPE_STRING:
       if(!strcmp(pointer,GAL_BLANK_STRING)) return 1; else return 0;
 
-    /* Complex types */
+    /* Complex types. */
     case GAL_TYPE_COMPLEX32:
     case GAL_TYPE_COMPLEX64:
       error(EXIT_FAILURE, 0, "%s: complex types are not yet supported",
@@ -448,7 +449,7 @@ gal_blank_present(gal_data_t *input, int updateflag)
   /* Go over the pixels and check: */
   switch(block->type)
     {
-    /* Numeric types */
+    /* Numeric types. */
     case GAL_TYPE_UINT8:     HAS_BLANK( uint8_t  );    break;
     case GAL_TYPE_INT8:      HAS_BLANK( int8_t   );    break;
     case GAL_TYPE_UINT16:    HAS_BLANK( uint16_t );    break;
@@ -463,21 +464,21 @@ gal_blank_present(gal_data_t *input, int updateflag)
     /* String. */
     case GAL_TYPE_STRING:
       if(input!=block)
-        error(EXIT_FAILURE, 0, "%s: tile mode is currently not supported for "
-              "strings", __func__);
+        error(EXIT_FAILURE, 0, "%s: tile mode is currently not "
+              "supported for strings", __func__);
       strf = (str=input->array) + input->size;
       do
         if(*str==NULL || !strcmp(*str,GAL_BLANK_STRING)) return 1;
       while(++str<strf);
       break;
 
-    /* Complex types */
+    /* Complex types. */
     case GAL_TYPE_COMPLEX32:
     case GAL_TYPE_COMPLEX64:
       error(EXIT_FAILURE, 0, "%s: complex types are not yet supported",
             __func__);
 
-    /* Bit */
+    /* Bit. */
     case GAL_TYPE_BIT:
       error(EXIT_FAILURE, 0, "%s: bit type datasets are not yet supported",
             __func__);
@@ -557,21 +558,28 @@ gal_blank_number(gal_data_t *input, int updateflag)
 #define FLAG_BLANK(IT) {                                                \
     IT b, *a=input->array;                                              \
     gal_blank_write(&b, input->type);                                   \
-    if(b==b) /* Blank value can be checked with the equal comparison */ \
-      do { *o = *a==b;  ++a; } while(++o<of);                           \
-    else     /* Blank value will fail with the equal comparison */      \
-      do { *o = *a!=*a; ++a; } while(++o<of);                           \
+    if(b==b) /* Blank value can be checked with the equal. */           \
+      do {*o = blank1_not0 ? *a==b  : *a!=b;  ++a;} while(++o<of);      \
+    else     /* Blank value will fail with the equal comparison. */     \
+      do {*o = blank1_not0 ? *a!=*a : *a==*a; ++a;} while(++o<of);      \
   }
-gal_data_t *
-gal_blank_flag(gal_data_t *input)
+static gal_data_t *
+blank_flag(gal_data_t *input, int blank1_not0)
 {
   uint8_t *o, *of;
   gal_data_t *out;
   char **str=input->array, **strf=str+input->size;
 
-  /* The datasets may be empty. In this case the output should also be
-     empty (we can have tables and images with 0 rows or pixels!). */
-  if(input->size==0 || input->array==NULL) return input;
+  /* The datasets may be empty. In this case, the output should also be
+     empty, but with the standard 'uint8' type of a flag (we can have
+     tables and images with 0 rows or pixels!). */
+  if(input->size==0 || input->array==NULL)
+    {
+      out=gal_data_alloc_empty(input->ndim, input->minmapsize,
+                               input->quietmmap);
+      out->type=GAL_TYPE_UINT8;
+      return out;
+    }
 
   /* Do all the checks and allocations if a blank is actually present! */
   if( gal_blank_present(input, 0) )
@@ -579,8 +587,8 @@ gal_blank_flag(gal_data_t *input)
       /* Allocate a non-cleared output array, we are going to parse the
          input and fill in each element. */
       out=gal_data_alloc(NULL, GAL_TYPE_UINT8, input->ndim, input->dsize,
-                         input->wcs, 0, input->minmapsize, input->quietmmap,
-                         NULL, "bool", NULL);
+                         input->wcs, 0, input->minmapsize,
+                         input->quietmmap, NULL, "bool", NULL);
 
       /* Set the pointers for easy looping. */
       of=(o=out->array)+input->size;
@@ -588,7 +596,7 @@ gal_blank_flag(gal_data_t *input)
       /* Go over the pixels and set the output values. */
       switch(input->type)
         {
-        /* Numeric types */
+        /* Numeric types. */
         case GAL_TYPE_UINT8:     FLAG_BLANK( uint8_t  );    break;
         case GAL_TYPE_INT8:      FLAG_BLANK( int8_t   );    break;
         case GAL_TYPE_UINT16:    FLAG_BLANK( uint16_t );    break;
@@ -602,7 +610,10 @@ gal_blank_flag(gal_data_t *input)
 
         /* String. */
         case GAL_TYPE_STRING:
-          do *o++ = !strcmp(*str,GAL_BLANK_STRING); while(++str<strf);
+          if(blank1_not0)
+            do *o++ = strcmp(*str,GAL_BLANK_STRING)==0; while(++str<strf);
+          else
+            do *o++ = strcmp(*str,GAL_BLANK_STRING)!=0; while(++str<strf);
           break;
 
         /* Currently unsupported types. */
@@ -618,11 +629,22 @@ gal_blank_flag(gal_data_t *input)
                 __func__, input->type);
         }
     }
+
+  /* Input had no blanks */
   else
-    /* Allocate a CLEAR data structure (all zeros). */
-    out=gal_data_alloc(NULL, GAL_TYPE_UINT8, input->ndim, input->dsize,
-                       input->wcs, 1, input->minmapsize, input->quietmmap,
-                       NULL, "bool", NULL);
+    {
+      /* Allocate the output data structure (if the caller wants to flag
+         blanks, then we will "clear" the output also). */
+      out=gal_data_alloc(NULL, GAL_TYPE_UINT8, input->ndim, input->dsize,
+                         input->wcs, blank1_not0 ? 1 : 0,
+                         input->minmapsize, input->quietmmap,
+                         NULL, "bool", NULL);
+
+      /* If the caller wants to flag the non-blank elements, we should now
+         set all the output values to 1 (the input had no blanks!). */
+      if(blank1_not0==0)
+        { of=(o=out->array)+out->size; do *o++=1; while(o<of); }
+    }
 
   /* Return */
   return out;
@@ -632,6 +654,256 @@ gal_blank_flag(gal_data_t *input)
 
 
 
+gal_data_t *
+gal_blank_flag(gal_data_t *input)
+{
+  return blank_flag(input, 1);
+}
+
+
+
+
+
+gal_data_t *
+gal_blank_flag_not(gal_data_t *input)
+{
+  return blank_flag(input, 0);
+}
+
+
+
+#define NMM_CRDS_CHECK                                                  \
+  gal_dimension_index_to_coord(a-as, ndim, input->dsize, c);            \
+  for(i=0;i<ndim;++i)                                                   \
+    { if(c[i]<min[i]) min[i]=c[i]; if(c[i]>max[i]) max[i]=c[i]; }
+
+#define NMM_CRDS(IT) {                                                  \
+  IT b, *a=input->array, *as=a, *af=a+input->size;                      \
+  gal_blank_write(&b, input->type);                                     \
+  if(b==b) /* Blank value can be checked with the equal. */             \
+    do { if(*a!=b)  {NMM_CRDS_CHECK} } while(++a<af);                   \
+  else     /* Blank value will fail with the equal comparison. */       \
+    do { if(*a==*a) {NMM_CRDS_CHECK} } while(++a<af);                   \
+  }
+
+size_t *
+gal_blank_not_minmax_coords(gal_data_t *input)
+{
+  char **strarr=input->array;
+  size_t c[3], i, ndim, *out, *min, *max;
+
+  /* Sanity check. */
+  if(input==NULL) error(EXIT_FAILURE, 0, "%s: input is NULL", __func__);
+
+  /* Allocate the output and min/max datasets; and initialize the 'min'
+     dataset (note that the 'max' dataset has already been "clear"ed to
+     zero; which is what we want). */
+  ndim=input->ndim;
+  min=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__, "min");
+  max=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 1, __func__, "max");
+  out=gal_pointer_allocate(GAL_TYPE_SIZE_T, 2*ndim, 0, __func__, "out");
+  for(i=0;i<ndim;++i) min[i]=GAL_BLANK_SIZE_T;
+
+  /* The input dataset may be empty. In this case, the output should also
+     be empty, but with the standard 'uint8' type of a flag (we can have
+     tables and images with 0 rows or pixels!). */
+  if(input->size==0 || input->array==NULL)
+    for(i=0;i<2*ndim;++i)
+      out[i]=GAL_BLANK_SIZE_T;
+
+  /* Do all the checks and allocations if a blank is actually present! */
+  if( gal_blank_present(input, 0) )
+    {
+      /* Go over the pixels and set the output values. */
+      switch(input->type)
+        {
+        /* Numeric types */
+        case GAL_TYPE_UINT8:     NMM_CRDS( uint8_t  );    break;
+        case GAL_TYPE_INT8:      NMM_CRDS( int8_t   );    break;
+        case GAL_TYPE_UINT16:    NMM_CRDS( uint16_t );    break;
+        case GAL_TYPE_INT16:     NMM_CRDS( int16_t  );    break;
+        case GAL_TYPE_UINT32:    NMM_CRDS( uint32_t );    break;
+        case GAL_TYPE_INT32:     NMM_CRDS( int32_t  );    break;
+        case GAL_TYPE_UINT64:    NMM_CRDS( uint64_t );    break;
+        case GAL_TYPE_INT64:     NMM_CRDS( int64_t  );    break;
+        case GAL_TYPE_FLOAT32:   NMM_CRDS( float    );    break;
+        case GAL_TYPE_FLOAT64:   NMM_CRDS( double   );    break;
+
+        /* String. */
+        case GAL_TYPE_STRING:
+          for(i=0;i<input->size;++i)
+            if(strcmp(strarr[i],GAL_BLANK_STRING))
+              {
+                gal_dimension_index_to_coord(i, ndim, input->dsize, c);
+                for(i=0;i<ndim;++i)
+                  { if(c[i]<min[i]) min[i]=c[i];
+                    if(c[i]>max[i]) max[i]=c[i]; }
+            }
+          break;
+
+        /* Currently unsupported types. */
+        case GAL_TYPE_BIT:
+        case GAL_TYPE_COMPLEX32:
+        case GAL_TYPE_COMPLEX64:
+          error(EXIT_FAILURE, 0, "%s: %s type not yet supported",
+                __func__, gal_type_name(input->type, 1));
+
+        /* Bad input. */
+        default:
+          error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
+                __func__, input->type);
+        }
+
+      /* Write the values into the output. For the maximum ranges, we are
+         adding by one so the callers can simply calculate the number of
+         pixels by subtracting the two (since counting starts from 0). */
+      for(i=0;i<ndim;++i) { out[i*2]=min[i]; out[i*2+1]=max[i]+1; }
+    }
+
+  /* Input had no blanks, just fill the output using the size of the
+     input. */
+  else for(i=0;i<ndim;++i) { out[i*2]=0; out[i*2+1]=input->dsize[i]; }
+
+  /* For a check:
+  for(i=0;i<ndim;++i)
+    printf("%s:%zu: %zu:%zu\n", __func__, i, out[i*2], out[i*2+1]);
+  printf("%s: GOOD\n", __func__); exit(0);
+  */
+
+  /* Clean up and return. */
+  free(min);
+  free(max);
+  return out;
+}
+
+
+
+
+
+/* Trim all NaN-valued rows (in 1D) or columns/rows (in 2D). If 'inplace'
+   is non-zero, no new array will be allocated. */
+gal_data_t *
+gal_blank_trim(gal_data_t *input, int inplace)
+{
+  int coversall=1;
+  void *to, *from;
+  gal_data_t *out=NULL;
+  struct wcsprm *owcs=NULL;
+  size_t *odsize, *idsize, *mmc;
+  size_t i, j, cbytes, osize, opsize, ipsize, ndim;
+
+  /* Sanity checks. */
+  if(input==NULL) error(EXIT_FAILURE, 0, "%s: input is NULL", __func__);
+  ndim=input->ndim;
+  if(ndim>3)
+    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to "
+          "fix the problem. This function is currently not implemented "
+          "for %zu-dimensional inputs", __func__, PACKAGE_BUGREPORT,
+          ndim);
+
+  /* Find the smallest/largest coordinates containing the  */
+  mmc=gal_blank_not_minmax_coords(input);
+
+  /* In case the extrema cover the full image, then just return the input
+     (no trimming necessary!). We first assume that that 'coversall' is
+     true (1). We then parse over the dimensions and check the minimum and
+     maximum and multiply the result of the check (0 or 1) to
+     'coversall'. As a result, if a single dimesion doesn't cover the full
+     range, 'coversall' will be zero.*/
+  for(i=0;i<ndim;++i)
+    coversall *= mmc[i*2]==0 && mmc[i*2+1]==input->dsize[i];
+  if(coversall) return input;
+
+  /* Triming is necessary, so prepare the output. If the input can be
+     freed, then we'll just use its already allocated space. Otherwise,
+     we'll allocate a new dataset. */
+  odsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
+                              "odsize");
+  for(i=0;i<ndim;++i) odsize[i]=mmc[i*2+1]-mmc[i*2];
+  if(inplace) out=input;
+  else
+    {
+      owcs=gal_wcs_copy(input->wcs);
+      out=gal_data_alloc(NULL, input->type, input->ndim, odsize, owcs,
+                         0, input->minmapsize, input->quietmmap, NULL,
+                         NULL, NULL);
+    }
+
+  /* Copy (along the fastest dimension). */
+  idsize=input->dsize;
+  cbytes = odsize[ndim-1] * gal_type_sizeof(input->type);
+  switch(ndim)
+    {
+    /* 1D data. */
+    case 1:
+      to=out->array;
+      from=gal_pointer_increment(input->array, mmc[0], input->type);
+      if(inplace) memmove(to, from, cbytes); /* Overlap is possible. */
+      else        memcpy(to, from, cbytes); /* Overlap not possible. */
+      break;
+
+    /* 2D data. */
+    case 2:
+      for(i=mmc[0]; i<mmc[1]; ++i)
+        {
+          to=gal_pointer_increment(out->array, (i-mmc[0])*odsize[1],
+                                   out->type);
+          from=gal_pointer_increment(input->array,
+                                     i*idsize[1]+mmc[2],
+                                     input->type);
+          if(inplace) memmove(to, from, cbytes); /* Overlap is possible. */
+          else        memcpy(to, from, cbytes); /* Overlap not possible. */
+        }
+      break;
+
+    /* 3D data. */
+    case 3:
+      opsize=odsize[1]*odsize[2]; /* These are "plane"-sizes: number of */
+      ipsize=idsize[1]*idsize[2]; /* elements to change the slowest dim. */
+      for(i=mmc[0]; i<mmc[1]; ++i)
+        for(j=mmc[2]; j<mmc[3]; ++j)
+          {
+            to=gal_pointer_increment(out->array,
+                                     (i-mmc[0])*opsize+(j-mmc[2])*odsize[2],
+                                     out->type);
+            from=gal_pointer_increment(input->array,
+                                       i*ipsize+j*idsize[2]+mmc[4],
+                                       input->type);
+            if(inplace) memmove(to, from, cbytes); /* Overlap is possible. */
+            else        memcpy(to, from, cbytes); /* Overlap not possible. */
+          }
+      break;
+
+    /* Other dimensions. */
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to "
+            "find and fix it. This function does not support "
+            "%zu-dimensional inputs", __func__, PACKAGE_BUGREPORT, ndim);
+    }
+
+  /* Correct the dimensionality of the output (only relevant when the
+     output is not "in-place"). */
+  osize=1;
+  if(inplace)
+    {
+      for(i=0;i<ndim;++i) osize *= (out->dsize[i]=odsize[i]);
+      out->size=osize;
+    }
+
+  /* Correct the WCS of the output (if it has any!). Recall that in WCSLIB,
+     the coordinates are in FITS order, not C order.*/
+  if(out->wcs) for(i=0;i<ndim;++i) out->wcs->crpix[ndim-1-i] -= mmc[i*2];
+
+  /* Clean up and return the output. */
+  free(mmc);
+  free(odsize);
+  return out;
+}
+
+
+
+
+
 /* Write a blank value in the input anywhere that the flag dataset is not
    zero or not blank. */
 #define BLANK_FLAG_APPLY(IT) {                                          \
@@ -733,6 +1005,11 @@ gal_blank_flag_remove(gal_data_t *input, gal_data_t *flag)
     error(EXIT_FAILURE, 0, "%s: the 'flag' argument doesn't have the same "
           "size as the 'input' argument", __func__);
 
+  /* If there is no elements in the input or the flag (we have already
+     confirmed that they are the same size), then just return (nothing is
+     necessary to do). */
+  if(flag->size==0 || flag->array==NULL) return;
+
   /* Shift all non-blank elements to the start of the array. */
   switch(input->type)
     {
@@ -750,9 +1027,9 @@ gal_blank_flag_remove(gal_data_t *input, gal_data_t *flag)
       strarr=input->array;
       for(i=0;i<input->size;++i)
         {
-          if( *f && *f!=GAL_BLANK_UINT8 )        /* Flagged to be removed */
+          if( *f && *f!=GAL_BLANK_UINT8 )    /* Flagged to be removed */
             { free(strarr[i]); strarr[i]=NULL; }
-          else strarr[num++]=strarr[i];          /* Keep. */
+          else strarr[num++]=strarr[i];      /* Keep. */
           ++f;
         }
       break;
@@ -770,6 +1047,90 @@ gal_blank_flag_remove(gal_data_t *input, gal_data_t *flag)
 
 
 
+/* Remove flagged elements from a dataset (which may not necessarily
+   blank), convert it to a 1D dataset and adjust the size properly. In
+   practice this function doesn't 'realloc' the input array, all it does is
+   to shift the blank eleemnts to the end and adjust the size elements of
+   the 'gal_data_t'. */
+#define BLANK_FLAG_REMOVE_D0(IT) {                                      \
+    IT *a=input->array, *af=a+input->size, *o=input->array;             \
+    do {                                                                \
+      printf("\n%s: HERE\n", __func__);                                   \
+      if(*f) a+=nelem;                                                  \
+      else {num++; for(i=0;i<nelem;++i) {*o++=*a; ++a; printf("%zu ", i);} } \
+      ++f;                                                              \
+    }                                                                   \
+    while(a<af);                                                        \
+  }
+static void
+blank_flag_remove_dim0(gal_data_t *input, gal_data_t *flag)
+{
+  size_t i, num=0;
+  uint8_t it=input->type;
+  uint8_t *f=flag->array;
+  size_t nelem = input->ndim==1 ? 1 : input->size/input->dsize[0];
+
+  /* Sanity check. */
+  if(flag->type!=GAL_TYPE_UINT8)
+    error(EXIT_FAILURE, 0, "%s: the 'flag' argument has a '%s' type, it "
+          "must have an unsigned 8-bit type", __func__,
+          gal_type_name(flag->type, 1));
+  if(input->dsize[0]!=flag->dsize[0])
+    error(EXIT_FAILURE, 0, "%s: the 'flag' argument doesn't have the same "
+          "size as the 'input' argument", __func__);
+
+  /* This is a special function! */
+  if(flag->ndim!=1)
+    error(EXIT_FAILURE, 0, "%s: this function's 'flag' should only "
+          "have a single dimension", __func__);
+  if(input->ndim==flag->ndim)
+    error(EXIT_FAILURE, 0, "%s: this function is only for scenarios "
+          "where 'flag' is single-dimensional and the input is "
+          "multi-dimensional. Please use 'gal_blank_flag_remove' when "
+          "the input and flag have the same number of dimensions",
+          __func__);
+
+  /* If there is no elements in the input or the flag (we have already
+     confirmed that they are the same size), then just return (nothing is
+     necessary to do). */
+  if(flag->size==0 || flag->array==NULL) return;
+
+  /* Move all the non-flagged elements to the start of the array. */
+  for(i=0;i<flag->size;++i)
+    if(f[i]==0)
+      memmove(gal_pointer_increment(input->array, (num++)*nelem, it),
+              gal_pointer_increment(input->array, i*nelem,       it),
+              nelem*gal_type_sizeof(it));
+
+  /* Shift all non-blank elements to the start of the array.
+  switch(input->type)
+    {
+    case GAL_TYPE_UINT8:    BLANK_FLAG_REMOVE_D0( uint8_t  );    break;
+    case GAL_TYPE_INT8:     BLANK_FLAG_REMOVE_D0( int8_t   );    break;
+    case GAL_TYPE_UINT16:   BLANK_FLAG_REMOVE_D0( uint16_t );    break;
+    case GAL_TYPE_INT16:    BLANK_FLAG_REMOVE_D0( int16_t  );    break;
+    case GAL_TYPE_UINT32:   BLANK_FLAG_REMOVE_D0( uint32_t );    break;
+    case GAL_TYPE_INT32:    BLANK_FLAG_REMOVE_D0( int32_t  );    break;
+    case GAL_TYPE_UINT64:   BLANK_FLAG_REMOVE_D0( uint64_t );    break;
+    case GAL_TYPE_INT64:    BLANK_FLAG_REMOVE_D0( int64_t  );    break;
+    case GAL_TYPE_FLOAT32:  BLANK_FLAG_REMOVE_D0( float    );    break;
+    case GAL_TYPE_FLOAT64:  BLANK_FLAG_REMOVE_D0( double   );    break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
+            __func__, input->type);
+    }
+  */
+
+  /* Adjust the size elements of the dataset. */
+  input->size=1;
+  input->dsize[0]=num;
+  for(i=0;i<input->ndim;++i) input->size*=input->dsize[i];
+}
+
+
+
+
+
 /* Remove blank elements from a dataset, convert it to a 1D dataset and
    adjust the size properly. In practice this function doesn't 'realloc'
    the input array, all it does is to shift the blank eleemnts to the end
@@ -857,11 +1218,23 @@ gal_blank_remove_realloc(gal_data_t *input)
 
 
 static gal_data_t *
-blank_remove_in_list_merge_flags(gal_data_t *thisdata, gal_data_t *flag)
+blank_remove_in_list_merge_flags(gal_data_t *thisdata, gal_data_t *flag,
+                                 int onlydim0)
 {
   size_t i;
   uint8_t *u, *tu;
   gal_data_t *flagtmp;
+  static int warningprinted=0;
+
+  /* Ignore the dataset if it has more than one dimension and 'onlydim0' is
+     called. */
+  if(onlydim0 && thisdata->ndim>1 && warningprinted==0)
+    {
+      warningprinted=1;
+      error(EXIT_SUCCESS, 0, "%s: WARNING: multi-dimensional columns "
+            "are not supported when 'onlydim0' is non-zero", __func__);
+      return NULL;
+    }
 
   /* Build the flag of blank elements for this column. */
   flagtmp=gal_blank_flag(thisdata);
@@ -894,7 +1267,8 @@ blank_remove_in_list_merge_flags(gal_data_t *thisdata, gal_data_t *flag)
 
 /* Remove any row that has a blank in any of the given columns. */
 gal_data_t *
-gal_blank_remove_rows(gal_data_t *columns, gal_list_sizet_t *column_indexs)
+gal_blank_remove_rows(gal_data_t *columns, gal_list_sizet_t *column_indexs,
+                      int onlydim0)
 {
   size_t i;
   gal_list_sizet_t *tcol;
@@ -918,15 +1292,18 @@ gal_blank_remove_rows(gal_data_t *columns, gal_list_sizet_t *column_indexs)
                 __func__, gal_list_data_number(columns), tcol->v);
 
         /* Build the flag of blank elements for this column. */
-        flag=blank_remove_in_list_merge_flags(tmp, flag);
+        flag=blank_remove_in_list_merge_flags(tmp, flag, onlydim0);
       }
   else
     for(tmp=columns; tmp!=NULL; tmp=tmp->next)
-      flag=blank_remove_in_list_merge_flags(tmp, flag);
+      flag=blank_remove_in_list_merge_flags(tmp, flag, onlydim0);
 
   /* Now that the flags have been set, remove the rows. */
   for(tmp=columns; tmp!=NULL; tmp=tmp->next)
-    gal_blank_flag_remove(tmp, flag);
+    {
+      if(tmp->ndim==1 || onlydim0==0) gal_blank_flag_remove(tmp, flag);
+      else blank_flag_remove_dim0(tmp, flag);
+    }
 
   /* For a check.
   double *d1=columns->array, *d2=columns->next->array;
