@@ -5,7 +5,8 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+     Pedram Ashofteh Ardakani <pedramardakani@pm.me>
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -49,7 +50,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /**************************************************************/
 /* The GSL random number generator (RNG) reads values from the
    environment. This function is designed to make the job easier for any
-   program using GSL's RNG. If the user doesn't want to set the */
+   Gnuastro program using GSL's RNG functions. */
 gsl_rng *
 gal_checkset_gsl_rng(uint8_t envseed_bool, const char **name,
                      unsigned long int *seed)
@@ -59,12 +60,12 @@ gal_checkset_gsl_rng(uint8_t envseed_bool, const char **name,
   /* Let GSL read the environment and convert the type name (as string) to
      'gsl_rng_type'. After this function, 'gsl_rng_default' contains the
      generator's type and 'gsl_rng_default_seed' contains the (possibly)
-     given seed.*/
+     given seed. */
   gsl_rng_env_setup();
 
   /* Allocate the random number generator based on the requested type and
      save its name. If no 'GSL_RNG_TYPE' is set, then use a fixed
-     generator.*/
+     generator. */
   rng=gsl_rng_alloc(secure_getenv("GSL_RNG_TYPE")
                     ? gsl_rng_default
                     : gsl_rng_ranlxs1);
@@ -85,28 +86,131 @@ gal_checkset_gsl_rng(uint8_t envseed_bool, const char **name,
 
 
 
+static size_t
+checkset_meminfo_line(char *line, char *keyname, size_t keylen,
+                      char *file)
+{
+  size_t *freemem=NULL, out=GAL_BLANK_SIZE_T;
+  char *linecp, *token, *units="kB", delimiters[] = " ", *saveptr;
+
+  if( !strncmp(line, keyname, keylen) )
+    {
+      /* We need to work on a copied line to avoid messing up the
+         contents of the actual line. */
+      gal_checkset_allocate_copy(line, &linecp);
+
+      /* The first token (which we don't need). */
+      token=strtok_r(linecp, delimiters, &saveptr);
+
+      /* The second token (which is the actual number we want). */
+      token=strtok_r(NULL, delimiters, &saveptr);
+      if(token)
+        {
+          /* Read the token as a number. */
+          if( gal_type_from_string((void **)(&freemem), token,
+                                   GAL_TYPE_SIZE_T) )
+            error(EXIT_SUCCESS, 0, "WARNING: %s: value of '%s' "
+                  "keyword couldn't be read as an integer. Hence "
+                  "the amount of available RAM couldn't be "
+                  "determined. If a large volume of data is "
+                  "provided, the program may crash. Please contact "
+                  "us at '%s' to fix the problem",
+                  file, keyname, PACKAGE_BUGREPORT);
+          else
+            {
+              /* The third token should be the units ('kB'). If it
+                 isn't, there should be an error because we currently
+                 assume kilobytes. */
+              token=strtok_r(NULL, delimiters, &saveptr);
+              if(token)
+                {
+                  /* The units should be 'kB' (for kilobytes). */
+                  if( !strncmp(token, units, 2) )
+                    out=freemem[0]*1000;
+                  else
+                    error(EXIT_SUCCESS, 0, "WARNING: %s: the units of "
+                          "the value of '%s' keyword is (usually 'kB') "
+                          "isn't recognized. Hence the amount of "
+                          "available RAM couldn't be determined. If a "
+                          "large volume of data is provided, the "
+                          "program may crash. Please contact us at "
+                          "'%s' to fix the problem", file, keyname,
+                          PACKAGE_BUGREPORT);
+                }
+              else
+                error(EXIT_SUCCESS, 0, "WARNING: %s: the units of the "
+                      "value of '%s' keyword (usually 'kB') couldn't "
+                      "be read as an integer. Hence the amount of "
+                      "available RAM couldn't be determined. If a "
+                      "large volume of data is provided, the program "
+                      "may crash. Please contact us at '%s' to fix "
+                      "the problem", file, keyname, PACKAGE_BUGREPORT);
+            }
+
+          /* Clean up. */
+          if(freemem) free(freemem);
+        }
+      else
+        error(EXIT_SUCCESS, 0, "WARNING: %s: line with the '%s' "
+              "keyword didn't have a value. Hence the amount of "
+              "available RAM couldn't be determined. If a large "
+              "volume of data is provided, the program may crash. "
+              "Please contact us at '%s' to fix the problem",
+              file, keyname, PACKAGE_BUGREPORT);
+
+      /* Clean up. */
+      free(linecp);
+    }
+
+  /* Return the value. */
+  return out;
+}
+
+
+
+
+
 /* On the Linux kernel, due to "overcommitting" (which is activated by
    default), malloc will not return NULL when we allocate more memory than
    the physically available memory. It is possible to disable overcommiting
    with root permissions, but I have not been able to find any way to do
    this as a normal user. So the only way is to look into the
-   '/proc/meminfo' file (constantly filled by the Linux kernel) and read
+   '/proc/meminfo' file (constantly updated by the Linux kernel) and read
    the available memory from that.
 
    Note that this overcommiting apparently only occurs on Linux. From what
    I have read, other kernels are much more strict and 'malloc' will indeed
    return NULL if there isn't any physical RAM to support it. So if the
    '/proc/meminfo' doesn't exist, we can assume that 'malloc' works as
-   expected, until its inverse is proven. */
+   expected, until its inverse is proven.
+
+   Most GNU/Linux distributions have 'MemAvailable', which is "an estimate
+   of how much memory is available for starting new applications, without
+   swapping" (see [1]). It is therefore the main parameter to check for
+   this purpose. However, on some systems (for example Ubuntu, installed
+   within Windows using Windows Subsystem for Linux), 'MemAvailable' is not
+   defined! So we will have to use 'MemFree' (which is "The amount of
+   physical RAM, in kibibytes, left unused by the system"; from [1]).
+
+   From that description I feel their difference is that the Linux kernel
+   allocates some memory for programs that it starts from the RAM
+   ('MemAvailable'). However, 'MemFree' is the amount of free RAM
+   (independent of the Linux kernel; not including the "Available" space
+   that Linux has allocated for programs it starts). Since Windows
+   Subsystem for Linux doesn't use the Linux kernel, it doesn't have
+   'MemAvailable', and only 'MemFree'.
+
+   [1] https://lynxbee.com/understanding-proc-meminfo-analyzing-linux-memory-utilization
+*/
 size_t
 gal_checkset_ram_available(int quietmmap)
 {
   FILE *file;
-  int keyfound=0;
-  size_t *freemem=NULL;
-  size_t linelen=80, out=GAL_BLANK_SIZE_T;
-  char *token, *line, *linecp, *saveptr, delimiters[] = " ";
-  char *meminfo="/proc/meminfo", *keyname="MemAvailable", *units="kB";
+  size_t linelen=80;
+  char *meminfo="/proc/meminfo", *line;
+  char *mastr="MemAvailable", *mfstr="MemFree";
+  size_t mf=GAL_BLANK_SIZE_T, ma=GAL_BLANK_SIZE_T;
+  size_t malen=strlen(mastr), mflen=strlen(mfstr);
 
   /* If /proc/meminfo exists, read it. Otherwise, don't bother doing
      anything. */
@@ -119,97 +223,35 @@ gal_checkset_ram_available(int quietmmap)
         error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for line",
               __func__, linelen*sizeof *line);
 
-      /* Read it line-by-line until you find 'MemAvailable'.  */
-      while( getline(&line, &linelen, file) != -1 )
-        if( !strncmp(line, keyname, 12) )
-          {
-            /* Necessary for final check: */
-            keyfound=1;
-
-            /* We need to work on a copied line to avoid messing up the
-               contents of the actual line. */
-            gal_checkset_allocate_copy(line, &linecp);
-
-            /* The first token (which we don't need). */
-            token=strtok_r(linecp, delimiters, &saveptr);
-
-            /* The second token (which is the actual number we want). */
-            token=strtok_r(NULL, delimiters, &saveptr);
-            if(token)
-              {
-                /* Read the token as a number. */
-                if( gal_type_from_string((void **)(&freemem), token,
-                                         GAL_TYPE_SIZE_T) )
-                  error(EXIT_SUCCESS, 0, "WARNING: %s: value of '%s' "
-                        "keyword couldn't be read as an integer. Hence "
-                        "the amount of available RAM couldn't be "
-                        "determined. If a large volume of data is "
-                        "provided, the program may crash. Please contact "
-                        "us at '%s' to fix the problem",
-                        meminfo, keyname, PACKAGE_BUGREPORT);
-                else
-                  {
-                    /* The third token should be the units ('kB'). If it
-                       isn't, there should be an error because we currently
-                       assume kilobytes. */
-                    token=strtok_r(NULL, delimiters, &saveptr);
-                    if(token)
-                      {
-                        /* The units should be 'kB' (for kilobytes). */
-                        if( !strncmp(token, units, 2) )
-                          out=freemem[0]*1000;
-                        else
-                          error(EXIT_SUCCESS, 0, "WARNING: %s: the units of "
-                                "the value of '%s' keyword is (usually 'kB') "
-                                "isn't recognized. Hence the amount of "
-                                "available RAM couldn't be determined. If a "
-                                "large volume of data is provided, the "
-                                "program may crash. Please contact us at "
-                                "'%s' to fix the problem", meminfo, keyname,
-                                PACKAGE_BUGREPORT);
-                      }
-                    else
-                      error(EXIT_SUCCESS, 0, "WARNING: %s: the units of the "
-                            "value of '%s' keyword (usually 'kB') couldn't "
-                            "be read as an integer. Hence the amount of "
-                            "available RAM couldn't be determined. If a "
-                            "large volume of data is provided, the program "
-                            "may crash. Please contact us at '%s' to fix "
-                            "the problem", meminfo, keyname, PACKAGE_BUGREPORT);
-                  }
-
-                /* Clean up. */
-                if(freemem) free(freemem);
-              }
-            else
-              error(EXIT_SUCCESS, 0, "WARNING: %s: line with the '%s' "
-                    "keyword didn't have a value. Hence the amount of "
-                    "available RAM couldn't be determined. If a large "
-                    "volume of data is provided, the program may crash. "
-                    "Please contact us at '%s' to fix the problem",
-                    meminfo, keyname, PACKAGE_BUGREPORT);
-
-            /* Clean up. */
-            free(linecp);
-          }
+      /* Read it line-by-line until you find the key. */
+      while( getline(&line, &linelen, file) != -1
+             && (ma == GAL_BLANK_SIZE_T || mf == GAL_BLANK_SIZE_T) )
+        {
+          if( ma == GAL_BLANK_SIZE_T )
+            ma=checkset_meminfo_line(line, mastr, malen, meminfo);
+          if( mf == GAL_BLANK_SIZE_T )
+            mf=checkset_meminfo_line(line, mfstr, mflen, meminfo);
+        }
 
       /* The file existed but a keyname couldn't be found. In this case we
          should inform the user to be aware that we can't automatically
          determine the available memory. */
-      if(keyfound==0 && quietmmap==0)
-        error(EXIT_SUCCESS, 0, "WARNING: %s: didn't contain a '%s' keyword "
-              "hence the amount of available RAM couldn't be determined. "
-              "If a large volume of data is provided, the program may "
-              "crash. Please contact us at '%s' to fix the problem",
-              meminfo, keyname, PACKAGE_BUGREPORT);
+      if(ma==GAL_BLANK_SIZE_T && mf==GAL_BLANK_SIZE_T && quietmmap==0)
+        error(EXIT_SUCCESS, 0, "WARNING: %s: didn't contain a '%s' "
+              "or '%s' keywords hence the amount of available RAM "
+              "couldn't be determined. If a large volume of data is "
+              "provided, the program may crash. Please contact us at "
+              "'%s' to fix the problem", meminfo, mastr, mfstr,
+              PACKAGE_BUGREPORT);
 
       /* Close the opened file and free the line. */
       free(line);
       fclose(file);
     }
 
-  /* Return the final value. */
-  return out;
+  /* If 'MemAvailable' was found, return it. Otherwise, return the value of
+     'MemFree'. */
+  return ma==GAL_BLANK_SIZE_T ? mf : ma;
 }
 
 
@@ -310,6 +352,18 @@ gal_checkset_string_has_space(char *in)
 
 
 
+void
+gal_checkset_string_case_change(char *in, int toupper1_tolower0)
+{
+  if(toupper1_tolower0) do *in=toupper(*in); while(*in++!='\0');
+  else                  do *in=tolower(*in); while(*in++!='\0');
+
+}
+
+
+
+
+
 char *
 gal_checkset_malloc_cat(char *inname, char *toappend)
 {
@@ -334,7 +388,7 @@ gal_checkset_malloc_cat(char *inname, char *toappend)
 
 
 /* Copy the input string to the output (and also allocate the
-   output. */
+   output). */
 void
 gal_checkset_allocate_copy(const char *arg, char **copy)
 {
@@ -404,6 +458,41 @@ gal_checkset_dataset_name(char *filename, char *hdu)
 
 
 
+/* Remove the given prefix from the name, then compare. If the prefix is
+   NULL, then this becomes a basic string comparison '!strcmp(a,b)'. */
+int
+gal_checkset_noprefix_isequal(char *string, char *prefix,
+                              const char *tocompare)
+{
+  size_t plen;
+
+  /* If the input string is empty, then this option has nothing to do
+     (should return false): like NaN in math, two NULLs are not equal in
+     this scenario. */
+  if(string==NULL) return 0;
+  if(tocompare==NULL) return 0;
+
+  /* Do the comparison. */
+  if(prefix)
+    {
+      /* Check if 'string' starts with 'prefix'. When it does start with
+         the prefix, then do the comparison for the string after the
+         prefix. */
+      plen=strlen(prefix);
+      if( !strncmp(string, prefix, plen) )
+        return !strcmp(string+plen, tocompare);
+    }
+
+  /* If a prefix wasn't given, OR the string didn't start with the given
+     prefix, then ignore the prefix and simply compare the full input
+     string and the 'tocompare' string. */
+  return !strcmp(string, tocompare);
+}
+
+
+
+
+
 
 
 
@@ -453,14 +542,14 @@ gal_checkset_dir_part(char *filename)
 
 /* Given a file name, keep the non-directory part. Note that if there
    is no forward slash in the input name, the full input name is
-   considered to be the notdir output.*/
+   considered to be the notdir output. */
 char *
 gal_checkset_not_dir_part(char *filename)
 {
   size_t i, l;
   char *out, *tmp=filename;
 
-  /* Find the first '/' to identify the directory */
+  /* Find the first '/' to identify the directory. */
   l=strlen(filename);
   for(i=l;i!=0;--i)
     if(filename[i]=='/')
@@ -520,7 +609,7 @@ gal_checkset_suffix_separate(char *name, char **outsuffix)
 /* Given a reference filename, add a format of AAAAA-XXXXXX.CCCC where
    'AAAAA' is the base name of the 'reference' argument, 'XXXXX' is a
    random/unique sequence of characters, and 'YYYYY' is the string given to
-   'suffix'. If 'suffix' is NULL, the suffix of 'reference' will be used.*/
+   'suffix'. If 'suffix' is NULL, the suffix of 'reference' will be used. */
 char *
 gal_checkset_make_unique_suffix(char *reference, char *suffix)
 {
@@ -666,7 +755,8 @@ gal_checkset_writable_notexist(char *filename)
    won't be deleted, but the program will abort with an error, informing
    the user that the output can't be made. */
 void
-gal_checkset_writable_remove(char *filename, int keep, int dontdelete)
+gal_checkset_writable_remove(char *filename, char *basename, int keep,
+                             int dontdelete)
 {
   char *dir;
   FILE *tmpfile;
@@ -676,6 +766,16 @@ gal_checkset_writable_remove(char *filename, int keep, int dontdelete)
   if(filename==NULL)
     return;
 
+  /* If a base-name was given, then this name may be related to the input's
+     name. In that case, we should check if they are the same string. */
+  if(basename)
+    if( !strcmp(filename, basename) )
+      error(EXIT_FAILURE, 0, "the output filename name ('%s') is the same "
+            "as the input's file name! The output file is re-written if "
+            "it already exists. Therefore, when the input and output file "
+            "names are the same there is a problem! Please select a "
+            "different output name", filename);
+
   /* We want to make sure that we can open and write to this file. But
      the user might have asked to not delete the file, so the
      contents should not be changed. Therefore we have to open it with
@@ -732,7 +832,8 @@ gal_checkset_writable_remove(char *filename, int keep, int dontdelete)
    a directory) it will return 0. Finally, if it exists but cannot be
    deleted, report an error and abort. */
 int
-gal_checkset_dir_0_file_1(char *name, int dontdelete)
+gal_checkset_dir_0_file_1(struct gal_options_common_params *cp, char *name,
+                          char *basename)
 {
   FILE *tmpfile;
   struct stat nameinfo;
@@ -761,20 +862,20 @@ gal_checkset_dir_0_file_1(char *name, int dontdelete)
             }
           else
             error(EXIT_FAILURE, errno, "%s", name);
-          return 1;                    /* It is a file name, GOOD */
+          return 1;                    /* It is a file name, GOOD. */
         }
-      else                             /* Some strange condition, ABORT */
+      else                             /* Some strange condition, ABORT. */
         error(EXIT_FAILURE, errno, "%s", name);
     }
 
-  if(S_ISDIR(nameinfo.st_mode))        /* It is a directory, BAD */
+  if(S_ISDIR(nameinfo.st_mode))        /* It is a directory, BAD. */
     return 0;
   else if (S_ISREG(nameinfo.st_mode))  /* It is a file, GOOD. */
     {
-      gal_checkset_writable_remove(name, 0, dontdelete);
+      gal_checkset_writable_remove(name, basename, cp->keep, cp->dontdelete);
       return 1;
     }
-  else                                 /* Not a file or a dir, ABORT */
+  else                                 /* Not a file or a dir, ABORT. */
     error(EXIT_FAILURE, 0, "%s not a file or a directory", name);
 
   error(EXIT_FAILURE, 0, "%s: a bug! The process should not reach the end "
@@ -798,14 +899,14 @@ gal_checkset_automatic_output(struct gal_options_common_params *cp,
   size_t i, l, offset=0;
 
   /* Merge the contents of the input name and suffix name (while also
-     allocating the necessary space).*/
+     allocating the necessary space). */
   out=gal_checkset_malloc_cat(inname, suffix);
 
   /* If there is actually a suffix, replace it with the (possibly) existing
      suffix. */
   if(suffix)
     {
-      /* Start from the end of the input array*/
+      /* Start from the end of the input array. */
       l=strlen(inname);
       for(i=l-1;i!=0;--i)
         {
@@ -831,7 +932,7 @@ gal_checkset_automatic_output(struct gal_options_common_params *cp,
           /* The input file names can be compressed names (for example
              '.fits.gz'). Currently the only compressed formats
              (decompressed within CFITSIO) are listed in
-             'gal_fits_name_is_fits' and 'gal_fits_suffix_is_fits'.*/
+             'gal_fits_name_is_fits' and 'gal_fits_suffix_is_fits'. */
           else if(out[i]=='.' && !( ( out[i+1]=='g' && out[i+2]=='z' )
                                     || (out[i+1]=='f' && out[i+2]=='z' )
                                     || out[i+1]=='Z' ) )
@@ -857,7 +958,7 @@ gal_checkset_automatic_output(struct gal_options_common_params *cp,
     }
 
   /* Remove the created filename if it already exits. */
-  gal_checkset_writable_remove(out, cp->keep, cp->dontdelete);
+  gal_checkset_writable_remove(out, inname, cp->keep, cp->dontdelete);
 
   /* Return the resulting filename. */
   return out;
@@ -910,9 +1011,10 @@ checkset_directory_writable(char *dirname)
 
 
 
+
 /* Check if dirname is actually a real directory and that we can
    actually write inside of it. To insure all conditions an actual
-   file will be made */
+   file will be made. */
 void
 gal_checkset_check_dir_write_add_slash(char **dirname)
 {
@@ -984,3 +1086,40 @@ gal_checkset_mkdir(char *dirname)
   /* Return the final 'errno'. */
   return errnum;
 }
+
+
+
+
+
+/* Timestamp the file name (or any other string) with a YYYY-MM-DD prefix
+   and a HH-MM-SS suffix. This is useful in making temporary debugging
+   files during a program. The 'filename' is wrapped between the dates so
+   it is easy to sort files by name and easily seperate files by their date
+   and name, where the same files will be stacked regardless of their
+   creation hour. */
+char *
+gal_checkset_timestamp(char *filename, char *newext)
+{
+  time_t t;
+  struct tm *now;
+  char suffix[20], prefix[20], *top, *rawname, *oldext;
+
+  /* Parse time */
+  time( &t );
+  now=localtime( &t );
+  strftime(prefix, 20, "%Y-%m-%d_", now);
+  strftime(suffix, 20, "_%H-%M-%S", now);
+
+  /* If parsing a filename, take care of its parent directory and
+     update the extension. */
+  top=gal_checkset_dir_part(filename);
+  rawname=gal_checkset_not_dir_part(filename);
+  filename=gal_checkset_suffix_separate(rawname, &oldext);
+  filename=gal_checkset_malloc_cat(filename, suffix);
+  filename=gal_checkset_malloc_cat(prefix, filename);
+  filename=gal_checkset_malloc_cat(top, filename);
+  if(newext) filename=gal_checkset_malloc_cat(filename, newext);
+  else       filename=gal_checkset_malloc_cat(filename, oldext);
+
+  return filename;
+}
