@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -28,6 +28,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <gsl/gsl_errno.h>
 #include <gsl/gsl_const_mksa.h>
 #include <gsl/gsl_integration.h>
 
@@ -85,7 +86,7 @@ struct cosmology_integrand_t
     sum should not exceed 1. */
 static void
 cosmology_density_check(double o_lambda_0, double o_matter_0,
-                        double o_radiation_0)
+                        double o_radiation_0, int quiet)
 {
   double sum = o_lambda_0 + o_matter_0 + o_radiation_0;
 
@@ -110,13 +111,12 @@ cosmology_density_check(double o_lambda_0, double o_matter_0,
 
   /* Check if the density fractions add up to 1 (within floating point
       error). */
-  if( sum > (1+1e-8) || sum < (1-1e-8) )
-    error(EXIT_FAILURE, 0, "sum of fractional densities is not 1, "
-          "but %.8f. The cosmological constant ('olambda'), matter "
-          "('omatter') and radiation ('oradiation') densities are given "
-          "as %.8f, %.8f, %.8f", sum, o_lambda_0, o_matter_0,
-          o_radiation_0);
-
+  if( (sum > (1+1e-8) || sum < (1-1e-8)) && quiet==0 )
+    error(EXIT_SUCCESS, 0, "WARNING: non-flat FLRW model: the curvature "
+          "density parameter is %.8f; therefore angular diameter based "
+          "distances like will be wrong in Gnuastro's current "
+          "implementation; see https://savannah.gnu.org/bugs/?65195. "
+          "This warning message can be disabled with '--quiet'", 1.0-sum);
 }
 
 
@@ -219,9 +219,8 @@ cosmology_integrand_comoving_volume(double z, void *params)
    fractional densities must add up to 1. */
 double
 gal_cosmology_age(double z, double H0, double o_lambda_0, double o_matter_0,
-                  double o_radiation_0)
+                  double o_radiation_0, int quiet)
 {
-  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0);
   gsl_function F;
   double result, error;
   double o_curv_0 = 1.0 - ( o_lambda_0 + o_matter_0 + o_radiation_0 );
@@ -230,6 +229,10 @@ gal_cosmology_age(double z, double H0, double o_lambda_0, double o_matter_0,
   struct cosmology_integrand_t p={o_lambda_0, o_curv_0, o_matter_0,
                                   o_radiation_0};
 
+  /* Basic sanity check (no problem with the usage of these variables in
+     the definitions above; they are just  */
+  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0, quiet);
+
   /* Set the GSL function parameters. */
   F.params=&p;
   F.function=&cosmology_integrand_age;
@@ -246,24 +249,51 @@ gal_cosmology_age(double z, double H0, double o_lambda_0, double o_matter_0,
 /* Proper distance to z (Mpc). */
 double
 gal_cosmology_proper_distance(double z, double H0, double o_lambda_0,
-                              double o_matter_0, double o_radiation_0)
+                              double o_matter_0, double o_radiation_0,
+                              int quiet)
 {
-  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0);
+  int status;
   size_t neval;
   gsl_function F;
-  double result, error, c=GSL_CONST_MKSA_SPEED_OF_LIGHT;
+  gsl_integration_workspace *w;
+  gsl_error_handler_t *gslerrhandler;
+  double result, gslerr, c=GSL_CONST_MKSA_SPEED_OF_LIGHT;
   double o_curv_0 = 1.0 - ( o_lambda_0 + o_matter_0 + o_radiation_0 );
   double H0s=H0/1000/GSL_CONST_MKSA_PARSEC;  /* H0 in units of seconds. */
   struct cosmology_integrand_t p={o_lambda_0, o_curv_0, o_matter_0,
                                   o_radiation_0};
 
+  /* Basic sanity check (no problem with the usage of these variables in
+     the definitions above; they are just  */
+  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0, quiet);
+
   /* Set the GSL function parameters */
   F.params=&p;
   F.function=&cosmology_integrand_proper_dist;
 
-  /* Do the integration. */
-  gsl_integration_qng(&F, 0.0f, z, GSLIEPSABS, GSLIEPSREL, &result,
-                      &error, &neval);
+  /* Temporarily switch off error handling */
+  gslerrhandler=gsl_set_error_handler_off();
+
+  /* Do the integration (first with the fast GSL function). */
+  status=gsl_integration_qng(&F, 0.0f, z, GSLIEPSABS, GSLIEPSREL, &result,
+                             &gslerr, &neval);
+
+  /* If the first integration failed, try a slower, but more robust one. */
+  if (status!=GSL_SUCCESS)
+    {
+      w=gsl_integration_workspace_alloc(GSLILIMIT);
+      status=gsl_integration_qag(&F, 0.0f, z, GSLIEPSABS, GSLIEPSREL,
+                                 GSLILIMIT, GSL_INTEG_GAUSS21, w,
+                                 &result, &gslerr);
+      gsl_integration_workspace_free(w);
+      if (status!=GSL_SUCCESS)
+        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to "
+              "fix the problem. The status of the second integration is "
+              "%i.",  __func__, PACKAGE_BUGREPORT, status);
+  }
+
+  /* Reactivate "normal" error handling */
+  gslerrhandler=gsl_set_error_handler(gslerrhandler);
 
   /* Return the result. */
   return result * c / H0s / (1e6 * GSL_CONST_MKSA_PARSEC);
@@ -276,12 +306,15 @@ gal_cosmology_proper_distance(double z, double H0, double o_lambda_0,
 /* Comoving volume over 4pi stradian to z (Mpc^3). */
 double
 gal_cosmology_comoving_volume(double z, double H0, double o_lambda_0,
-                              double o_matter_0, double o_radiation_0)
+                              double o_matter_0, double o_radiation_0,
+                              int quiet)
 {
-  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0);
+  int status;
   size_t neval;
   gsl_function F;
-  double result, error;
+  double result, gslerr;
+  gsl_integration_workspace *w;
+  gsl_error_handler_t *gslerrhandler;
   double c=GSL_CONST_MKSA_SPEED_OF_LIGHT;
   double H0s=H0/1000/GSL_CONST_MKSA_PARSEC;     /* H0 in units of seconds. */
   double cH = c / H0s / (1e6 * GSL_CONST_MKSA_PARSEC);
@@ -289,13 +322,37 @@ gal_cosmology_comoving_volume(double z, double H0, double o_lambda_0,
   struct cosmology_integrand_t p={o_lambda_0, o_curv_0, o_matter_0,
                                   o_radiation_0};
 
+  /* Basic sanity check (no problem with the usage of these variables in
+     the definitions above; they are just  */
+  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0, quiet);
+
   /* Set the GSL function parameters */
   F.params=&p;
   F.function=&cosmology_integrand_comoving_volume;
 
+  /* temporarily switch off error handling */
+  gslerrhandler=gsl_set_error_handler_off();
+
   /* Do the integration. */
-  gsl_integration_qng(&F, 0.0f, z, GSLIEPSABS, GSLIEPSREL,
-                      &result, &error, &neval);
+  status=gsl_integration_qng(&F, 0.0f, z, GSLIEPSABS, GSLIEPSREL,
+                      &result, &gslerr, &neval);
+
+  /* If the first integration failed, try a slower, but more robust one. */
+  if (status!=GSL_SUCCESS)
+    {
+      w=gsl_integration_workspace_alloc(GSLILIMIT);
+     status=gsl_integration_qag(&F, 0.0f, z, GSLIEPSABS, GSLIEPSREL,
+                                GSLILIMIT, GSL_INTEG_GAUSS21, w,
+                                &result, &gslerr);
+     gsl_integration_workspace_free(w);
+     if (status!=GSL_SUCCESS)
+       error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to "
+             "fix the problem. The status of the second integration is "
+             "%i.",  __func__, PACKAGE_BUGREPORT, status);
+    }
+
+  /* Reactivate "normal" error handling */
+  gslerrhandler=gsl_set_error_handler(gslerrhandler);
 
   /* Return the result. */
   return result * 4 * M_PI * cH*cH*cH;
@@ -308,15 +365,19 @@ gal_cosmology_comoving_volume(double z, double H0, double o_lambda_0,
 /* Critical density at redshift z in units of g/cm^3. */
 double
 gal_cosmology_critical_density(double z, double H0, double o_lambda_0,
-                               double o_matter_0, double o_radiation_0)
+                               double o_matter_0, double o_radiation_0,
+                               int quiet)
 {
-  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0);
   double H;
   double H0s=H0/1000/GSL_CONST_MKSA_PARSEC;     /* H0 in units of seconds. */
   double o_curv_0 = 1.0 - ( o_lambda_0 + o_matter_0 + o_radiation_0 );
   struct cosmology_integrand_t p={o_lambda_0, o_curv_0, o_matter_0,
                                   o_radiation_0};
 
+  /* Basic sanity check (no problem with the usage of these variables in
+     the definitions above; they are just  */
+  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0, quiet);
+
   /* Set the place holder, then return the result. */
   H = H0s * cosmology_integrand_Ez(z, &p);
   return 3*H*H/(8*M_PI*GSL_CONST_MKSA_GRAVITATIONAL_CONSTANT)/1000;
@@ -329,11 +390,16 @@ gal_cosmology_critical_density(double z, double H0, double o_lambda_0,
 /* Angular diameter distance to z (Mpc). */
 double
 gal_cosmology_angular_distance(double z, double H0, double o_lambda_0,
-                               double o_matter_0, double o_radiation_0)
+                               double o_matter_0, double o_radiation_0,
+                               int quiet)
 {
-  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0);
+  /* Sanity checks. */
+  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0, quiet);
+
+  /* Note that here 'quiet' is activated because we already do the
+     density check once. */
   return gal_cosmology_proper_distance(z, H0, o_lambda_0, o_matter_0,
-                                       o_radiation_0) / (1+z);
+                                       o_radiation_0, 1) / (1+z);
 }
 
 
@@ -343,11 +409,16 @@ gal_cosmology_angular_distance(double z, double H0, double o_lambda_0,
 /* Luminosity distance to z (Mpc). */
 double
 gal_cosmology_luminosity_distance(double z, double H0, double o_lambda_0,
-                                  double o_matter_0, double o_radiation_0)
+                                  double o_matter_0, double o_radiation_0,
+                                  int quiet)
 {
-  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0);
+  /* Sanity checks. */
+  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0, quiet);
+
+  /* Note that here 'quiet' is activated because we already do the density
+     check once. */
   return gal_cosmology_proper_distance(z, H0, o_lambda_0, o_matter_0,
-                                       o_radiation_0) * (1+z);
+                                       o_radiation_0, 1) * (1+z);
 }
 
 
@@ -357,11 +428,16 @@ gal_cosmology_luminosity_distance(double z, double H0, double o_lambda_0,
 /* Distance modulus at z (no units). */
 double
 gal_cosmology_distance_modulus(double z, double H0, double o_lambda_0,
-                               double o_matter_0, double o_radiation_0)
+                               double o_matter_0, double o_radiation_0,
+                               int quiet)
 {
-  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0);
+  /* Sanity checks. */
+  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0, quiet);
+
+  /* Note that here 'quiet' is activated because we already do the density
+     check once. */
   double ld=gal_cosmology_luminosity_distance(z, H0, o_lambda_0, o_matter_0,
-                                              o_radiation_0);
+                                              o_radiation_0, 1);
   return 5*(log10(ld*1000000)-1);
 }
 
@@ -372,11 +448,16 @@ gal_cosmology_distance_modulus(double z, double H0, double o_lambda_0,
 /* Convert apparent to absolute magnitude. */
 double
 gal_cosmology_to_absolute_mag(double z, double H0, double o_lambda_0,
-                              double o_matter_0, double o_radiation_0)
+                              double o_matter_0, double o_radiation_0,
+                              int quiet)
 {
-  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0);
+  /* Sanity checks. */
+  cosmology_density_check(o_lambda_0, o_matter_0, o_radiation_0, quiet);
+
+  /* Note that here 'quiet' is activated because we already do the density
+     check once. */
   double dm=gal_cosmology_distance_modulus(z, H0, o_lambda_0, o_matter_0,
-                                           o_radiation_0);
+                                           o_radiation_0, 1);
   return dm-2.5*log10(1.0+z);
 }
 
