@@ -6,7 +6,7 @@ Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
      Pedram Ashofteh Ardakani <pedramardakani@pm.me>
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -82,6 +82,9 @@ gal_checkset_known_types(char *optarg, int *type, char *filename,
 int
 gal_checkset_string_has_space(char *in);
 
+void
+gal_checkset_string_case_change(char *in, int toupper1_tolower0);
+
 char *
 gal_checkset_malloc_cat(char *inname, char *toappend);
 
@@ -97,6 +100,9 @@ gal_checkset_dataset_name(char *filename, char *hdu);
 char *
 gal_checkset_timestamp(char *filename, char *newext);
 
+int
+gal_checkset_noprefix_isequal(char *string, char *prefix,
+                              const char *tocompare);
 
 
 /**************************************************************/
