@@ -6,7 +6,7 @@ IMPORTANT: This header must only be included the programs, not libraries.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2022 Free Software Foundation, Inc.
+Copyright (C) 2017-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -315,6 +315,58 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "outfitsnodate",
+      GAL_OPTIONS_KEY_OUTFITSNODATE,
+      0,
+      0,
+      "No 'DATE' in 0-th HDU of output FITS.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &cp->outfitsnodate,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "outfitsnoversions",
+      GAL_OPTIONS_KEY_OUTFITSNOVERSIONS,
+      0,
+      0,
+      "No versions in 0-th HDU of output FITS.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &cp->outfitsnoversions,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "outfitsnocommit",
+      GAL_OPTIONS_KEY_OUTFITSNOCOMMIT,
+      0,
+      0,
+      "No Git commit in 0-th HDU of output FITS.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &cp->outfitsnocommit,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "outfitsnoconfig",
+      GAL_OPTIONS_KEY_OUTFITSNOCONFIG,
+      0,
+      0,
+      "No metadata in 0-th HDU of output FITS.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &cp->outfitsnoconfig,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
 
 
@@ -449,6 +501,19 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_NOT_SET,
       gal_options_check_config
     },
+    {
+      "config-prefix",
+      GAL_OPTIONS_KEY_CONFIGPREFIX,
+      "STR",
+      0,
+      "Custom prefix of option names config files.",
+      GAL_OPTIONS_GROUP_OPERATING_MODE,
+      &cp->configprefix,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+    },
     {
       "setdirconf",
       GAL_OPTIONS_KEY_SETDIRCONF,
