@@ -5,7 +5,8 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2021, Free Software Foundation, Inc.
+     Sachin Kumar Singh <sachinkumarsingh092@gmail.com>
+Copyright (C) 2017-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -45,11 +46,17 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 gal_data_t *
-gal_match_coordinates(gal_data_t *coord1, gal_data_t *coord2,
+gal_match_sort_based(gal_data_t *coord1, gal_data_t *coord2,
                       double *aperture, int sorted_by_first,
                       int inplace, size_t minmapsize, int quietmmap,
                       size_t *nummatched);
 
+gal_data_t *
+gal_match_kdtree(gal_data_t *coord1, gal_data_t *coord2,
+                 gal_data_t *coord1_kdtree, size_t kdtree_root,
+                 double *aperture, size_t numthreads, size_t minmapsize,
+                 int quietmmap, size_t *nummatched);
+
 
 
 
