@@ -6,7 +6,7 @@ Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
      Sachin Kumar Singh <sachinkumarsingh092@gmail.com>
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -65,7 +65,10 @@ int
 gal_polygon_is_convex(double *v, size_t n);
 
 double
-gal_polygon_area(double *v, size_t n);
+gal_polygon_area_flat(double *v, size_t n);
+
+double
+gal_polygon_area_sky(double *v, size_t n);
 
 int
 gal_polygon_is_inside(double *v, double *p, size_t n);
