@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -171,9 +171,9 @@ gal_table_comments_add_intro(gal_list_str_t **comments,
                              char *program_string, time_t *rawtime);
 
 void
-gal_table_write(gal_data_t *cols, struct gal_fits_list_key_t **keylist,
+gal_table_write(gal_data_t *cols, struct gal_fits_list_key_t *keylist,
                 gal_list_str_t *comments, int tableformat, char *filename,
-                char *extname, uint8_t colinfoinstdout);
+                char *extname, uint8_t colinfoinstdout, int freekeys);
 
 void
 gal_table_write_log(gal_data_t *logll, char *program_string,
