@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2022-2023 Free Software Foundation, Inc.
+Copyright (C) 2022-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -46,11 +46,16 @@ int plugin_is_GPL_compatible=1;
 
 
 
-/* Names of the separate functions */
+/* Names of the separate functions. */
 #define MAKEPLUGIN_FUNC_PREFIX "ast"
-static char *version_is_name=MAKEPLUGIN_FUNC_PREFIX"-version-is";
+/* Basic text functions */
+static char *text_to_upper=MAKEPLUGIN_FUNC_PREFIX"-text-to-upper";
+static char *text_to_lower=MAKEPLUGIN_FUNC_PREFIX"-text-to-lower";
 static char *text_contains_name=MAKEPLUGIN_FUNC_PREFIX"-text-contains";
 static char *text_not_contains_name=MAKEPLUGIN_FUNC_PREFIX"-text-not-contains";
+
+/* Gnuastro analysis functions */
+static char *version_is_name=MAKEPLUGIN_FUNC_PREFIX"-version-is";
 static char *fits_with_keyvalue_name=MAKEPLUGIN_FUNC_PREFIX"-fits-with-keyvalue";
 static char *fits_unique_keyvalues_name=MAKEPLUGIN_FUNC_PREFIX"-fits-unique-keyvalues";
 
@@ -76,7 +81,7 @@ makeplugin_version_is(const char *caller, unsigned int argc, char **argv)
   /* If the version matches, set the value of 'check'. */
   if( version && !strcmp(PACKAGE_VERSION, version) ) check=1;
 
-  /* Write the value into the 'out' pointer.*/
+  /* Write the value into the 'out' pointer. */
   if( asprintf(&out, "%d", check)<0 )
     error(EXIT_FAILURE, 0, "%s: couldn't allocate output string",
           __func__);
@@ -141,7 +146,7 @@ makeplugin_text_contains_base(char **argv, int has1_not0)
 /* Return any of the input strings that contain the given string. It takes
    two arguments:
       0. String to check.
-      1. List of text.*/
+      1. List of text. */
 static char *
 makeplugin_text_contains(const char *caller, unsigned int argc,
                          char **argv)
@@ -156,7 +161,7 @@ makeplugin_text_contains(const char *caller, unsigned int argc,
 /* Return any of the input strings that contain the given string. It takes
    two arguments:
       0. String to check.
-      1. List of text.*/
+      1. List of text. */
 static char *
 makeplugin_text_not_contains(const char *caller, unsigned int argc,
                              char **argv)
@@ -168,6 +173,35 @@ makeplugin_text_not_contains(const char *caller, unsigned int argc,
 
 
 
+/* Convert input string to upper-case. */
+static char *
+makeplugin_text_to_upper(const char *caller, unsigned int argc,
+                         char **argv)
+{
+  char *out;
+  gal_checkset_allocate_copy(argv[0], &out);
+  gal_checkset_string_case_change(out, 1);
+  return out;
+}
+
+
+
+
+
+/* Convert input string to upper-case. */
+static char *
+makeplugin_text_to_lower(const char *caller, unsigned int argc,
+                         char **argv)
+{
+  char *out;
+  gal_checkset_allocate_copy(argv[0], &out);
+  gal_checkset_string_case_change(out, 0);
+  return out;
+}
+
+
+
+
 
 
 
@@ -237,12 +271,12 @@ makeplugin_fits_with_keyvalue(const char *caller, unsigned int argc,
     return NULL;
 
   /* Extract the components in the arguments with possibly multiple
-     values and find the output files.*/
+     values and find the output files. */
   files=gal_list_str_extract(argv[3]);
   values=gal_list_str_extract(argv[1]);
   outlist=gal_fits_with_keyvalue(files, hdu, name, values, NULL);
 
-  /* Write the output string */
+  /* Write the output string. */
   out=gal_list_str_cat(outlist, ' ');
 
   /* Clean up and return. */
@@ -274,7 +308,7 @@ makeplugin_fits_unique_keyvalues(const char *caller, unsigned int argc,
     return NULL;
 
   /* Extract the components in the arguments with possibly multiple
-     values and find the output files.*/
+     values and find the output files. */
   files=gal_list_str_extract(argv[2]);
   outlist=gal_fits_unique_keyvalues(files, hdu, name, NULL);
 
@@ -291,14 +325,28 @@ makeplugin_fits_unique_keyvalues(const char *caller, unsigned int argc,
 
 
 
-/* Top-level function (that should have this name). */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/**********************************************************************/
+/********          High-level interface with Make             *********/
+/**********************************************************************/
 int
 libgnuastro_make_gmk_setup()
 {
-  /* Return 1 if Gnuastro has the requested version. */
-  gmk_add_function(version_is_name, makeplugin_version_is,
-                   1, 1, GMK_FUNC_DEFAULT);
-
+  /* ------------ Basic useful text functions ------------ */
   /* Return the input strings that contain the given string. */
   gmk_add_function(text_contains_name, makeplugin_text_contains,
                    2, 2, GMK_FUNC_DEFAULT);
@@ -307,13 +355,30 @@ libgnuastro_make_gmk_setup()
   gmk_add_function(text_not_contains_name, makeplugin_text_not_contains,
                    2, 2, GMK_FUNC_DEFAULT);
 
+  /* Convert input sting into upper-case. */
+  gmk_add_function(text_to_upper, makeplugin_text_to_upper,
+                   1, 1, GMK_FUNC_DEFAULT);
+
+  /* Convert input string to lower-case. */
+  gmk_add_function(text_to_lower, makeplugin_text_to_lower,
+                   1, 1, GMK_FUNC_DEFAULT);
+
+
+
+
+
+  /* ------------ Gnuastro related functions ------------ */
+  /* Return 1 if Gnuastro has the requested version. */
+  gmk_add_function(version_is_name, makeplugin_version_is,
+                   1, 1, GMK_FUNC_DEFAULT);
+
   /* Select files, were a certain keyword has a certain value. It takes
      four arguments. */
   gmk_add_function(fits_with_keyvalue_name, makeplugin_fits_with_keyvalue,
                    4, 4, GMK_FUNC_DEFAULT);
 
   /* Return the unique values given to a certain keyword in many FITS
-     files.*/
+     files. */
   gmk_add_function(fits_unique_keyvalues_name,
                    makeplugin_fits_unique_keyvalues,
                    3, 3, GMK_FUNC_DEFAULT);
