@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2024 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -58,7 +58,7 @@ gal_type_sizeof(uint8_t type)
       /* The parenthesis after sizeof is not a function, it is actually a
          type cast, so we have put a space between size of and the
          parenthesis to highlight this. In C, 'sizeof' is an operator, not
-         a function.*/
+         a function. */
     case GAL_TYPE_UINT8:     return sizeof (uint8_t);
     case GAL_TYPE_INT8:      return sizeof (int8_t);
     case GAL_TYPE_UINT16:    return sizeof (uint16_t);
@@ -237,9 +237,9 @@ gal_type_from_name(char *str)
 
 
 
-/* Put the minimum (or maximum for the 'gal_data_type_max') value for the
+/* Put the minimum (or maximum for the 'gal_type_max') value for the
    type in the space (that must already be allocated before the call to
-   this function) pointed to by in.  */
+   this function) pointed to by in. */
 void
 gal_type_min(uint8_t type, void *in)
 {
@@ -427,9 +427,17 @@ gal_type_to_string(void *ptr, uint8_t type, int quote_if_str_has_space)
     case GAL_TYPE_INT32:   TO_STRING( int32_t,  "%"PRId32 );  break;
     case GAL_TYPE_UINT64:  TO_STRING( uint64_t, "%"PRIu64 );  break;
     case GAL_TYPE_INT64:   TO_STRING( int64_t,  "%"PRId64 );  break;
-    case GAL_TYPE_FLOAT32: TO_STRING( float,    "%.6g"    );  break;
-    case GAL_TYPE_FLOAT64: TO_STRING( double,   "%.10g"   );  break;
 
+    /* We aren't using '%g' for floating points because it can remove
+       statisically significant digits in some scenarios and its result is
+       generally not easily predictable: can be fixed-point or exponential
+       depending on printed length! But the printed length of a number can
+       hide statisical significance. See the discussion in
+       'bin/table/asttable.conf' for the values used here. */
+    case GAL_TYPE_FLOAT32: TO_STRING( float,  "%.6e" ); break;
+    case GAL_TYPE_FLOAT64: TO_STRING( double, "%.14e"); break;
+
+    /* Unknown type! */
     default:
       error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
             __func__, type);
@@ -457,7 +465,7 @@ gal_type_to_string(void *ptr, uint8_t type, int quote_if_str_has_space)
    want the value to be stored, for example &(array[i]).
 
    If parsing was successful, it will return a 0. If there was a problem,
-   it will return 1.  */
+   it will return 1. */
 int
 gal_type_from_string(void **out, char *string, uint8_t type)
 {
@@ -561,8 +569,8 @@ void *
 gal_type_string_to_number(char *string, uint8_t *type)
 {
   long int l;
+  size_t digits;
   void *ptr, *out;
-  int fnz=-1, lnz=0;     /* 'F'irst (or 'L'ast) 'N'on-'Z'ero. */
   uint8_t forcedfloat=0;
   char *c, *tailptr, *cp;
 
@@ -656,38 +664,46 @@ gal_type_string_to_number(char *string, uint8_t *type)
     }
   else
     {
-      /* The maximum number of decimal digits to store in float or double
-         precision floating point are:
-
-         float:  23 mantissa bits + 1 hidden bit: log(224)÷log(10) = 7.22
-         double: 52 mantissa bits + 1 hidden bit: log(253)÷log(10) = 15.95
-
-         FLT_DIG (at least 6 in ISO C) keeps the number of digits (not zero
-         before or after) that can be represented by a single precision
-         floating point number. If there are more digits, then we should
-         store the value as a double precision.
-
-         Note that the number can have non-digit characters that we don't
-         want, like: '.', 'e', 'E', ','. */
+      /* Start counting the number of digits from the start of the string
+         (while ignoring any '0's at the start). */
+      digits=0;
       for(cp=string;*cp!='\0';++cp)
-        if(isdigit(*cp) && *cp!='0' && fnz==-1)
-          fnz=cp-string;
+        {
+          if(isdigit(*cp))
+            { if(!(digits==0 && *cp=='0')) ++digits; }
+          if(*cp=='e') break;
+        }
 
-      /* In the previous loop, we went to the end of the string, so 'cp'
-         now points to its '\0'. We just have to iterate backwards! */
+      /* In the previous loop, we went to the end of the string (or the 'e'
+         character in an exponential), so 'cp' now points to its end. We
+         just have to iterate backwards and stop when we hit a non-zero
+         character. */
       for(;cp!=string;--cp)
-        if(isdigit(*cp) && *cp!='0')
+        if(isdigit(*cp))
           {
-            lnz=cp-string;
-            break;
+            if(*cp=='0') --digits;
+            else break;
           }
 
       /* Calculate the number of decimal digits and decide if it the number
-         should be a float or a double. */
-      if( lnz-fnz > FLT_DIG || fabs(d)>FLT_MAX || fabs(d)<FLT_MIN )
+         should be a float or a double.
+
+         The maximum number of decimal digits to store 32-bit floating
+         point is 7.22 (see "Printing floating point numbers" section of
+         the book). We will round this to 7 to be on the safe side. If the
+         given number has more than 7 decimal digits, or is outside the
+         range of possible values for a 32-bit float, it should be saved as
+         a 64-bit float. */
+      if( digits > 7 || fabs(d)>FLT_MAX || fabs(d)<FLT_MIN )
         {      ptr=&d; *type=GAL_TYPE_FLOAT64; }
       else
         { f=d; ptr=&f; *type=GAL_TYPE_FLOAT32; }
+
+      /* For a check:
+      printf("%s:%s: %zu %s\n", __func__, string, digits,
+             gal_type_name(*type, 1));
+      printf("%s: GOOD\n", __func__); exit(0);
+      */
     }
 
   /* Allocate a one-element dataset, then copy the number into it. */
